//*************************************************************************
//
//                        SDCARD Function
//
//*************************************************************************


#include "SDCARD/sd_card.h"

#include "mmc_config.h"
#include "mmc.h"
#include "fat.h"
#include "file.h"
#include "lfo.h"
#include "filter.h"
#include "sequencer.h"
#include "oscillators.h"
#include "voice.h"
#include "gui.h"
#include "init_system.h"
#include "miniscope.h"
#include "patch.h"
#include "gui.h"



//*************************************************************************
// init SDCARD
//*************************************************************************
void init_SDCARD(void)
{
	// clear Display
	Paint(color_black); // draw black background
	
	// check SDCARD config status -------------------------
	if( FALSE == mmc_init() ){
		
		_delay_ms(200);
		
		// init Error
		fillrectangle(51,49,270,140,color_red);
		rectangle(53,51,86,214,color_white);
		print_string("SDCARD init fail !",80,70,6,color_white);
		print_string("Check SDCARD and",80,95,6,color_white);
		print_string("reset Power",80,110,6,color_white);
		while(1){}
	}
	
	//unsigned long int seek;
	
	if( FALSE == fat_loadFatData() ){	
	}
}

//*************************************************************************
// Load init preset
//*************************************************************************
void load_init_preset(void)
{
	// init Osc values ----------------------------------------------------
	Osc.wave[0]				= 0;
	Osc.wave[1]				= 0;
	Osc.shape[0]			= 1;
	Osc.shape[1]			= 0;
	Osc.prm_value[0]		= 0;
	Osc.prm_value[1]		= 0;
	Osc.op_value			= 0;
	Osc.rng_value[0]		= 36;
	Osc.rng_value[1]		= 36;
	Osc.tune_value			= 0;
	Osc.mix_value			= 0;
	Osc.mix					= 0;
	Osc.noise_value			= 0;
	Osc.wavebank[0]			= 0;
	Osc.wavebank[1]			= 0;
	Osc.wave_mode[0]		= 0;
	Osc.wave_mode[1]		= 2;
	Osc.sync_function[0]	= 1;
	Osc.sync_function[1]	= 1;
	Osc.loop_function[0]	= 0;
	Osc.loop_function[1]	= 0;
	Osc.loop_A[0]			= 0;
	Osc.loop_B[0]			= 0;
	Osc.loop_A[1]			= 0;
	Osc.loop_B[1]			= 0;
	Xram.sample_length[0]   = 0;
	Xram.sample_length[1]   = 0;
	Osc.track_value[0]		= 1;
	Osc.track_value[1]		= 1;
	
	// init Envelope values -----------------------------------------------
	Env.attack[0]			= 0;
	Env.attack[1]			= 0;
	Env.decay[0]			= 0;
	Env.decay[1]			= 0;
	Env.sustain[0]			= 127;
	Env.sustain[1]			= 127;
	Env.release[0]			= 60;
	Env.release[1]			= 51;
	
	// init Filter values -------------------------------------------------
	VCF.cutoff_value		= 127;
	VCF.reso_value			= 0;
	VCF.ktrack				= 0;
	VCF.filter_mode			= 0;
	VCF.ktrack_value		= 0;
	Gui.filtertyp           = 1;	// SMR4
	VCF.filter_mode         = 0;	// 6dB LP
	set_filter_switch();
	
	// init LFO values ----------------------------------------------------
	LFO.shape[0]			= 0;
	LFO.shape[1]			= 0;
	LFO.shape[2]			= 0;
	LFO.rate_volume[0]		= 0;
	LFO.rate_volume[1]		= 0;
	LFO.rate_volume[2]		= 0;
	LFO.rate[0]				= 1;
	LFO.rate[1]				= 1;
	LFO.rate[2]				= 1;
	LFO.sync_flag[0]		= 0;
	LFO.sync_flag[1]		= 0;
	LFO.sync_flag[2]		= 0;
	LFO.oneshoot_flag[0]	= 0;
	LFO.oneshoot_flag[1]	= 0;
	LFO.oneshoot_flag[2]	= 0;
	
	// init Mixer values --------------------------------------------------
	pan_mix					= 0;
	
	// init Fx delay values -----------------------------------------------
	Fx.Value				= 0;
	Fx.Pan					= 0;
	Fx.Time_value			= 0;
	Fx.Time					= 127;
	Fx.Feedback				= 0;
	
	// init Sequencer & Arpeggiater ---------------------------------------
	SEQ.seq_mode			= 1;
	SEQ.seq_bpm				= 120;
	SEQ.seq_div				= 0;
	SEQ.seq_dir				= 0;
	SEQ.seq_oct				= 0;
	
	// init Modulationsmatrix values --------------------------------------
	slot_array[0][0]		= 0;	// Slot 1: Env1 > Cut > + 32
	slot_array[0][1]		= 2;
	slot_array[0][2]		= 96;
	slot_array[1][0]		= 1;	// Slot 2: Env2 > VCA > + 64
	slot_array[1][1]		= 0;
	slot_array[1][2]		= 127;
	slot_array[2][0]		= 2;	// Slot 3: LFO1 > LFO1 > 0
	slot_array[2][1]		= 5;
	slot_array[2][2]		= 63;
	slot_array[3][0]		= 2;	// Slot 4: LFO1 > LFO1 > 0	
	slot_array[3][1]		= 5;
	slot_array[3][2]		= 63;
	slot_array[4][0]		= 2;	// Slot 5: LFO1 > LFO1 > 0
	slot_array[4][1]		= 5;
	slot_array[4][2]		= 63;
	slot_array[5][0]		= 2;	// Slot 6: LFO1 > LFO1 > 0
	slot_array[5][1]		= 5;
	slot_array[5][2]		= 63;
	slot_array[6][0]		= 2;	// Slot 7: LFO1 > LFO1 > 0
	slot_array[6][1]		= 5;
	slot_array[6][2]		= 63;
	slot_array[7][0]		= 2;	// Slot 8: LFO1 > LFO1 > 0
	slot_array[7][1]		= 5;
	slot_array[7][2]		= 63;
	slot_array[8][0]		= 2;	// Slot 9: LFO1 > LFO1 > 0
	slot_array[8][1]		= 5;
	slot_array[8][2]		= 63;
	slot_array[9][0]		= 2;	// Slot 10: LFO1 > LFO1 > 0
	slot_array[9][1]		= 5;
	slot_array[9][2]		= 63;
	slot_array[10][0]		= 2;	// Slot 11: LFO1 > LFO1 > 0
	slot_array[10][1]		= 5;
	slot_array[10][2]		= 63;
	slot_array[11][0]		= 2;	// Slot 12: Env1 > VCA > 0
	slot_array[11][1]		= 5;
	slot_array[11][2]		= 63;
	
	// Menu function
	Gui.file_not_found_flag		= 0;
}

//*************************************************************************
// Load PRESET-List
//*************************************************************************
void load_preset_list(uint8_t page_nr)
{
	set_presetname_folder();
	
	// open "Name.bin" in Preset Folder -----------------------------------
	if( MMC_FILE_OPENED == ffopen("Name.bin",'r') )
	{
		// clear window area
		fillrectangle(3,25,231,163, color_black);

		uint8_t row = 30;
		//uint8_t col = 0;
		//uint8_t temp_start_entry = (Menu.select_preset / 7) * 7; // calc start-entry for list
		uint8_t temp_start_entry = page_nr * 7;
		
		
		for (uint8_t i = temp_start_entry; i < (temp_start_entry + 7); i++)
		{
			if (i < 128)
			{
				// print list-no. -----------------------------------------
				draw_3digit_with(i,8,row, 6, color_white);
				
				// load wave symbol ---------------------------------------
				uint16_t seek_pointer = ((i * 32) + 16);
				ffseek(seek_pointer);
				uint8_t sym_temp = ffread();
				if (sym_temp == 1 || sym_temp == 2)
				{
					if (sym_temp == 1)
					{
						draw_sine_symbol(168,row+6,color_white);
					}
					if (sym_temp == 2)
					{
						draw_wavefile_symbol(168,row+6,color_yello);
					}
					
					// print File length ----------------------------------
					uint16_t file_len;
					file_len = ffread();
					file_len = file_len << 8;
					file_len = file_len + ffread();
					if (file_len > 0)
					{
						draw_4digit_without(file_len,183,row+3,2,color_white);
						print_string("KB",210,row+3,2,color_white);
					}
					
					// load preset name ---------------------------------------
					seek_pointer = (i * 32);
					ffseek(seek_pointer);		// file seek_pointer
					uint8_t string_len;
					for (string_len = 0; string_len < 13; string_len++)
					{
						soundname_x[string_len] = ffread();
					}
					// print preset name --------------------------------------
					print_string(soundname_x,52,row,6,color_white);
				}
			}
				
				// next row
				row = row +19;
		}		
	}
	else
	{
		print_info_box("Name.bin not found!",80,80,1);
	}
	
	ffclose();
}

//*************************************************************************
// set Preset.Data Folder for load and save function
//*************************************************************************
void set_preset_folder(void)
{
	// set PRBANK Folder --------------------------------------------------
	ffcd("");
	ffcd("PRESET");
	char folder_temp[9];
	strcpy(folder_temp,"PRBANK");
	
	char xx[2];
	sound_bank = select_presetbank;

	itoa(sound_bank,xx,10);
	if (sound_bank > 9)
	{
		strcat(folder_temp,xx);
	}
	else
	{
		strcat(folder_temp,"0");
		strcat(folder_temp,xx);
	}
	ffcd(folder_temp);
	
	// Set Preset Folder-Nr -----------------------------------------------
	char folder_temp1[10];
	strcpy(folder_temp1,"SOUND");
	
	char xx1[4];
	sound_bank = select_preset;

	itoa(sound_bank,xx1,10);
	if (sound_bank < 10)
	{
		strcat(folder_temp1,"00");
		strcat(folder_temp1,xx1);
	}
	else if(sound_bank < 100)
	{
		strcat(folder_temp1,"0");
		strcat(folder_temp1,xx1);
	}
	else
	{
		strcat(folder_temp1,xx1);
	}
	
	if (fat_loadFileDataFromDir(folder_temp1) == FALSE)
	{
		if (save_flag == 1 && load_flag == 0)
		{
			ffmkdir(folder_temp1);
		}
	}
	
	ffcd(folder_temp1);
	ffclose();
}

//*************************************************************************
// read parameter_value 
//*************************************************************************
uint8_t read_par_value(uint8_t group_id, uint8_t par_id)
{
	
	if (ffread() == group_id && ffread() == par_id)
	{
		return ffread();
	}
	else
	{
		ffread();
		return 0;
	}			
}

//*************************************************************************
// read parameter_value 32Bit
//*************************************************************************
uint32_t read_par_val32(uint8_t group_id, uint8_t par_id)
{
	uint8_t temp_byte;
	uint32_t temp_long;
	
	if (ffread() == group_id && ffread() == par_id)
	{
		temp_byte = ffread();
		temp_long = temp_long + temp_byte;
		temp_byte = ffread();
		temp_long = temp_long + (temp_byte << 8);
		temp_byte = ffread();
		temp_long = temp_long + (temp_byte << 16);
		temp_byte = ffread();
		temp_long = temp_long + (temp_byte << 24);
		return temp_long;
									
	}
	else
	{
		ffread();
		return 0;
	}
	
}


//*************************************************************************
// Load patch
//*************************************************************************
void load_preset_data(void)
{
	// stop 40KHz Audio Interuppt -----------------------------------------
	TCC0.INTCTRLA = 0b00000000;			// stop 40KHz Audio Interrupt
	
	// set preset.data folder ---------------------------------------------
	set_preset_folder();
	
	// load Preset Data ---------------------------------------------------
	if(MMC_FILE_OPENED == ffopen("Data.bin",'r'))
	{		
		// set list marker
		current_preset = select_preset;
		current_presetbank = select_presetbank;	
		
		// draw loading info and bar
		print_info_box("Loading Preset..",80,80,250);
		rectangle(94,110,3,128,color_gray);
		
		// Oscillator Parameter 0000-01FF ---------------------------------
		seek_pointer16 = 0x0000;
		ffseek(seek_pointer16);
		
		Osc.wave[0]			= read_par_value(0x00,0x00);
		Osc.wave[1]			= read_par_value(0x00,0x01);
		Osc.shape[0]		= read_par_value(0x00,0x02);
		Osc.shape[1]		= read_par_value(0x00,0x03);
		Osc.prm_value[0]	= read_par_value(0x00,0x04);
		Osc.prm_value[1]	= read_par_value(0x00,0x05);
		Osc.op_value		= read_par_value(0x00,0x06);
		Osc.rng_value[0]	= read_par_value(0x00,0x07);
		Osc.rng_value[1]	= read_par_value(0x00,0x08);
		Osc.tune_value		= read_par_value(0x00,0x09);
		Osc.mix_value		= read_par_value(0x00,0x0A);
		Osc.mix = Osc.mix_value << 1;
		Osc.noise_value		= read_par_value(0x00,0x0B);
		Osc.wavebank[0]		= read_par_value(0x00,0x0C);
		Osc.wavebank[1]		= read_par_value(0x00,0x0D);
		Osc.wave_mode[0]	= read_par_value(0x00,0x0E);
		Osc.wave_mode[1]	= read_par_value(0x00,0x0F);
		Osc.sync_function[0]= read_par_value(0x00,0x10);
		Osc.sync_function[1]= read_par_value(0x00,0x11);
		Osc.loop_function[0]= read_par_value(0x00,0x12);
		Osc.loop_function[1]= read_par_value(0x00,0x13);
		
		// 24Bit Loop addr
		uint8_t data[3];
		data[0]= read_par_value(0x00,0x14);
		data[1]= read_par_value(0x00,0x15);
		data[2]= read_par_value(0x00,0x16);
		Osc.loop_A[0] =  *((__uint24*) data);
		data[0]= read_par_value(0x00,0x17); // 0x47
		data[1]= read_par_value(0x00,0x18);	// 0x4a
		data[2]= read_par_value(0x00,0x19);	// 0x4d
		Osc.loop_B[0] =  *((__uint24*) data);
		data[0]= read_par_value(0x00,0x20);
		data[1]= read_par_value(0x00,0x21);
		data[2]= read_par_value(0x00,0x22);
		Osc.loop_A[1] =  *((__uint24*) data);
		data[0]= read_par_value(0x00,0x23);	// 0x59
		data[1]= read_par_value(0x00,0x24);	// 0x5c
		data[2]= read_par_value(0x00,0x25);	// 0x5f
		Osc.loop_B[1] =  *((__uint24*) data);
		
		if (Osc.loop_function[0] == 0)
		{
			Osc.loop_A[0] = 0;
			Osc.loop_B[0] = Xram.sample_length[0];
		}
		if (Osc.loop_function[1] == 0)
		{
			Osc.loop_A[1] = 0;
			Osc.loop_B[1] = Xram.sample_length[1];
		}
		if (Osc.wave_mode[0] == 0)
		{
			Osc.loop_A[0] = 0;
			Osc.loop_B[0] = 0;
		}
		if (Osc.wave_mode[1] == 0)
		{
			Osc.loop_A[1] = 0;
			Osc.loop_B[1] = 0;
		}
		
		// Osc1&2 Ktrack 
		Osc.track_value[0]		= read_par_value(0x00,0x26);
		Osc.track_value[1]		= read_par_value(0x00,0x27);
		
		// Envelope Paramter 0200-02FF ------------------------------------
		seek_pointer16 = 0x0200;
		ffseek(seek_pointer16);
		
		Env.attack[0]		= read_par_value(0x01,0x00);
		Env.attack[1]		= read_par_value(0x01,0x01);
		Env.decay[0]		= read_par_value(0x01,0x02);
		Env.decay[1]		= read_par_value(0x01,0x03);
		Env.sustain[0]		= read_par_value(0x01,0x04);
		Env.sustain[1]		= read_par_value(0x01,0x05);
		Env.release[0]		= read_par_value(0x01,0x06);
		Env.release[1]		= read_par_value(0x01,0x07);
		
		// Filter Parameter 0300-03FF -------------------------------------
		seek_pointer16 = 0x0300;
		ffseek(seek_pointer16);
		
		VCF.cutoff_value		= read_par_value(0x02,0x00);
		VCF.reso_value			= read_par_value(0x02,0x01);
		VCF.ktrack				= read_par_value(0x02,0x02);
		VCF.filter_mode			= read_par_value(0x02,0x04);
		VCF.ktrack_value		= read_par_value(0x02,0x05);
		
		set_filter_switch();
		
		// LFO Parameter 0400-04FF ----------------------------------------
		seek_pointer16 = 0x0400;
		ffseek(seek_pointer16);
		LFO.shape[0]			= read_par_value(0x03,0x00);
		LFO.shape[1]			= read_par_value(0x03,0x01);
		LFO.shape[2]			= read_par_value(0x03,0x02);
		LFO.rate_volume[0]		= read_par_value(0x03,0x03);
		LFO.rate_volume[1]		= read_par_value(0x03,0x04);
		LFO.rate_volume[2]		= read_par_value(0x03,0x05);
		LFO.rate[0] = (pgm_read_word (&(LFO_increment[LFO.rate_volume[0]])));
		LFO.rate[1] = (pgm_read_word (&(LFO_increment[LFO.rate_volume[1]])));
		LFO.rate[2] = (pgm_read_word (&(LFO_increment[LFO.rate_volume[2]])));
		LFO.sync_flag[0]		= read_par_value(0x03,0x0F);
		LFO.sync_flag[1]		= read_par_value(0x03,0x10);
		LFO.sync_flag[2]		= read_par_value(0x03,0x11);
		LFO.oneshoot_flag[0]	= read_par_value(0x03,0x12);
		LFO.oneshoot_flag[1]	= read_par_value(0x03,0x13);
		LFO.oneshoot_flag[2]	= read_par_value(0x03,0x14);
		
		// Mixer Parameter 0500-05FF --------------------------------------
		seek_pointer16 = 0x0500;
		ffseek(seek_pointer16);
		pan_mix				= read_par_value(0x04,0x00);
				
		// Fx parameter 0600-06FF -----------------------------------------
		seek_pointer16 = 0x0600;
		ffseek(seek_pointer16);
		Fx.Value		= read_par_value(0x05,0x00);
		Fx.Pan			= read_par_value(0x05,0x01);
		Fx.Time_value	= read_par_value(0x05,0x02);
		Fx.Time = 128 - Fx.Time_value;
		Fx.Feedback		= read_par_value(0x05,0x03);
		
		// Sequenzer & Arpeggiator ----------------------------------------
		seek_pointer16 = 0x680;
		ffseek(seek_pointer16);
		SEQ.seq_mode		= read_par_value(0x05,0x80);
		SEQ.seq_bpm			= read_par_value(0x05,0x81);
		SEQ.seq_div_value	= read_par_value(0x05,0x82);
		SEQ.seq_dir			= read_par_value(0x05,0x83);
		SEQ.seq_oct			= read_par_value(0x05,0x84);
		
		
		write_par_value(0x05,0x80,SEQ.seq_mode);
		write_par_value(0x05,0x81,SEQ.seq_bpm);
		write_par_value(0x05,0x82,SEQ.seq_div);
		write_par_value(0x05,0x83,SEQ.seq_dir);
		write_par_value(0x05,0x84,SEQ.seq_oct);
		
		// Modulation Matrix parameter 0700-08FF --------------------------
		seek_pointer16 = 0x0700;
		ffseek(seek_pointer16);
		slot_array[0][0]	= read_par_value(0x06,0x00);
		slot_array[0][1]	= read_par_value(0x06,0x01);
		slot_array[0][2]	= read_par_value(0x06,0x02);
		slot_array[1][0]	= read_par_value(0x06,0x03);
		slot_array[1][1]	= read_par_value(0x06,0x04);
		slot_array[1][2]	= read_par_value(0x06,0x05);
		slot_array[2][0]	= read_par_value(0x06,0x06);
		slot_array[2][1]	= read_par_value(0x06,0x07);
		slot_array[2][2]	= read_par_value(0x06,0x08);
		slot_array[3][0]	= read_par_value(0x06,0x09);
		slot_array[3][1]	= read_par_value(0x06,0x0A);
		slot_array[3][2]	= read_par_value(0x06,0x0B);
		slot_array[4][0]	= read_par_value(0x06,0x0C);
		slot_array[4][1]	= read_par_value(0x06,0x0D);
		slot_array[4][2]	= read_par_value(0x06,0x0E);
		slot_array[5][0]	= read_par_value(0x06,0x0F);
		slot_array[5][1]	= read_par_value(0x06,0x10);
		slot_array[5][2]	= read_par_value(0x06,0x11);
		slot_array[6][0]	= read_par_value(0x06,0x12);
		slot_array[6][1]	= read_par_value(0x06,0x13);
		slot_array[6][2]	= read_par_value(0x06,0x14);
		slot_array[7][0]	= read_par_value(0x06,0x15);
		slot_array[7][1]	= read_par_value(0x06,0x16);
		slot_array[7][2]	= read_par_value(0x06,0x17);
		slot_array[8][0]	= read_par_value(0x06,0x18);
		slot_array[8][1]	= read_par_value(0x06,0x19);
		slot_array[8][2]	= read_par_value(0x06,0x1A);
		slot_array[9][0]	= read_par_value(0x06,0x1B);
		slot_array[9][1]	= read_par_value(0x06,0x1C);
		slot_array[9][2]	= read_par_value(0x06,0x1D);
		slot_array[10][0]	= read_par_value(0x06,0x1E);
		slot_array[10][1]	= read_par_value(0x06,0x1F);
		slot_array[10][2]	= read_par_value(0x06,0x20);
		slot_array[11][0]	= read_par_value(0x06,0x21);
		slot_array[11][1]	= read_par_value(0x06,0x22);
		slot_array[11][2]	= read_par_value(0x06,0x23);
		
		// load sample_name -----------------------------------------------
		if (Osc.wave_mode[0] == 1)
		{
			seek_pointer16 = 0x0900;
			ffseek(seek_pointer16);
			for (uint8_t i = 0; i < 12; i++)
			{
				Gui.file_name1[i] = ffread();
			}			
		}
		if (Osc.wave_mode[1] == 1)
		{
			seek_pointer16 = 0x0910;
			ffseek(seek_pointer16);
			for (uint8_t i = 0; i < 12; i++)
			{
				Gui.file_name2[i] = ffread();
			}
		}
		
		ffclose();
		
		// load sample-files if available ---------------------------------
		if (Osc.wave_mode[0] == 1)
		{
			load_xwave(0);
		}
		if (Osc.wave_mode[1] == 1)
		{
			load_xwave(1);
		}
		
		// draw loading_bar -----------------------------------------------
		fillrectangle(94,110,222,113,color_gray);
		
		// load user waveforms --------------------------------------------	
		load_user_waveforms(0);
		load_user_waveforms(1);
				
		// draw menu_page 1: SOUND PROGRAMS	
		Patch.init_flag = 1;	
		draw_menupage_soundprg();
		Gui.osc_submenu[0] = 0;
		Gui.osc_submenu[1] = 0;
		LFO.phase[0] = 0;
		LFO.phase[1] = 0;
		LFO.phase[2] = 0;
		LFO.run_flag[0] = 1;
		LFO.run_flag[1] = 1;
		LFO.run_flag[2] = 1;
		
		
		// save Patch-Nr and Bank-Nr for Autoload into eeprom -------------
		if (Gui.last_patch == 1) // Autoload enabled
		{
			EEPWriteByte(0x0000, select_preset);		// Patch-Nr
			EEPWriteByte(0x0001, select_presetbank);	// Bank-Nr.
			_delay_ms(5);
		}
		
		
		// run 40KHz Audio Interuppt --------------------------------------
		TCC0.INTCTRLA = 0b00000011;			// run 40KHz Audio Interrupt
		init_Voice();
		
	}
	// Error: File not found! ---------------------------------------------
	else
	{
		print_info_box("No Preset..",80,80,1000);
		Patch.init_flag = 0;
		draw_menupage_soundprg();
	}
	
	ffclose();
}

//*************************************************************************
// set preset folder for Name.bin
//************************************************************************* 
void set_presetname_folder(void)
{
	ffcd(""); ffcd("PRESET");

	// convert sound_bank_nr in string
	char folder_temp[9];
	strcpy(folder_temp,"PRBANK");

	char xx[2];
	sound_bank = select_presetbank;

	itoa(sound_bank,xx,10);
	if (sound_bank > 9)
	{
		strcat(folder_temp,xx);
	}
	else
	{
		strcat(folder_temp,"0");
		strcat(folder_temp,xx);
	}
	

	// check folder there -------------------------------------------------
	if (fat_loadFileDataFromDir(folder_temp) == FALSE && save_flag == 1)
	{
		print_info_box("No Folder..",80,70,250);
		print_info_box("Create Folder..",80,70,250);
		ffmkdir(folder_temp);
		ffclose();
		_delay_ms(250);
		Gui.file_not_found_flag = 0;
	}
	
	ffcd(""); ffcd("PRESET"); ffcd(folder_temp);
}


//*************************************************************************
// write parameter_value 8Bit
//*************************************************************************
void write_par_value(uint8_t group_id, uint8_t par_id, uint8_t data)
{
	ffwrite(group_id);
	ffwrite(par_id);
	ffwrite(data);	
}

//*************************************************************************
// write parameter_value 32Bit
//*************************************************************************
void write_par_val32(uint8_t group_id, uint8_t par_id, uint32_t data)
{
	ffwrite(group_id);
	ffwrite(par_id);
	ffwrite(data);
	ffwrite(data >> 8);
	ffwrite(data >> 16);
	ffwrite(data >> 24);
}

//*************************************************************************
// delete PRESET-Data
//*************************************************************************
void delete_preset_data(void)
{
	// set folder
	set_presetname_folder();
	
	// file available ?
	if (fat_loadFileDataFromDir("Name.bin") == FALSE)
	{
		fillrectangle(30,38,280,145,color_blue1);
		rectangle(33,42,99,243,color_white);
		fillrectangle(139,42,180,65,color_blue1);
		print_string("File 'Name.bin' not found!",42,75,6,color_white);
		ffclose();
		_delay_ms(2000);
		return;
	}
	// delete Preset-Name
	else if( MMC_FILE_OPENED == ffopen("Name.bin",'r') )
	{
		ffseek(select_preset * 32);
		// save sound_data
		ffwrites("                ");		// 16 Byte for prg name
		for (uint8_t i = 0; i < 16; i++)    // 16 Byte for parameters
		{
			ffwrite(0); 
		}
		
		ffclose();
	}
	

	// delete Preset Data and Sample --------------------------------------
	set_preset_folder();
	
	if (fat_loadFileDataFromDir("Data.bin") == TRUE)
	{
		ffrm("Data.bin");
	}
	
	if (fat_loadFileDataFromDir("Sample1.bin") == TRUE)
	{
		ffrm("Sample1.bin");
	}
	
	if (fat_loadFileDataFromDir("Sample2.bin") == TRUE)
	{
		ffrm("Sample2.bin");
	}
	
	ffclose();
	
}

//*************************************************************************
// save Patch
//*************************************************************************
void save_preset_data(void)
{
	// stop 40KHz Audio Interuppt -----------------------------------------
	TCC0.INTCTRLA = 0b00000000;			// stop 40KHz Audio Interrupt
	
	// set folder
	set_presetname_folder();
	
	// file available ?
	if (fat_loadFileDataFromDir("Name.bin") == FALSE)
	{
		if(MMC_FILE_CREATED == ffopen("Name.bin",'c') )
		{
			print_info_box("Create Folder..",80,70,1);
			for (uint16_t i = 0; i <= 0x0FFF; i++)
			{
				ffwrite(0);
			}
			ffclose();
			_delay_ms(350);
		}
	}
	
	if (MMC_FILE_OPENED == ffopen("Name.bin",'r') );
	{
		uint16_t seek_pointer = (select_preset * 32);
		ffseek(seek_pointer);
		// save name
		ffwrites(preset_name);		// write string
		
		// set sine ore wave symbol
		ffseek(seek_pointer+16);
		if (Osc.wave_mode[0] == 1 || Osc.wave_mode[1] == 1)
		{
			ffwrite(0x02);
		}
		else {ffwrite(0x01);}
		
		// calc and save file lenght in 1KByte
		if (Osc.wave_mode[0] != 1 && Osc.wave_mode[1] != 1)
		{
			ffwrite(0);
			ffwrite(1);
		}
		else
		{
			uint16_t file_len = (Xram.sample_length[0] + Xram.sample_length[1]) / 1024;
			if (file_len < 1)
			{
				file_len = 1;
			}
			ffwrite(file_len >> 8);
			ffwrite((uint8_t)file_len);
		}
		
		ffclose();
	}
	
		
		
	// save Preset Data ---------------------------------------------------
	set_preset_folder();
	
	// Data.bin file available ?
	if (fat_loadFileDataFromDir("Data.bin") == FALSE)
	{
		if(MMC_FILE_CREATED == ffopen("Data.bin",'c') )
		{
			print_info_box("Create Data.bin..",80,70,250);
			for (uint16_t i = 0; i <= 0x0FFF; i++)
			{
				ffwrite(0);
			}
		}
		ffclose();
	}

	
	// draw save info
	print_info_box("Save File..",80,70,250);
		
	// draw loading bar
	rectangle(95,100,3,128,color_gray);
	
	// open Data.bin file -------------------------------------------------
	if( MMC_FILE_OPENED == ffopen("Data.bin",'r'))
	{				
		// Oscillator Parameter 0000-01FF ---------------------------------
		seek_pointer16 = 0x0000;
		ffseek(seek_pointer16);
		
		write_par_value(0x00,0x00,Osc.wave[0]);
		write_par_value(0x00,0x01,Osc.wave[1]);
		write_par_value(0x00,0x02,Osc.shape[0]);
		write_par_value(0x00,0x03,Osc.shape[1]);
		write_par_value(0x00,0x04,Osc.prm_value[0]);
		write_par_value(0x00,0x05,Osc.prm_value[1]);
		write_par_value(0x00,0x06,Osc.op_value);
		write_par_value(0x00,0x07,Osc.rng_value[0]);
		write_par_value(0x00,0x08,Osc.rng_value[1]);
		write_par_value(0x00,0x09,Osc.tune_value);
		write_par_value(0x00,0x0A,Osc.mix_value);
		write_par_value(0x00,0x0B,Osc.noise_value);
		write_par_value(0x00,0x0C,Osc.wavebank[0]);
		write_par_value(0x00,0x0D,Osc.wavebank[1]);
		write_par_value(0x00,0x0E,Osc.wave_mode[0]);
		write_par_value(0x00,0x0F,Osc.wave_mode[1]);
		write_par_value(0x00,0x10,Osc.sync_function[0]);
		write_par_value(0x00,0x11,Osc.sync_function[1]);
		write_par_value(0x00,0x12,Osc.loop_function[0]);
		write_par_value(0x00,0x13,Osc.loop_function[1]);
		
		// save 24Bit loop addr Osc1
		if (Osc.wave_mode[0] == 1 && Osc.loop_function[0] == 1)
		{
			write_par_value(0x00,0x14,Osc.loop_A[0]);
			write_par_value(0x00,0x15,Osc.loop_A[0] >> 8);
			write_par_value(0x00,0x16,Osc.loop_A[0] >> 16);
			write_par_value(0x00,0x17,Osc.loop_B[0]);
			write_par_value(0x00,0x18,Osc.loop_B[0] >> 8);
			write_par_value(0x00,0x19,Osc.loop_B[0] >> 16);
		}
		else
		{
			write_par_value(0x00,0x14,0);
			write_par_value(0x00,0x15,0);
			write_par_value(0x00,0x16,0);
			write_par_value(0x00,0x17,0);
			write_par_value(0x00,0x18,0);
			write_par_value(0x00,0x19,0);
		}
		
		// save 24Bit loop addr Osc2
		if (Osc.wave_mode[1] == 1 && Osc.loop_function[1] == 1)
		{
			write_par_value(0x00,0x20,Osc.loop_A[1]);
			write_par_value(0x00,0x21,Osc.loop_A[1] >> 8);
			write_par_value(0x00,0x22,Osc.loop_A[1] >> 16);
			write_par_value(0x00,0x23,Osc.loop_B[1]);
			write_par_value(0x00,0x24,Osc.loop_B[1] >> 8);
			write_par_value(0x00,0x25,Osc.loop_B[1] >> 16);
		}
		else
		{
			write_par_value(0x00,0x20,0);
			write_par_value(0x00,0x21,0);
			write_par_value(0x00,0x22,0);
			write_par_value(0x00,0x23,0);
			write_par_value(0x00,0x24,0);
			write_par_value(0x00,0x25,0);
		}
		
		// save ktrack function
		write_par_value(0x00,0x026,Osc.track_value[0]);
		write_par_value(0x00,0x027,Osc.track_value[1]);
		
		// Envelope Paramter 0200-02FF ------------------------------------
		seek_pointer16 = 0x0200;
		ffseek(seek_pointer16);
		write_par_value(0x01,0x00,Env.attack[0]);
		write_par_value(0x01,0x01,Env.attack[1]);
		write_par_value(0x01,0x02,Env.decay[0]);
		write_par_value(0x01,0x03,Env.decay[1]);
		write_par_value(0x01,0x04,Env.sustain[0]);
		write_par_value(0x01,0x05,Env.sustain[1]);
		write_par_value(0x01,0x06,Env.release[0]);
		write_par_value(0x01,0x07,Env.release[1]);
		
		// Filter Parameter 0300-03FF
		seek_pointer16 = 0x0300;
		ffseek(seek_pointer16);
		write_par_value(0x02,0x00,VCF.cutoff_value);
		write_par_value(0x02,0x01,VCF.reso_value);
		write_par_value(0x02,0x02,VCF.ktrack);
		write_par_value(0x02,0x04,VCF.filter_mode);
		write_par_value(0x02,0x05,VCF.ktrack_value);		
		
		// LFO Parameter 0400-04FF
		seek_pointer16 = 0x0400;
		ffseek(seek_pointer16);
		write_par_value(0x03,0x00,LFO.shape[0]);
		write_par_value(0x03,0x01,LFO.shape[1]);
		write_par_value(0x03,0x02,LFO.shape[2]);
		write_par_value(0x03,0x03,LFO.rate_volume[0]);
		write_par_value(0x03,0x04,LFO.rate_volume[1]);
		write_par_value(0x03,0x05,LFO.rate_volume[2]);
		write_par_value(0x03,0x0F,LFO.sync_flag[0]);
		write_par_value(0x03,0x10,LFO.sync_flag[1]);
		write_par_value(0x03,0x11,LFO.sync_flag[2]);
		write_par_value(0x03,0x12,LFO.oneshoot_flag[0]);
		write_par_value(0x03,0x13,LFO.oneshoot_flag[1]);
		write_par_value(0x03,0x14,LFO.oneshoot_flag[2]);
		
		// Mixer Parameter 0500-05FF
		seek_pointer16 = 0x0500;
		ffseek(seek_pointer16);
		write_par_value(0x04,0x00,pan_mix);
		
		// Fx parameter 0600-06FF
		seek_pointer16 = 0x0600;
		ffseek(seek_pointer16);
		write_par_value(0x05,0x00,Fx.Value);
		write_par_value(0x05,0x01,Fx.Pan);
		write_par_value(0x05,0x02,Fx.Time_value);
		write_par_value(0x05,0x03,Fx.Feedback);
		
		// Sequenzer & Arpeggiator ----------------------------------------
		seek_pointer16 = 0x680;
		ffseek(seek_pointer16);
		write_par_value(0x05,0x80,SEQ.seq_mode);
		write_par_value(0x05,0x81,SEQ.seq_bpm);
		write_par_value(0x05,0x82,SEQ.seq_div_value);
		write_par_value(0x05,0x83,SEQ.seq_dir);
		write_par_value(0x05,0x84,SEQ.seq_oct);
		
		// Modulation Matrix parameter 0700-08FF
		seek_pointer16 = 0x0700;
		ffseek(seek_pointer16);	
		write_par_value(0x06,0x00,slot_array[0][0]);	
		write_par_value(0x06,0x01,slot_array[0][1]);
		write_par_value(0x06,0x02,slot_array[0][2]);
		write_par_value(0x06,0x03,slot_array[1][0]);
		write_par_value(0x06,0x04,slot_array[1][1]);
		write_par_value(0x06,0x05,slot_array[1][2]);
		write_par_value(0x06,0x06,slot_array[2][0]);
		write_par_value(0x06,0x07,slot_array[2][1]);
		write_par_value(0x06,0x08,slot_array[2][2]);
		write_par_value(0x06,0x09,slot_array[3][0]);
		write_par_value(0x06,0x0A,slot_array[3][1]);
		write_par_value(0x06,0x0B,slot_array[3][2]);
		write_par_value(0x06,0x0C,slot_array[4][0]);
		write_par_value(0x06,0x0D,slot_array[4][1]);
		write_par_value(0x06,0x0E,slot_array[4][2]);
		write_par_value(0x06,0x0F,slot_array[5][0]);
		write_par_value(0x06,0x10,slot_array[5][1]);
		write_par_value(0x06,0x11,slot_array[5][2]);
		write_par_value(0x06,0x12,slot_array[6][0]);
		write_par_value(0x06,0x13,slot_array[6][1]);
		write_par_value(0x06,0x14,slot_array[6][2]);
		write_par_value(0x06,0x15,slot_array[7][0]);
		write_par_value(0x06,0x16,slot_array[7][1]);
		write_par_value(0x06,0x17,slot_array[7][2]);
		write_par_value(0x06,0x18,slot_array[8][0]);
		write_par_value(0x06,0x19,slot_array[8][1]);
		write_par_value(0x06,0x1A,slot_array[8][2]);
		write_par_value(0x06,0x1B,slot_array[9][0]);
		write_par_value(0x06,0x1C,slot_array[9][1]);
		write_par_value(0x06,0x1D,slot_array[9][2]);
		write_par_value(0x06,0x1E,slot_array[10][0]);
		write_par_value(0x06,0x1F,slot_array[10][1]);
		write_par_value(0x06,0x20,slot_array[10][2]);
		write_par_value(0x06,0x21,slot_array[11][0]);
		write_par_value(0x06,0x22,slot_array[11][1]);
		write_par_value(0x06,0x23,slot_array[11][2]);
		
		// clear old sample_name ------------------------------------------
		seek_pointer16 = 0x0900;
		ffseek(seek_pointer16);
		ffwriten(0,0x10);		
		seek_pointer16 = 0x0910;
		ffseek(seek_pointer16);
		ffwriten(0,0x10);
		
		// save new sample_name -------------------------------------------
		if (Osc.wave_mode[0] == 1)
		{
			seek_pointer16 = 0x0900;
			ffseek(seek_pointer16);
			ffwrites(Gui.file_name1);		// save new sample file_name
			
		}
		if (Osc.wave_mode[1] == 1)
		{
			
			seek_pointer16 = 0x0910;
			ffseek(seek_pointer16);
			ffwrites(Gui.file_name2);		// save new sample file_name
		}
		
		// close file -----------------------------------------------------
		ffclose();
	}

		// save sample_file1 (Osc1) ---------------------------------------
		if (Osc.wave_mode[0] == 1)
		{
			// delete old sample1.bin if available 
			if (fat_loadFileDataFromDir("SAMPLE1.bin") == TRUE)
			{
				ffrm("SAMPLE1.bin");
			}
			ffclose();
			
			// created new sample file
			if(MMC_FILE_CREATED == ffopen("SAMPLE1.bin",'c') )
			{
				// write sample data
				hugemem_ptr_t p = BOARD_EBI_SRAM_BASE;
				
				// calc loading-bar 
				uint16_t bar_count = Xram.sample_length[0] >> 6 ;
				uint16_t bar_count_temp = bar_count;
				uint8_t bar_value = 0;
				
				for (__uint24 i = 0; i < Xram.sample_length[0];i++)
				{
					bar_count_temp--;
					if (bar_count_temp == 0)
					{
						bar_value++;
						bar_count_temp = bar_count;
						fillrectangle(95,100,95+bar_value,103,color_gray);
					}
					ffwrite(hugemem_read8(p+i));
				}
				ffclose();	
			}
		}
		else
		{	// if osc wave_mode than delete old sample-file
			if (fat_loadFileDataFromDir("SAMPLE1.bin") == TRUE)
			{
				ffrm("SAMPLE1.bin");
				ffclose();
			}
		}
		
		// save Osc2 sample file ------------------------------------------
		if (Osc.wave_mode[1] == 1)
		{
			// delete old sample1.bin if available
			if (fat_loadFileDataFromDir("SAMPLE2.bin") == TRUE)
			{
				ffrm("SAMPLE2.bin");
			}
			ffclose();
			
			// created new sample file
			if(MMC_FILE_CREATED == ffopen("SAMPLE2.bin",'c') )
			{
				// write sample data --------------------------------------
				hugemem_ptr_t p = BOARD_EBI_SRAM_BASE + (0xef1ff - Xram.sample_length[1]);
								
				// calc loading-bar ---------------------------------------
				uint16_t bar_count = Xram.sample_length[1] >> 6 ;
				uint16_t bar_count_temp = bar_count;
				uint8_t bar_value = 0;
				
				for (__uint24 i = 0; i < Xram.sample_length[1];i++)
				{
					bar_count_temp--;
					if (bar_count_temp == 0)
					{
						bar_value++;
						bar_count_temp = bar_count;
						fillrectangle(95,100,95+64+bar_value,103,color_gray);
					}
					ffwrite(hugemem_read8(p+i));
				}
				ffclose();	
			}
		}
		else
		{	// if osc wave_mode than delete old sample-file
			if (fat_loadFileDataFromDir("SAMPLE2.bin") == TRUE)
			{
				ffrm("SAMPLE2.bin");
				ffclose();
			}
			
		}
		
	fillrectangle(95,100,95+128,103,color_gray);
	_delay_ms(250);
				
		
	// run 40KHz Audio Interuppt --------------------------------------
	TCC0.INTCTRLA = 0b00000011;			// run 40KHz Audio Interrupt
		
}

//*************************************************************************
// draw Soundpatch List
//*************************************************************************
void draw_SoundList(void)
{
	// clear window area
	fillrectangle(0,20,320,165, color_black);
	// draw preset-nr	
	uint8_t row = 30;
	//uint8_t col = 0;
	for (uint8_t i = Sound_start_entry; i <= (Sound_start_entry + 6); i++)
	{
		if (i < 128)
		{
			draw_3digit_with(i,8,row, 6, color_white);
		}
		// next row
		row = row +19;	
	}
	
	// calc preset entry
	if ((Sound_start_entry+6) < 128)
	{
		Sound_last_entry = Sound_start_entry+6;
	}
	else {Sound_last_entry = 128;}
	
	// convert bank-nr in string ------------------------------------------
	char filename[13];
	strcpy(filename,"SOBANK");
	char *xx = "";
	itoa(sel_sound_bank,xx,10);
	if (sel_sound_bank > 9)
	{
		strcat(filename,xx);
	}
	else
	{
		strcat(filename,"0");
		strcat(filename,xx);
	}
	strcat(filename,".bin");
	
	// open file ----------------------------------------------------------
	ffcd("");
	ffcd("PRESET");
	if( MMC_FILE_OPENED == ffopen(filename,'r') )
	{
		// read sound_name
		row = 30;
		//col = 25;
		//uint8_t new_selEntry = 0;
		uint16_t seek_pointer = 0;
		for (uint8_t i = 0; i <=(Sound_last_entry-Sound_start_entry) ; i++)
		{
			seek_pointer = ((Sound_start_entry+i) * 256);
			ffseek(seek_pointer);		// file seek_pointer
			uint8_t string_len;
			for (string_len = 0; string_len < 16; string_len++)
			{
				soundname_x[string_len] = ffread();
			}
			// draw sound name
			print_string(soundname_x,40,row,6,color_white);
			// next row
			row=row+19;
			
			// test end of file
			if (seek_pointer == 32512)
			{
				break;
			}
		}
		//old_selEntry = new_selEntry;
	}
	ffclose();
}

//*************************************************************************
// load_save sequence
//*************************************************************************
void load_save_seq(void)
{
	if (load_flag == 1)
	{
		// loading sequenz ------------------------------------------------
		// print loading..
		fillrectangle(80,105,240,130,color_blue1);
		print_string("Loading...",115,105,6,color_white);		
		
		// open sequencer file --------------------------------------------
		ffcd("");
		ffcd("SYSTEM");
		
		// load sequencer file --------------------------------------------
		if( MMC_FILE_OPENED == ffopen("SEQ.bin",'r') )
		{
			// set seq_nr -------------------------------------------------
			//uint16_t seek_pointer = seq_nr * 80;
			ffseek(SEQ.seq_nr * 80);		// file seek_pointer
			
			uint8_t i;
			// load sequencer notes ---------------------------------------
			for (i = 0; i <= 15; i++)
			{
				SEQ.step_note[i] = ffread();
			}
			
			// load notes holdtime ----------------------------------------
			for (i = 0; i <= 15; i++)
			{
				SEQ.step_hold[i] = ffread();
			}
			
			// load mod_value --------------------------------------------
			for (i = 0; i <= 15; i++)
			{
				SEQ.step_mod[i] = ffread();
			}
			
			// load step function -----------------------------------------
			for (i = 0; i <= 15; i++)
			{
				SEQ.step_function[i] = ffread();
			}
			
			// load seq_parameter -----------------------------------------
			SEQ.seq_speed = ffread();				// read 16Bit speed value
			SEQ.seq_speed = ((SEQ.seq_speed << 8) | ffread());
			SEQ.seq_shuffle = ffread();
			SEQ.seq_dir = ffread();
			SEQ.seq_oct = ffread();
			_delay_ms(350);
		}
		// Error: file not found
		else
		{
			fillrectangle(95,105,240,130,color_blue1);
			print_string("File not found !",95,105,6,color_white);
			_delay_ms(1000);
		}
		
		
	}
	
	// save sequence ------------------------------------------------------
	if (save_flag == 1)
	{
		// print Save...
		fillrectangle(80,105,240,130,color_blue1);
		print_string("Saving...",115,105,6,color_white);
		
		// open sequencer file --------------------------------------------
		ffcd("");
		ffcd("SYSTEM");
		
		// save sequencer file --------------------------------------------
		if( MMC_FILE_OPENED == ffopen("SEQ.bin",'r') )
		{
			// set seq_nr -------------------------------------------------
			//uint16_t seek_pointer = seq_nr * 80;
			ffseek(SEQ.seq_nr * 80);		// file seek_pointer
			
			uint8_t i = 0;
			// load sequencer notes ---------------------------------------
			for (i = 0; i <= 15; i++)
			{
				ffwrite(SEQ.step_note[i]);
			}
			
			// load notes holdtime ----------------------------------------
			for (i = 0; i <= 15; i++)
			{
				ffwrite(SEQ.step_hold[i]);
			}
			
			// load velo value --------------------------------------------
			for (i = 0; i <= 15; i++)
			{
				ffwrite(SEQ.step_mod[i]);
			}
			
			// load step function -----------------------------------------
			for (i = 0; i <= 15; i++)
			{
				ffwrite(SEQ.step_function[i]);
			}
			
			// load seq_parameter -----------------------------------------
			ffwrite(SEQ.seq_speed >> 8);			// save 16Bit speed value
			ffwrite(SEQ.seq_speed & 0xFF);
			ffwrite(SEQ.seq_shuffle);
			ffwrite(SEQ.seq_dir);
			ffwrite(SEQ.seq_oct);
			_delay_ms(350);
		}
		// Error: file not found
		else
		{
			fillrectangle(95,105,240,130,color_blue1);
			print_string("File not found !",95,105,6,color_white);
			_delay_ms(1000);
		}
	}
	
	ffclose();
	seq_submenu = 0;
	draw_menupage_seq();
}

//*************************************************************************
// create new "SBank_A" file
//*************************************************************************
void create_Soundbank(void)
{		
	// read Sample Files
	ffcd("");
	ffcd("PRESET");
	// write info to lcd
	//lcd_moveto_xy(7,0);
	//lcd_put_string(FONT_PROP_8,0,"write new SBank...");
	
	char *file_name = "SOBANK00.bin";	// Sound_Bank A
	uint8_t str1 [17] = "---             \0";	// Sound-Name (16 Bytes)
	
	if(MMC_FILE_CREATED == ffopen(file_name,'c') )
	{
		uint16_t seek_pointer;
		
		for (uint8_t ix = 0; ix < 128;ix++)
		{
			// sound data
			seek_pointer = ix * 256;
			ffseek(seek_pointer);  // file seek_pointer
			ffwrites(str1);
			
			for (uint16_t i = 16; i <=255; i++)
			{
				ffwrite(0);
			}
		}
		
		ffclose();
		
		//lcd_put_string(FONT_PROP_8,0,"New SBank is written...");
		_delay_ms(1000);		
	}
}

//*************************************************************************
// load user waveforms from sdcard
//*************************************************************************
Bool load_user_waveforms(uint8_t osc_nr)
{

	// set offset in xram
	__uint24 offset;
	uint8_t wavebank_temp;
	if (osc_nr == 0)
	{
		offset = 0xEFC00;
		wavebank_temp = Osc.wavebank[0];
	}
	else
	{
		offset = 0xF7C00;
		wavebank_temp = Osc.wavebank[1];
	}
	
	// set baseaddress to sram
	hugemem_ptr_t   p = BOARD_EBI_SRAM_BASE + offset; // 0x800000UL + offset
	
	//uint8_t    data = 0;
	//uint8_t sample;
	
	// open userwave file -------------------------------------------------
	ffcd("");
	ffcd("WAVEFORM");
	
	// convert bank-nr in string ------------------------------------------
	char filename[13];
	strcpy(filename,"WAFORM");
	
	char xx[2];
	itoa(wavebank_temp,xx,10);
	if (wavebank_temp > 9)
	{
		strcat(filename,xx);
	}
	else
	{
		strcat(filename,"0");
		strcat(filename,xx);
	}
	strcat(filename,".bin");
	
	// check file there ---------------------------------------------------
	if (fat_loadFileDataFromDir(filename) == FALSE)
	{
		ffclose();
		return FALSE;
	}
	
	// open bin.file ------------------------------------------------------
	if( MMC_FILE_OPENED == ffopen(filename,'r') )
	{
		uint16_t seek_pointer;
		seek_pointer = 0;
		ffseek(seek_pointer);		// file seek_pointer
		
		// load 32KB user waveform from sdcard to xram addr. Osc1=0x0000
		uint8_t sample_buf[16];
		
		for (uint16_t addr = 0; addr <= 0x7FFF;)
		{
			for (uint8_t count = 0; count < 16;++count)
			{
				sample_buf[count] = ffread(); // load sample and invert
			}
			hugemem_write_block(p + addr, sample_buf, 16);			
			addr += 16;
		}
		ffclose();		
		return TRUE;
	}	
}

//*************************************************************************
// delete Wavefile on SDCARD
//*************************************************************************
void delete_wavefile(uint8_t osc_nr)
{
	// build folder name
	char folder[9];
	char *xx = "";
	
	strcpy(folder,"SABANK");
	
	itoa(Osc.wavebank[osc_nr],xx,10);
	if (Osc.wavebank[osc_nr] > 9)
	{
		strcat(folder,xx);
	}
	else
	{
		strcat(folder,"0");
		strcat(folder,xx);
	}
	
	// set folder
	ffcd(""); ffcd("SAMPLE"); ffcd(folder);
		
	// file available ?
	if (fat_loadFileDataFromDir(sample_name) == TRUE)
	{
		// delete file
		ffrm(sample_name);
		ffclose();
	}
	else
	{
		print_file_not_found(1000,20,80);
	}
	
}

//*************************************************************************
// save wav.file
//*************************************************************************
void save_wave_file(uint8_t osc_nr)
{	
	
	// build folder name ----------------------------------
	char folder[9];
	char xx[2];
	
	strcpy(folder,"SABANK");
	
	itoa(Osc.wavebank[osc_nr],xx,10);
	if (Osc.wavebank[osc_nr] > 9)
	{
		strcat(folder,xx);
	}
	else
	{
		strcat(folder,"0");
		strcat(folder,xx);
	}
	
	// set folder
	ffcd(""); ffcd("SAMPLE");
	
	// check folder there -------------------------------------------------
	if (fat_loadFileDataFromDir(folder) == FALSE)
	{
		print_info_box("No Folder..",80,80,250);
		print_info_box("Create Folder..",80,80,250);
		ffmkdir(folder);
		ffclose();
		_delay_ms(250);
		Gui.file_not_found_flag = 0;
	}
	ffcd(""); ffcd("SAMPLE"); ffcd(folder);
		
	// set file-name ------------------------------------------------------
	strcat(file_name,".wav");

	// file_name exist?
	if (fat_loadFileDataFromDir(file_name) == TRUE)
	{
		// print error massage --------------------------------------------
		print_info_box("File exist !!!",80,80,2000);
		ffclose();
	}
	else
	{
		// save new file --------------------------------------------------
		if(MMC_FILE_CREATED == ffopen(file_name,'c') )
		{
			ffseek(0);
			
			// write wav-header -----------------------------------------------
			ffwrite(0x52);		// 'RIFF'						// 0000h
			ffwrite(0x49);
			ffwrite(0x46);
			ffwrite(0x46);
			uint32_t temp = ((Xram.sample_length[osc_nr] * 2) + 44) - 8;			
			ffwrite(temp);										// 0002h
			ffwrite(temp >> 8);
			ffwrite(temp >> 16);
			ffwrite(temp >> 24);
			ffwrite(0x57);		// 'WAVE'
			ffwrite(0x41);
			ffwrite(0x56);
			ffwrite(0x45);
			ffwrite(0x66);		// 'fmt '
			ffwrite(0x6d);
			ffwrite(0x74);
			ffwrite(0x20);
			ffwrite(0x10);		// Subchunk1 Size = 16
			ffwrite(0x00);
			ffwrite(0x00);
			ffwrite(0x00);
			ffwrite(0x01);		// Audio Format = 1 PCM
			ffwrite(0x00);
			ffwrite(0x01);		// Channels = 1
			ffwrite(0x00);
			ffwrite(0x44);		// Sample rate = 44100
			ffwrite(0xac);
			ffwrite(0x00);
			ffwrite(0x00);
			ffwrite(0x88);		// Byte/sec = 88200
			ffwrite(0x58);
			ffwrite(0x01);
			ffwrite(0x00);
			ffwrite(0x02);		// NumChannels * BitsPerSample / 8 = 2
			ffwrite(0x00);
			ffwrite(0x10);		// Bit per sample = 16
			ffwrite(0x00);
			ffwrite(0x64);		// 'data'
			ffwrite(0x61);
			ffwrite(0x74);
			ffwrite(0x61);
			temp = (Xram.sample_length[osc_nr]*2);
			ffwrite(temp);
			ffwrite(temp >> 8);
			ffwrite(temp >> 16);
			ffwrite(temp >> 24);
			
			// write sample data ------------------------------------------
			hugemem_ptr_t p = BOARD_EBI_SRAM_BASE;
			
			// draw info and bar
			print_info_box("Save Sample..",80,80,1);
			rectangle(95,109,3,128,color_gray);
			
			uint16_t bar_count = Xram.sample_length[osc_nr] >> 7;
			uint16_t bar_count_temp = bar_count;
			uint8_t bar_value = 0;
			__uint24 i = 0;
			__uint24 count = 0;
			
			if (osc_nr != 0)
			{
				i = 0xef1fd - Xram.sample_length[1];
				count = 0xef1fd;
			}
			else count = Xram.sample_length[0];
			
			for (; i < count; i++)
			{
				bar_count_temp--;
				if (bar_count_temp == 0)
				{
					bar_value++;
					bar_count_temp = bar_count;
					fillrectangle(95,109,95+bar_value,112,color_gray);
				}
				
				int8_t temp = ((hugemem_read8(p+i)) - 128);	// load 8Bit sample and invert and make signed
				ffwrite(0x00);
				ffwrite(temp);
			}
			
			// write loop points ------------------------------------------
			if (Gui.CurA_addr != Gui.CurB_addr && Gui.sample_edit_function == 2)
			{
				ffwrite('s');
				ffwrite('m');
				ffwrite('p');
				ffwrite('l');
				ffwrite(60);
				for (uint8_t i = 0; i < 15; i++)
				{
					ffwrite(0);
				}
				ffwrite(60);
				for (uint8_t i = 0; i < 15; i++)
				{
					ffwrite(0);
				}
				ffwrite(1);
				for (uint8_t i = 0; i < 15; i++)
				{
					ffwrite(0);
				}
				
				temp = Gui.CurA_addr;
				ffwrite(temp);
				ffwrite(temp >> 8);
				ffwrite(temp >> 16);
				ffwrite(0);
				temp = Gui.CurB_addr;
				ffwrite(temp);
				ffwrite(temp >> 8);
				ffwrite(temp >> 16);
				ffwrite(0);
			}
			ffclose();
		}
	}
	
	
}


//*************************************************************************
// get sample-file from sample-folder
//*************************************************************************
uint8_t load_wavefile(uint8_t osc_nr)
{	
	// stop osc and audio interrupt ---------------------------------------
	TCC0.INTCTRLA = 0b00000000;			// run 40KHz Audio Interrupt
	
	// convert bank-nr in string ------------------------------------------
	char foldername[9];
	strcpy(foldername,"SABANK");
	char xx[2];
	itoa(Osc.wavebank[osc_nr],xx,10);
	if (Osc.wavebank[osc_nr] > 9)
	{
		strcat(foldername,xx);
	}
	else
	{
		strcat(foldername,"0");
		strcat(foldername,xx);
	}
	strcat(foldername,"\0");
	
	// set wave-file name -------------------------------------------------
	read_samplename(osc_nr);
			
	// set folder ---------------------------------------------------------
	uint8_t error_status = 0;
	ffcd("");
	ffcd("SAMPLE");ffcd(foldername);
	
	// open file ----------------------------------------------------------
	if( MMC_FILE_OPENED == ffopen(sample_name,'r'))
	{		
		// set baseaddress sram
		hugemem_ptr_t p = BOARD_EBI_SRAM_BASE; // Base-Addr. 0x00000-EFEFE
		//uint8_t    data = 0;
		//uint8_t sample;
		uint8_t bits_sample;
		//uint8_t temp = 25;
		__uint24 offset;
		quant_error16 = 0;
				
		// Load Wave Files from Sample-Bank
		if (Osc.wave_mode[osc_nr] == 1)
		{
			// read bits/sample -------------------------------------------
			ffseek(0x22);
			bits_sample = ffread();
			
			// read sound-file lenght -------------------------------------
			ffseek(0x28);
			uint8_t data_lenght[4];
			data_lenght[0] = ffread();
			data_lenght[1] = ffread();
			data_lenght[2] = ffread();
			data_lenght[3] = ffread();
			file_lenght = *((uint32_t*) data_lenght);
			//file_lenght = file_lenght;
			
			// calc max data lenght ---------------------------------------
			file_lenght >>= 1; // convert 16Bit lenght to 8Bit lenght
			if (osc_nr == 0)
			{
				Xram.free_space = 0xef1ff - Xram.sample_length[1];
			}
			else{Xram.free_space = 0xef1ff - Xram.sample_length[0];}
			
			if (file_lenght >= Xram.free_space)
			{
				// draw loading info and bar
				print_info_box("File to long..",82,82,1000);
				fillrectangle(80,80,240,120,color_black);
				ffclose();
				return 0;
			}
			
			// set file_length --------------------------------------------
			if (osc_nr == 0)
			{
				Xram.sample_length[0] = file_lenght;
				offset = 0;
				Gui.redraw_flag[0] = 0;
			}
			else
			{
				Xram.sample_length[1] = file_lenght;
				offset = 0xef1ff - Xram.sample_length[1];
				Gui.redraw_flag[1] = 0;
			}
						
			// set buffer_addr for sample screen in xram ------------------
			__uint24 sample_curve_addr;
			if (osc_nr == 0){sample_curve_addr = 0xFFC00;}
			else {sample_curve_addr = 0xFFD00;}
			
			// read 16bit format ------------------------------------------
			if(bits_sample == 16)
			{
				quantization_error = 0;
				__uint24 sample_block = Xram.sample_length[osc_nr] / 90;
				uint16_t x = 16;
				uint8_t y1 = 25;
				uint8_t y2 = 25;
				uint8_t count = 0;
				
				// draw loading info and bar
				fillrectangle(39,40,260,146,color_white);
				rectangle(41,42,102,216,color_black);
				print_string("Loading Sample...",65,56,6,color_black);
								
				// calc and draw loding counter ---------------------------
				__uint24 bar_count = Xram.sample_length[osc_nr] / 100;
				__uint24 bar_count_temp = bar_count;
				uint16_t bar_value = 0;
				print_string("%",232,56,6,color_black);
				uint8_t sample = 0;
				
				for (__uint24 i = 0; i < Xram.sample_length[osc_nr]; i++)
				{
					
					bar_count_temp--;
					if (bar_count_temp == 0)
					{
						bar_value++;
						bar_count_temp = bar_count;
						fillrectangle(205,56,230,68,color_white);	// clear background counter
						draw_3digit_without(bar_value,205,56,6,color_black); // print %
					}

					// load 16Bit wave-file from sdcard -------------------
					uint8_t sample_l = ffread();		// load L-Byte
					uint8_t sample_h = ffread();		// load H-Byte
					
					// NoiseShaper ----------------------------------------------------
					__int24 sample_16 = (__int24)((sample_h <<8) | sample_l);
					sample_16 += quantization_error;
					if (sample_16 < -32768) sample_16 = -32768;
					else if (sample_16 > 32767) sample_16 = 32767;
					int8_t sample_8 = sample_16 >> 8;
					quantization_error = sample_16 - ((__int24)(sample_8) << 8);
					
					// convert to unsigned 8Bit ---------------------------
					sample = sample_8 + 128;
					//sample = sample_h + 128;
					
					// write to ext. SRAM ---------------------------------
					hugemem_write8(p + offset + i, sample);
										
					// draw wave-file if >= 128 Byte ----------------------
					if (file_lenght >= 0xFF)
					{
						// calc min-max in sample block -------------------
						sample = sample / 5;
						if (sample > 25)
						{
							if (sample > y1)
							{
								y1 = sample;
							}
						}
						else
						{
							if (sample < y2)
							{
								y2 = sample;
							}
						}
						
						// draw min-max sample from sample block-----------
						if (i == sample_block)
						{
							// draw envelope curve
							vline(x + 45, 129 - y1, (y1 - 25) + (25 - y2), color_black);
							
							// save max sample for envelope curve (Ram End 1.048.064)
							hugemem_write8(p + sample_curve_addr + count, y1);
							count++;
							hugemem_write8(p + sample_curve_addr + count , y2);
							count++;
							
							y1 = 25;	// clr max sample
							y2 = 25;	// clr min sample
							x += 2;
							sample_block += file_lenght / 90;	// inc next sample block
						}
					}
				}
				
				/*
				// copy first byte from sample and write to sample end ----
				// it's for interpolation
				for (uint8_t i = 0; i < 255; i++)
				{
					sample = hugemem_read8(p + offset + i);
					hugemem_write8(p + offset + Xram.sample_length[osc_nr] + i + 1, sample);
				}
				*/
								
				// set redraw_flag for sample screen
				Gui.redraw_flag[osc_nr] = 1;
								
				// loop_points available ? --------------------------------
				if (file.length > ((file_lenght * 2) + 44))
				{
					char string[6];
					for (uint8_t i = 0; i < 5; i++)
					{
						string[i] = ffread();
					}
					string[5] = 0;
										
					if (strcmp (string, "smpl<") == 0)
					{
						for (uint8_t i = 0; i < 47; i++)
						{
							//uint8_t temp = ffread();
							ffread();
						}
						uint8_t temp_data[4];
						temp_data[0] = ffread();
						temp_data[1] = ffread();
						temp_data[2] = ffread();
						temp_data[3] = ffread();
						Osc.loop_A[osc_nr] =  *((uint32_t*) temp_data);
						temp_data[0] = ffread();
						temp_data[1] = ffread();
						temp_data[2] = ffread();
						temp_data[3] = ffread();
						Osc.loop_B[osc_nr] =  *((uint32_t*) temp_data);
						Osc.loop_function[osc_nr] = 1;					
						Gui.sample_edit_function = 2; // set Loop function
					}
					else
					{
						Osc.loop_A[osc_nr] = 0;
						Osc.loop_B[osc_nr] = Xram.sample_length[osc_nr];
						Osc.loop_function[osc_nr] = 0;
						Gui.sample_edit_function = 1;
					}
					_delay_ms(100);					
				}
				
			}
			
			// Error: Bits per Sample Fault
			else
			{
				fillrectangle(0,0,320,20,0x31,0x86);
				print_string("16Bit Error!",0,4,6,color_yello);
				draw_3digit_without(bits_sample,175,4,6,color_yello);
				_delay_ms(2000);
			}
		}
		
		// set interpolation byte 
		//file_lenght++;
		//hugemem_write8(p + offset + file_lenght, sample_temp);
		// set Loop Points ------------------------------------------------
		Osc.loop_endaddr[osc_nr] = file_lenght-1;
		
		
	}
	
	// Error: "File not found.." ------------------------------------------
	else
	{
		error_status = 1;
		print_file_not_found(1000,20,80);
	}
	
	ffclose();
	
	// select osc and audio interrupt -------------------------------------
	TCC0.INTCTRLA = 0b00000011;			// run 40KHz Audio Interrupt
	
	// init parameter in sample-page --------------------------------------
	Enc.value[1] = 0;
	Enc.value_[1] = 0;
	Enc.value_old[1] = 0;
	Enc.value[2] = 0;
	Enc.value_[2] = 0;
	Enc.value_old[2] = 0;
	Enc.value[3] = 0;
	Enc.value_[3] = 0;
	Enc.value_old[3] = 0;
	//Gui.CurA_addr = 0;
	Gui.CurA_value = 0;
	Gui.CurA_old_value = 0;
	//Gui.CurB_addr = 0;
	Gui.CurB_value = 0;
	Gui.CurB_old_value = 0;
	Gui.addr_offset = 0;
	Gui.block_size = 0;
	Gui.Zoom_value = 0;
	Gui.block_size = (Xram.sample_length[0] >> 8) >> Gui.Zoom_value;
	
	return error_status;
	
}

//*************************************************************************
// Load binary sample-file if available in the preset folder
//*************************************************************************
void load_xwave(uint8_t osc_nr)
{
	char file_name_[12];
	
	if (osc_nr == 0)
	{
		strcpy(file_name_,"SAMPLE1.bin");
	}
	else {strcpy(file_name_,"SAMPLE2.bin");}
	
	// open file ----------------------------------------------------------	
	if( MMC_FILE_OPENED == ffopen(file_name_,'r'))
	{
		//uint8_t    data = 0;
		hugemem_ptr_t p = BOARD_EBI_SRAM_BASE;
		uint32_t offset = 0;
		uint8_t sample;
		//uint8_t bits_sample;
		uint32_t file_lenght = file.length;
		
		if (osc_nr == 0)
		{
			Xram.sample_length[0] = file_lenght;
			offset = 0;
			Gui.redraw_flag[0] = 0;
		}
		else 
		{
			Xram.sample_length[1] = file_lenght;
			offset = 0xef1ff - (Xram.sample_length[1]);
			Gui.redraw_flag[1] = 0;
		}
				
		uint32_t sample_block = file_lenght / 90;
		//uint16_t x0 = 0;
		//uint8_t last_sample = 0;
		//uint8_t last_sample_h = 127;
		//uint8_t last_sample_l = 127;
		//uint8_t hoehe_l = 0;
		//uint8_t hoehe_h = 0;
		//uint8_t hoehe_x = 0;
		uint8_t count = 0;
		uint16_t x = 16;
		uint8_t y1 = 25;
		uint8_t y2 = 25;
		
		// calc loading-bar ---------------------------------------
		uint16_t bar_count;
		uint8_t xpos_secBar;
		if (Osc.wave_mode[0] == 1 && Osc.wave_mode[1] == 1)
		{
			bar_count = file_lenght >> 6;
			if (osc_nr == 0)
			{
				xpos_secBar = 94;
			}
			else xpos_secBar = 158;
			
			
		}
		else
		{
			bar_count = file_lenght >> 7;
			xpos_secBar = 94;
		}
		
		uint16_t bar_count_temp = bar_count;
		uint8_t bar_value = 0;
		
		// draw loading-bar ---------------------------------------
		for (__uint24 i = 0; i <= file_lenght; i++)
		{
			bar_count_temp--;
			if (bar_count_temp == 0)
			{
				bar_value++;
				bar_count_temp = bar_count;
				//fillrectangle(94,110,222,113,color_gray);
				fillrectangle(94,110,xpos_secBar+bar_value,113,color_gray);
				//fillrectangle(68,99,xpos_secBar+bar_value,102,color_gray);
			}
			
			// load 8Bit wave-file from sdcard
			sample = ffread();			// load sample
			
			// write to xram
			hugemem_write8(p + offset + i, sample);
			
			__uint24 sample_curve_addr;
			
			// set screen buffer addr in xram ---------------
			if (osc_nr == 0)
			{
				sample_curve_addr = 0xFFC00;
			}
			else {sample_curve_addr = 0xFFD00;}
			
			// calc min-max in sample block -------------------
			sample = sample / 5;
			if (sample > 25)
			{
				if (sample > y1)
				{
					y1 = sample;
				}
			}
			else
			{
				if (sample < y2)
				{
					y2 = sample;
				}
			}
			
			// draw min-max sample from sample block-----------
			if (i == sample_block)
			{
				// save max sample for envelope curve (Ram End 1.048.064)
				hugemem_write8(p + sample_curve_addr + count, y1);
				count++;
				hugemem_write8(p + sample_curve_addr + count, y2);
				count++;
				
				y1 = 25;	// clr max sample
				y2 = 25;	// clr min sample
				x += 2;
				sample_block += file_lenght / 90;	// inc next sample block
			}
		}
		
		// fill loading bar and close file
		//fillrectangle(68,99,68+128,102,color_gray);
		Gui.redraw_flag[osc_nr] = 1;
		ffclose();
	}
	
	// Error: "File not found.." ------------------------------------------
	else
	{
		print_file_not_found(1000,20,80);
		ffclose();			
	}
}	


//*************************************************************************
//  read Directory Files
//*************************************************************************
void getFileNumbers(void)
{
	file_numbers++;	
}

//*************************************************************************
//  read directory and save name to buffer
//*************************************************************************
void getFileNames(char *ptr)
{	
	if ((file_temp_entry >= (file_current_entry +2)) && (file_temp_entry <= (file_last_entry + 2)))
	{
		strcat (filename_buffer, ptr);
	}
	
	file_temp_entry++;
}

//*************************************************************************
// read dir entrys from sample folder
//*************************************************************************
void read_dir_entrys(uint8_t current_entry)
{	
	// clear window area
	fillrectangle(2,23,317,162, color_black);
		
	// get file numbers ---------------------------------------------------
	file_numbers = 0;			// clear file counter
	ffls_ex(getFileNumbers);	// get file numbers
	file_numbers -= 2;
	
	// calc the last entry of page ----------------------------------------
	uint8_t last_entry = file_numbers-1;
	
	if (last_entry - current_entry > 6)
	{
		last_entry = current_entry + 6;
	}
	
	// print row-numbers --------------------------------------------------
	uint8_t row = 30;
	uint8_t col = 0;
	for (uint8_t i = current_entry; i <= last_entry; i++)
	{
		draw_3digit_with(i,7,row, 6, color_white);
		row = row +19;
	}
	
	// clear filename buffer ----------------------------------------------
	strcpy(filename_buffer,"\0");
	
	// set entrys and load filenames in buffer ----------------------------
	file_current_entry = current_entry;
	file_last_entry = last_entry;
	file_temp_entry = 0;
	ffls_ex(getFileNames);
	
	// print file-list ----------------------------------------------------
	row = 30;
	col = 45;
		
	for (uint8_t ix = 0; ix <= (last_entry - current_entry);ix++)
	{
		char file_namex[13] = "\0";
		char x;
		uint8_t iy = 0;
		for (uint8_t i = 0; i < 8;i++)
		{
			x = filename_buffer[(ix*11)+i];
			
			
			if (x != ' ')		// check space. this is end of name
			{
				file_namex[iy]=x;
				iy++;
			}
		}
		
		print_string(file_namex,col,row,6,color_white);
		
		
		// open wav-file --------------------------------------------------
		strcat(file_namex,".WAV");
		print_string(file_namex,col,row,6,color_white);
		ffopen(file_namex,'r');
		
		// print Sample-Resolution ----------------------------------------
		ffseek(0x22);
		uint8_t bits = ffread();
		draw_3digit_without(bits,160,row,6,color_yello);
		print_string("Bit",188,row,6,color_yello);
		
		// print sample-file lenght into KByte ----------------------------
		__uint24 file_len = file.length;
		file_len >>= 1;
		if (file_len >= 1024)
		{
			uint16_t len_temp = (file_len / 1024);
			draw_4digit_without(len_temp,262,row,6,color_white);
			print_string("KB",297,row,6,color_white);
		}
		else // ore bytes
		{
			draw_4digit_without(file_len,262,row,6,color_white);
			print_string("b",297,row,6,color_white);
		}
		
		// set next row ---------------------------------------------------
		row = row + 19;
	}
	
	// close file ---------------------------------------------------------
	ffclose();
	
}

//*************************************************************************
// read Sample Dir
//*************************************************************************
void read_sample_dir(uint8_t current_entry, uint8_t osc_nr)
{
	// build folder name ----------------------------------
	char folder[9];
	char xx[2];
	
	strcpy(folder,"SABANK");
	
	itoa(Osc.wavebank[osc_nr],xx,10);
	if (Osc.wavebank[osc_nr] > 9)
	{
		strcat(folder,xx);
	}
	else
	{
		strcat(folder,"0");
		strcat(folder,xx);
	}
	
	// set directory folder -----------------------------------
	ffcd("");		// to root directory
	getDir("SAMPLE",folder);
	ffcd("");
	ffcd("SAMPLE");ffcd(folder);

	// read folder entrys ---------------------------------
	read_dir_entrys(current_entry);
}

//*************************************************************************
// check  SABANK exist
//*************************************************************************
Bool check_SABANK_exist(uint8_t SABANK_NR)
{
	// build folder name ----------------------------------
	char folder_name[9];
	char xx[2];
	
	strcpy(folder_name,"SABANK");
	
	itoa(SABANK_NR,xx,10);
	if (SABANK_NR > 9)
	{
		strcat(folder_name,xx);
	}
	else
	{
		strcat(folder_name,"0");
		strcat(folder_name,xx);
	}
	
	// check sample folder exist ------------------------------------------
	ffcd(""); ffcd("SAMPLE");
	if(TRUE == fat_loadFileDataFromDir(folder_name))
	{
		ffclose();
		return TRUE;
	}
	else 
	{
		ffclose();
		return FALSE;
	}
}

//*************************************************************************
// check  PRBANK exist
//*************************************************************************
Bool check_PRBANK_exist(uint8_t PRBANK_NR)
{
	// build folder name ----------------------------------
	char folder_name[9];
	char xx[2];
	
	strcpy(folder_name,"PRBANK");
	
	itoa(PRBANK_NR,xx,10);
	if (PRBANK_NR > 9)
	{
		strcat(folder_name,xx);
	}
	else
	{
		strcat(folder_name,"0");
		strcat(folder_name,xx);
	}
	
	// check PRESET folder exist ------------------------------------------
	ffcd(""); ffcd("PRESET");
	if(TRUE == fat_loadFileDataFromDir(folder_name))
	{
		ffclose();
		return TRUE;
	}
	else 
	{
		ffclose();
		return FALSE;
	}
}

//*************************************************************************
// read select sample_name
//*************************************************************************
void read_samplename(uint8_t osc_nr)
{
	char filename[13] = "\0";
	uint8_t sel_entry = sel_sample %7;
	uint8_t i;
	char x;
	uint8_t iy = 0;
	for (i = 0; i < 8; i++)
	{
		x = filename_buffer[(sel_entry * 11)+i];
		
		if (x != ' ')		// check space. this is end of name
		{
			filename[iy]=x;
			iy++;
		}
	}
	
	strcat(filename,".WAV");
	strcpy(sample_name,filename);
	if (osc_nr == 0)
	{
		strcpy(Gui.file_name1,filename);
	}
	else{strcpy(Gui.file_name2,filename);}
}

//*************************************************************************
// get Sample Directory
//*************************************************************************
void getDir(char *directory, char *folder)
{
	char *dir = directory;
	char *fol = folder;
	ffcd("");
	ffcd(dir);ffcd(fol);
	file_numbers = 0;
	file_numbers = 50;
}

//*************************************************************************
// get Folder Name
//*************************************************************************
char *get_foldername(char *folder, uint8_t banknum)
{
	char *string = "";
	itoa(banknum, string, 10);
	if (banknum > 9)
	{
		strcat(folder,string);
	}
	else
	{
		strcat(folder,"0");
		strcat(folder,string);
	}
	strcpy(string,"");
	return folder;
}





//*************************************************************************
// SD CARD detect (Polling)
//*************************************************************************
void sdcard_status(void)
{
	static uint8_t sdcard_status_old;
	static uint8_t sdcard_status_new;
	
	// SDCARD is plugged --------------------------------------------------
	if (!(SDCARD_detect))	// PortF1 = low (sd card plugged)
	{
		sdcard_status_new = 1;
		if (sdcard_status_new != sdcard_status_old)
		{
			init_SDCARD();
			sd_disable = 0;
			draw_sdcard_symbol(255,3);
			sdcard_status_old = sdcard_status_new;
		}
	}
	// SDCARD not plugged -------------------------------------------------
	else
	{
		sdcard_status_new = 0;
		if (sdcard_status_new != sdcard_status_old)
		{
			if (sdcard_status_new != sdcard_status_old)
			{
				sd_disable = 1;
				draw_sdcard_symbol(255,3);
				sdcard_status_old = sdcard_status_new;
			}
		}
	}	
}

//*************************************************************************
// Save Osc_waveform
//*************************************************************************
void Save_osc_waveform(void)
{
	// set Preset Folder --------------------------------------------------
	ffcd("");
	static uint8_t number;
	number++;
	char number_str[3];
	
	itoa(number, number_str, 10);
	
	{
		// created new sample file
		if(MMC_FILE_CREATED == ffopen(number_str,'c'))
		{
			//uint8_t sample;
			for (uint8_t loop = 0; loop < 151; loop++)
			{
				ffwrite(~Scope.adc_buffer[loop]);
			}
						
		ffclose();
		}
	}
	
}

//*************************************************************************
// load SHAPE.bin
//*************************************************************************
void load_shape_bin(void)
{
	// stop osc and audio interrupt ---------------------------------------
	TCC0.INTCTRLA = 0b00000000;			// run 40KHz Audio Interrupt
	
	__uint24 offset = 0xEF200;
	hugemem_ptr_t p = BOARD_EBI_SRAM_BASE + offset;
	
	ffcd(""); ffcd("SYSTEM");
	if( MMC_FILE_OPENED == ffopen("SHAPE.bin",'r'))
	{
		uint16_t file_pointer = 0;
		
		ffseek(file_pointer);
		
		for (file_pointer = 0; file_pointer < 2560 ; file_pointer++)
		{
			hugemem_write8(p + file_pointer, ffread());			
		}
		
		
	}
	// Error: "File not found.." ------------------------------------------
	else
	{
		// print error massage
		fillrectangle(30,38,280,145,color_blue1);
		rectangle(33,42,99,243,color_white);
		//fillrectangle(139,42,180,65,color_blue1);
		print_string("Error!",135,50,6,color_white);
		print_string("File 'Shape.bin' not found",47,80,6,color_white);
		ffclose();
		_delay_ms(1000);
	}
	
	ffclose();
	
	// run osc and audio interrupt ----------------------------------------
	TCC0.INTCTRLA = 0b00000011;			// run 40KHz Audio Interrupt
}