//*************************************************************************
//
//           SainSmart TFT 3.2       grafic funktion
//
//*************************************************************************

#include "globals.h"
#include "display/displ_funktion.h"
#include "lfo.h"
#include "wavetable.h"
#include "filter.h"
#include "keyquery.h"
#include "miniscope.h"
#include "oscillators.h"
#include "midi.h"
#include "gui.h"
#include "SDCARD/sd_card.h"
#include "patch.h"
#include "init_system.h"

//*************************************************************************
// draw OSC waveform page
//*************************************************************************
void draw_osc_waveform_page(uint8_t osc_nr, uint8_t page_nr)
{
	// clear draw-area
	fillrectangle(0,25,319,165,color_black);
	
	// draw normal user waveforms -------------------------------------
	for (uint8_t row = 0; row < 2; row++)
	{
		uint8_t xwave_nr = page_nr * 8;
		uint16_t xpos = 0;
		uint8_t ypos = 60;
		uint8_t col = 0;
		
		for (uint8_t row = 0; row < 2; row++)		// draw 2 rows
		{
			for (uint8_t i = col; i < 4; i++)		// draw 4 waves per row
			{
				xpos = 8 + (i *80);
				
				draw_waveform_xram(osc_nr, xwave_nr, xpos, ypos);
				xwave_nr++;
			}
			ypos += 70;
			col = 0;
		}
	}
		
	// draw wave-numbers ------------------------------------------------------
	uint8_t xwave_nr = page_nr * 8;
	uint16_t xpos;
	uint8_t ypos = 31;
	
	for (uint8_t row = 0; row < 2; row++)	// draw 2 rows
	{
		for (uint8_t i = 0; i < 4; i++)		// draw 4 waves per row
		{
			xpos = 55 + (i *80);
			draw_3digit_with(xwave_nr,xpos,ypos,2,color_yello);
			xwave_nr++;
		}
		ypos = 101;
	}
}

//*************************************************************************
// draw wave from xram
//*************************************************************************
void draw_waveform_xram(uint8_t osc_nr, uint8_t wave_nr, uint16_t xpos, uint8_t ypos)
{
	__uint24 offset;
	
	if (osc_nr == 0)
	{
		offset = 0xEFC00;
	}
	else offset = 0xF7C00;
		
	uint16_t x1 = xpos;
	uint8_t y1 = ypos ;
	uint16_t x2 = xpos;
	uint8_t y2 = ypos;
	uint8_t addr_offset = 0;
	
	hugemem_ptr_t p = BOARD_EBI_SRAM_BASE + offset + (wave_nr * 256);
	
	for (uint8_t i = 0; i < 22; i++)
	{
		y1 = hugemem_read8 (p + addr_offset); // read sample
		y1 = (y1 / 5);
		y1 = ypos + 25 - y1;
		draw_line(x1,y1,x2,y2,color_white);
		x2=x1;y2=y1;
		
		addr_offset += 12;
		x1 += 3;
	}
	// draw last-wave point
	draw_line(x1-2,y1,x2+1,ypos,color_white);
}

//*************************************************************************
// draw select orange rectangle
//*************************************************************************
void draw_lfo_rectangle(uint8_t wave_nr)
{
	// draw gray rectangles -----------------------------------------------
	uint16_t pic_xoffset = 0;
	uint16_t x1 = 13;
	uint8_t  y1 = 25;
	uint8_t  height = 42;
	uint16_t widht = 43;
	
	for (uint8_t count = 0; count < 3; count++)
	{
		for (uint8_t i = 0; i < 6; i++)
		{
			rectangle(x1+pic_xoffset,y1,height,widht,color_gray2);
			rectangle(x1+pic_xoffset-1,y1-1,height+2,widht+2,color_black);
			pic_xoffset += 50;
		}
		
		pic_xoffset = 0;
		x1 = 13;
		y1 = y1 + 48;
	}
	
	// draw seleced (yello) rectangle -------------------------------------
	
	y1 = 25 + (48 * (wave_nr / 6));
	pic_xoffset = (wave_nr % 6) * 50;
	
	x1 = 13;
	height = 42;
	widht = 43;

	// draw select waveform
	rectangle(x1+pic_xoffset,y1,height,widht,color_orange);
	rectangle(x1+pic_xoffset-1,y1-1,height+2,widht+2,color_orange);
}

//*************************************************************************
// draw select rectangle OSC1+2 wave-mode
//*************************************************************************
void draw_osc_rectangle(uint8_t wave_nr)
{
	uint8_t yrow = 0;
	for (uint8_t row = 0; row < 2; row++)
	{
		// draw gray rectangles in row
		int pic_xoffset = 0;
		int x1 = 3;
		int y1 = 28 + yrow;
		int height = 63;
		int widht = 73;
		for (uint8_t i = 0; i < 4; i++)
		{
			rectangle(x1+pic_xoffset,y1,height,widht,color_gray2);
			rectangle(x1+pic_xoffset-1,y1-1,height+2,widht+2,color_black);
			pic_xoffset += 80; // next rectangle
		}
		yrow += 70;  // next row
	}
	
	
	// draw orange rectangle for select waveform
	uint8_t sel_wave;
	sel_wave = wave_nr % 8;
	
	if (sel_wave < 4)
	{	
		int pic_xoffset = (sel_wave * 40);
		int x1 = 3 + pic_xoffset;
		int y1 = 28;
		int height = 63;
		int widht = 73;
		// draw select waveform
		rectangle(x1+pic_xoffset,y1,height,widht,color_orange);
		rectangle(x1+pic_xoffset-1,y1-1,height+2,widht+2,color_orange);
		
	}
	else
	{
		int pic_xoffset = ((sel_wave -4) * 40);
		int x1 = 3 + pic_xoffset;
		int y1 = 28+70;
		int height = 63;
		int widht = 73;
		// draw select waveform
		rectangle(x1+pic_xoffset,y1,height,widht,color_orange);
		rectangle(x1+pic_xoffset-1,y1-1,height+2,widht+2,color_orange);
	}
}


//-------------------------------------------------------------------------
// Draw LoopA Line
//-------------------------------------------------------------------------
void draw_loopA_line(void)
{
	// delete old loop-line
	hline(30,28,255,color_black);
	
	// draw loop_line
	hline(cursor_loopA+31,28,cursor_loopB-cursor_loopA,color_white);
}
//-------------------------------------------------------------------------
// Draw LoopB Line
//-------------------------------------------------------------------------
void draw_loopB_line(void)
{
	// delete old loop-line
	hline(30,28,255,color_black);
	
	// draw loop_line
	hline(cursor_loopA+31,28,cursor_loopB-cursor_loopA,color_white);
}


//*************************************************************************
// draw enveloop line
//*************************************************************************
void draw_adsr_line(void)
{
	// clear window-area
	fillrectangle(3,25,250,165,color_black);

	// draw current adsr line ---------------------------------------------
	// set variable
	uint8_t xplot1 = 10+(Enc.value_[1]/3);
	uint8_t yplot2_sustain = (85+45) - ((Enc.value_[3]*85)/127);
	uint8_t xplot1_decay = Enc.value_[2]/2;
	
	// draw little rectangle for ADSR points
	fillrectangle(xplot1-3,42,xplot1+5,50,color_blue1);
	fillrectangle(xplot1 + xplot1_decay -3,yplot2_sustain-3,xplot1 + xplot1_decay +5,yplot2_sustain + 5,color_blue1);
	fillrectangle(167,yplot2_sustain-3,175,yplot2_sustain+5,color_blue1);
	fillrectangle(167 + ((Enc.value_[4]*50)/127),127,(174 + ((Enc.value_[4]*50) /127)),135,color_blue1);
	
	// draw Attack Line
	draw_width_line(10,130,xplot1,45,color_orange);
	
	// draw Decay Line
	draw_width_line(xplot1,45,xplot1 + xplot1_decay,yplot2_sustain,color_orange);
		
	// draw sustain line
	draw_width_line(xplot1 + xplot1_decay,yplot2_sustain,170,yplot2_sustain,color_orange);
	
	// draw relaese line
	draw_width_line(170,yplot2_sustain,(170 + ((Enc.value_[4]*50)/127)),130,color_orange);
	
	// draw vertikal point-line for relaese
	for (uint8_t i = 34; i < 131; i++)
	{
		//LCD_Plot_Pixel(105,i,1);
		pixel(170,i,color_white);
		i++;
		i++;
	}
	
	// draw value in adsr window
	draw_3digit_with(Enc.value_[1],xplot1-9,33,1,color_blue1);
	draw_3digit_with(Enc.value_[2],xplot1 + xplot1_decay+5,yplot2_sustain+6,1,color_blue1);
	draw_3digit_with(Enc.value_[3],147,yplot2_sustain-11,1,color_blue1);
	draw_3digit_with(Enc.value_[4],(173 + ((Enc.value_[4]*50) /127)),128,1,color_blue1);
}

//*************************************************************************
// draw wave from ext. sram
//*************************************************************************
void draw_wave_sram(void)
{
	// clear wave screen
	fillrectangle(3,24,317,166,color_black);
	
	hugemem_ptr_t p = BOARD_EBI_SRAM_BASE;
		
	for (uint8_t i = 0; i < 255 ; i++)
	{
		uint8_t hoehe_h = hugemem_read8(p + i + 0xFFE00);
		uint8_t hoehe_x = hugemem_read8(p + i + 0xFFF00);
		vline(i+31,94-hoehe_h, hoehe_x, color_orange);
	}		
}

//*************************************************************************
// draw cursor and move sample
//*************************************************************************
void draw_cursor_wave(void)
{
	/*
	hugemem_ptr_t p = BOARD_EBI_SRAM_BASE;
	uint8_t y0;
	uint8_t hight;
	
	// if cursor-addr <= file_lenght than move wave
	if (((uint32_t)new_cursor_pos * Cursor.zoom_addr_offset + wave_offset) <= file_lenght)
	{
		// move wave to right if cursor-pos = 0 ---------------------------
		if (new_cursor_pos == 0)
		{
			if (wave_offset < (Cursor.zoom_addr_offset * 8))
			{
				wave_offset = 0;
			}
			else
			{
				wave_offset = wave_offset - (Cursor.zoom_addr_offset * 8);
				cursor_wave_address = (((uint32_t)new_cursor_pos * Cursor.zoom_addr_offset) + wave_offset);
				if (cursor_wave_address > file_lenght)
				{
					cursor_wave_address = file_lenght;
				}
				draw_zoom_wave();
			}
			new_cursor_pos = 0;
			
			if (wave_offset > 0)
			{
				Enc.value_[1] = 1;
				Enc.value_[1]_old = 1;
			}
			else
			{
				Enc.value_[1] = 0;
				Enc.value_[1]_old = 0;
			}
		}
		
		// move wave to left if cursor-pos = 255 --------------------------
		else if (new_cursor_pos == 255)
		{
			wave_offset = wave_offset + (Cursor.zoom_addr_offset * 8);
			cursor_wave_address = (((uint32_t)new_cursor_pos * Cursor.zoom_addr_offset) + wave_offset);
			if (cursor_wave_address > file_lenght)
			{
				cursor_wave_address = file_lenght;
			}
			draw_zoom_wave();
			
			new_cursor_pos = 255;
			osc1_old_cursor_pos = 255;
			Enc.value_[1] = 254;
			Enc.value_[1]_old = 254;
		}
		
		// restore cursor background
		vline(osc1_old_cursor_pos + 31, 34, 120, color_black);
		y0 = hugemem_read8(p+osc1_old_cursor_pos + 0xFFE00);
		hight = hugemem_read8(p+osc1_old_cursor_pos + 0xFFF00);
		vline(osc1_old_cursor_pos + 31, 94 - y0, hight, color_orange);
		
		// draw new cursor position
		vline(new_cursor_pos + 31,34,120,color_white);
		
		// save cursor position
		osc1_old_cursor_pos = new_cursor_pos;
		
		// calc cursor sample-addr
		if (zoom_level == 0)
		{
			wave_offset = 0;
		}
		cursor_wave_address = (((uint32_t)new_cursor_pos * Cursor.zoom_addr_offset) + wave_offset);
		if (cursor_wave_address > file_lenght)
		{
			cursor_wave_address = file_lenght;
		}
	}
	
	// if cursor-addr > file_lenght than set cursor to eof
	else
	{
		cursor_pos_new = (uint32_t)((file_lenght-wave_offset) / Cursor.zoom_addr_offset);
		draw_encoder_position(2,cursor_pos_new);
		
		// restore cursor background
		vline(osc1_old_cursor_pos + 31, 34, 120, color_black);
		y0 = hugemem_read8(p+osc1_old_cursor_pos + 0xFFE00);
		hight = hugemem_read8(p+osc1_old_cursor_pos + 0xFFF00);
		vline(osc1_old_cursor_pos + 31, 94 - y0, hight, color_orange);
		
		// draw new cursor position
		vline(cursor_pos_new + 31,34,120,color_white);
		
		// save cursor position
		osc1_old_cursor_pos = cursor_pos_new;
		Enc.value_[1] = cursor_pos_new;
		Enc.value_[1]_old = cursor_pos_new;
		Enc.status_[1] = 0;
	}
	*/
}


//*************************************************************************
// draw zoom sample
//*************************************************************************
void draw_zoom_wave(void)
{
	/*
	// clear wave-window --------------------------------------------------
	fillrectangle(4,25,315,165,color_black);
	
	// calc addr-offset for zoom ------------------------------------------
	switch (zoom_level)
	{
		case 0: Cursor.zoom_addr_offset = 4096;
		break;
		case 1: Cursor.zoom_addr_offset = 2048;
		break;
		case 2: Cursor.zoom_addr_offset = 1024;
		break;
		case 3: Cursor.zoom_addr_offset = 512;
		break;
		case 4: Cursor.zoom_addr_offset = 256;
		break;
		case 5: Cursor.zoom_addr_offset = 128;
		break;
		case 6: Cursor.zoom_addr_offset = 64;
		break;
		case 7: Cursor.zoom_addr_offset = 32;
		break;
		case 8: Cursor.zoom_addr_offset = 16;
		break;
		case 9: Cursor.zoom_addr_offset = 8;
		break;
		case 10: Cursor.zoom_addr_offset = 4;
		break;
		case 11: Cursor.zoom_addr_offset = 2;
		break;
		case 12: Cursor.zoom_addr_offset = 1;
		break;
	}
	
	// check cursor position out of range 
	if ((cursor_wave_address < (new_cursor_pos * Cursor.zoom_addr_offset)) || (cursor_wave_address == 0))
	{
		wave_offset = 0;
		new_cursor_pos = (cursor_wave_address/Cursor.zoom_addr_offset);
		osc1_old_cursor_pos = new_cursor_pos;
		Enc.value_[1] = new_cursor_pos;
		Enc.status_[1] = 0;
		Enc.status_[2] = 0;
		screen_startaddr = wave_offset;
		screen_endaddr = screen_startaddr+(Cursor.zoom_addr_offset * new_cursor_pos);
	}

	else
	{
		wave_offset = cursor_wave_address-(Cursor.zoom_addr_offset * new_cursor_pos);		
	}


	
	// draw zoom sample ---------------------------------------------------
	hugemem_ptr_t p = BOARD_EBI_SRAM_BASE;
	p += + 0x8000 + wave_offset;
	
	// init xy position ---------------------------------------------------
	int16_t x1 = 31;
	uint8_t y1;
	int16_t x2 = 31;
	uint8_t y2 = (255-hugemem_read8(p));
	
	// draw middle h_line
	hline(31,94,255,color_orange);
	
	// draw shrinked wave -------------------------------------------------
	uint8_t hoehe_l;
	uint8_t hoehe_h;
	uint8_t hoehe_x;
	uint8_t yplot = 0;
	x1 = 0;
	y1 = 44;	// waveform centerline of Screen
	
	for (uint8_t i1 = 0; i1 < 255; i1++)
	{
		uint8_t sample_h = 127;
		uint8_t sample_l = 127;
		
		for(uint16_t i2 = 0; i2 < Cursor.zoom_addr_offset;i2++)
		{
			// read sample
			yplot = 255-hugemem_read8(p+i2);
			// if screen-plot over wave-end than draw zero-line
			if ((p+i2-BOARD_EBI_SRAM_BASE) > file_lenght-1)
			{
				//yplot = 127;
				goto zerro_line;
				
			}
			// calculate positive and negative wave line
			if (yplot > 127)
			{
				if (sample_h < yplot)
				{
					sample_h = yplot;
				}
			}
			else
			{
				if (sample_l > yplot)
				{
					sample_l = yplot;
				}
			}
		}
		// calc Y-High
		sample_h = sample_h >> 1;
		sample_l = sample_l >> 1;
		
		hoehe_h = sample_h - 64;
		hoehe_l = 64 - sample_l;
		hoehe_x = hoehe_l + hoehe_h;
		// draw wave-line
		vline(x1+31,94-hoehe_h, hoehe_x, color_orange);
		// save line for restore cursor background
		hugemem_write8(BOARD_EBI_SRAM_BASE + x1 + 0xFFE00,hoehe_h);
		hugemem_write8(BOARD_EBI_SRAM_BASE + x1 + 0xFFF00,hoehe_x);
		// inc x-position on screen
		x1++;
		// next sample_adr.
		p = p + Cursor.zoom_addr_offset;
	}
	
	zerro_line:
	
	for (x1; x1 < 255; x1++)
	{
		// draw zerro line
		vline(x1+31,94, 0, color_orange);
		// save line for restore cursor background
		hugemem_write8(BOARD_EBI_SRAM_BASE + x1 + 0xFFE00,0);
		hugemem_write8(BOARD_EBI_SRAM_BASE + x1 + 0xFFF00,0);
	}
	
	// draw new cursor position
	vline(new_cursor_pos + 31,34,120,color_white);
	*/
}

//*************************************************************************
// draw wave to intern ram
//*************************************************************************
void draw_wave_into_ram(void)
{
	/*
	zoom_level = 0;
	addoffset = 2855;
	//file_lenght = sampling_wr_addr;
	
	// check cursor position out of range 
	if ((cursor_wave_address < (new_cursor_pos*addoffset)) || (cursor_wave_address == 0))
	{
		wave_offset = 0;
		new_cursor_pos = (cursor_wave_address/addoffset);
		osc1_old_cursor_pos = new_cursor_pos;
		Enc.value_[1] = new_cursor_pos;
		Enc.status_[1] = 0;
		Enc.status_[2] = 0;
		screen_startaddr = wave_offset;
		screen_endaddr = screen_startaddr+(addoffset*new_cursor_pos);
	}

	else
	{
		wave_offset = cursor_wave_address-(new_cursor_pos*addoffset);		
	}


	
	// draw zoom sample ---------------------------------------------------
	hugemem_ptr_t p = BOARD_EBI_SRAM_BASE;
	p += wave_offset;
	
	// init xy position ---------------------------------------------------
	int16_t x1 = 31;
	uint8_t y1;
	int16_t x2 = 31;
	uint8_t y2 = (255-hugemem_read8(p));
	
	// draw shrinked wave -------------------------------------------------
	uint8_t hoehe_l;
	uint8_t hoehe_h;
	uint8_t hoehe_x;
	uint8_t yplot = 0;
	x1 = 0;
	y1 = 44;	// waveform centerline of Screen
	
	for (uint8_t i1 = 0; i1 < 255; i1++)
	{
		uint8_t sample_h = 127;
		uint8_t sample_l = 127;
		
		for(uint16_t i2 = 0; i2 < addoffset;i2++)
		{
			// read sample
			yplot = 255-hugemem_read8(p+i2);
			// if screen-plot over wave-end than draw zero-line
			if ((p+i2-BOARD_EBI_SRAM_BASE) > file_lenght-1)
			{
				yplot = 127;
			}
			// calculate positive and negative wave line
			if (yplot > 128)
			{
				if (sample_h < yplot)
				{
					sample_h = yplot;
				}
			}
			else
			{
				if (sample_l > yplot)
				{
					sample_l = yplot;
				}
			}
		}
		// calc Y-High
		hoehe_l = 65-(sample_l >>1);
		hoehe_h = (sample_h-128) >>1;
		hoehe_x = hoehe_l+hoehe_h;
		// save line for restore cursor background
		hugemem_write8(BOARD_EBI_SRAM_BASE + x1 + 0xFFE00,hoehe_h);
		hugemem_write8(BOARD_EBI_SRAM_BASE + x1 + 0xFFF00,hoehe_x);
		// inc x-position on screen
		x1++;
		// next sample_adr.
		p = p + addoffset;
	}
	*/
}

//*************************************************************************
// calc Cursor Position on Screen
//*************************************************************************
void calc_cursorpos(void)
{
	if (zoom_level == 0)
	{
		cursor_pos_new  = 0;cursor_pos_old = 0;
	}
	else{cursor_pos_new  = 0;cursor_pos_old = 0;}
}

//*************************************************************************
// SCOPE
//*************************************************************************
void draw_scope_line(void)
{
	if (Scope.adc_trigger != 99)
	{
		return;
	}
	
	// refresh timer for scope page (255msec)
	if (Scope.scope_wait_flag == 1)
	{
		return;
	}
	
	// draw scopeline in page 7+11 -----------------------------------------
	if (page_nr == 11)
	{
		// init xy position
		uint16_t x1 = 9;
		uint8_t y1 = 0;
		uint16_t x2 = 10;
		uint8_t y2 = 0;
		int8_t sample = 0;
		uint8_t gain_temp = Scope.scope_gain_old << 1;			// scope_gain x2 faster for calc
		
		uint16_t x1_b = 9;
		uint8_t y1_b = 0;
		uint16_t x2_b = 10;
		uint8_t y2_b = 0;
		uint8_t gain_temp_b = Scope.scope_gain << 1;				// scope_gain x2 faster for calc
		
		// clear old Sample Point and draw new --------------------------------
		for (uint8_t loop = 0; loop < 150; loop++)
		{
			y1 = Scope.adc_buffer_old[loop];							// load sample
			sample = (int16_t)y1 -127;							// make signed sample
			sample = (int16_t)sample * gain_temp >> 8;			// scope_gain
			y1 = (uint8_t)(95 + (int16_t)sample);				// add dc_offset for lcd window
			if (y1 < 25)										// limit sample_value
			{
				y1 = 25;
			}
			if (y1 > 163)
			{
				y1 = 163;
			}
			
			y1_b = Scope.adc_buffer[loop];							// load sample
			sample = (int16_t)y1_b -127;						// make signed sample
			sample = (int16_t)sample * gain_temp_b >> 8;		// scope_gain
			y1_b = (uint8_t)(95 + (int16_t)sample);				// add dc_offset for lcd window
			if (y1_b < 25)										// limit sample_value
			{
				y1_b = 25;
			}
			if (y1_b > 163)
			{
				y1_b = 163;
			}
			if (loop > 0)
			{
				draw_line(x2,y1,x1,y2,color_black);				// draw black sample line
				draw_line(x2_b,y1_b,x1_b,y2_b,color_white);		// draw white sample line
			}
			
			x1 = x2;											// calc next xy points
			x2 = x2 +2;
			y2 = y1;
			x1_b = x2_b;										// calc next xy points
			x2_b = x2_b +2;
			y2_b = y1_b;
		}

		
		// draw scope raster
		pixel(9,95,color_orange); // hor. line
		pixel(34,95,color_orange);
		pixel(59,95,color_orange);
		pixel(84,95,color_orange);
		pixel(109,95,color_orange);
		pixel(134,95,color_orange);
		pixel(159,95,color_orange);
		pixel(184,95,color_orange);
		pixel(209,95,color_orange);
		pixel(234,95,color_orange);
		pixel(259,95,color_orange);
		pixel(284,95,color_orange);
		pixel(309,95,color_orange);
		pixel(159,26,color_orange); // ver.line2
		pixel(159,49,color_orange);
		pixel(159,72,color_orange);
		pixel(159,95,color_orange);
		pixel(159,118,color_orange);
		pixel(159,141,color_orange);
		pixel(159,164,color_orange);

	}
	
	// draw miniscope line ------------------------------------------------
	else if(page_nr == 0 || page_nr == 8)
	{
		if (save_flag == 0 && load_flag == 0 && delete_flag == 0 )
		{
			// init xy position
			uint16_t x1 = 251;
			uint8_t y1 = 0;
			uint16_t x2 = 251;
			uint8_t y2 = 0;
			int8_t sample = 0;
			uint8_t gain_temp = Scope.scope_gain_old << 1;			// scope_gain x2 faster for calc
			
			uint16_t x1_b = 251;
			uint8_t y1_b = 0;
			uint16_t x2_b = 251;
			uint8_t y2_b = 0;
			uint8_t gain_temp_b = Scope.scope_gain << 1;				// scope_gain x2 faster for calc
			
			// clear old Sample Point and draw new --------------------------------
			for (uint8_t loop = 0; loop < 150; loop++)
			{
				y1 = Scope.adc_buffer_old[loop];							// load sample
				sample = (int16_t)y1 -127;							// make signed sample
				sample = (int16_t)sample * gain_temp >> 10;			// scope_gain
				y1 = (uint8_t)(65 + (int16_t)sample);				// add dc_offset for lcd window
				if (y1 < 49)										// limit sample_value
				{
					y1 = 49;
				}
				if (y1 > 83)
				{
					y1 = 83;
				}
				
				y1_b = Scope.adc_buffer[loop];							// load sample
				sample = (int16_t)y1_b -127;						// make signed sample
				sample = (int16_t)sample * gain_temp_b >> 10;		// scope_gain
				y1_b = (uint8_t)(65 + (int16_t)sample);				// add dc_offset for lcd window
				if (y1_b < 49)										// limit sample_value
				{
					y1_b = 49;
				}
				if (y1_b > 83)
				{
					y1_b = 83;
				}
				if (loop > 0)
				{
					draw_line(x2,y1,x1,y2,color_gray2);				// draw black sample line
					draw_line(x2_b,y1_b,x1_b,y2_b,color_white);		// draw white sample line
				}
				
				x1 = x2;											// calc next xy points
				x2 = x2 +2;
				y2 = y1;
				x1_b = x2_b;										// calc next xy points
				x2_b = x2_b +2;
				y2_b = y1_b;
				loop += 5;
				
				
				
				
				
				/*
				// draw scope raster
				pixel(275,49,color_orange); // vertikal line
				pixel(275,57,color_orange);
				pixel(275,65,color_orange);
				pixel(275,73,color_orange);
				pixel(275,81,color_orange);
				pixel(251,65,color_orange); // horizontal line2
				pixel(263,65,color_orange); // horizontal line2
				pixel(287,65,color_orange); // horizontal line2
				pixel(299,65,color_orange); // horizontal line2
				*/
				
			}
		}
	}
		
		// draw miniscope line on Osc1+2 Page -----------------------------
		else if(page_nr == 1 || page_nr == 2)
		{
			if (save_flag == 0 && load_flag == 0 && delete_flag == 0 )
			{
				// init xy position
				uint16_t x1 = 251;
				uint8_t y1 = 0;
				//uint16_t x2 = 251;
				uint16_t x2 = 230;
				uint8_t y2 = 0;
				int8_t sample = 0;
				uint8_t gain_temp = 45;			// scope_gain x2 faster for calc
				
				//uint16_t x1_b = 251;
				uint16_t x1_b = 230;
				uint8_t y1_b = 0;
				//uint16_t x2_b = 251;
				uint16_t x2_b = 230;
				uint8_t y2_b = 0;
				uint8_t gain_temp_b = 45;				// scope_gain x2 faster for calc
				
				// clear old Sample Point and draw new --------------------------------
				for (uint8_t loop = 0; loop < 150; loop++)
				{
					y1 = Scope.adc_buffer_old[loop];					// load sample
					sample = (int16_t)y1 -127;							// make signed sample
					sample = (int16_t)sample * gain_temp >> 8;			// scope_gain
					
					//y1 = (uint8_t)(65 + (int16_t)sample);				// add dc_offset for lcd window
					y1 = (uint8_t)(65 +72 + (int16_t)sample);	
					
					if (y1 < (44+72))									// limit sample_value
					{
						y1 = (44+72);
					}
					if (y1 > (88+72))
					{
						y1 = (88+72);
					}
					
					y1_b = Scope.adc_buffer[loop];						// load sample
					sample = (int16_t)y1_b -127;						// make signed sample
					sample = (int16_t)sample * gain_temp_b >> 8;		// scope_gain
					//y1_b = (uint8_t)(65 + (int16_t)sample);			// add dc_offset for lcd window
					y1_b = (uint8_t)(65 +72 + (int16_t)sample);
					
					if (y1_b < (44+72))									// limit sample_value
					{
						y1_b = (44+72);
					}
					if (y1_b > (88+72))
					{
						y1_b = (88+72);
					}
					if (loop > 0)
					{
						draw_line(x2,y1,x1,y2,color_black);				// draw black sample line
						draw_line(x2_b,y1_b,x1_b,y2_b,color_white);		// draw white sample line
					}
					
					x1 = x2;											// calc next xy points
					x2 = x2 +2;
					y2 = y1;
					x1_b = x2_b;										// calc next xy points
					x2_b = x2_b +2;
					y2_b = y1_b;
					loop += 3;
					
					
					
					
					
					/*
					// draw scope raster
					pixel(275,49,color_orange); // vertikal line
					pixel(275,57,color_orange);
					pixel(275,65,color_orange);
					pixel(275,73,color_orange);
					pixel(275,81,color_orange);
					pixel(251,65,color_orange); // horizontal line2
					pixel(263,65,color_orange); // horizontal line2
					pixel(287,65,color_orange); // horizontal line2
					pixel(299,65,color_orange); // horizontal line2
					*/
					
				}
			}
	}
	
	
	
	// save new buffer values and new gain_value
	for (uint8_t i = 0; i < 150; i++)
	{
		Scope.adc_buffer_old[i] = Scope.adc_buffer[i];
	}
	Scope.scope_gain_old = Scope.scope_gain;
	
	
	// set Scope Timer Flags ------------------------------------------
	soft_timer0 = 0;
	soft_timer0_flag = 0;
	Scope.adc_flag = 0;
	
	// clear trigger-flag for new sampling --------------------------------
	Scope.adc_trigger = 0;
	// set scope wait flag and wait time for new sample (255msec)
	Scope.scope_wait_flag = 1;
	Scope.scope_wait_time = 50;
}

//*************************************************************************
// draw VU-Meter and sampling curve
//*************************************************************************
void draw_sampling_curve(void)
{
	// timer-flag for redraw the vu-meter
	if (Gui.time1_flag == 1)
	{
	
		
	uint8_t x1 = 39;
	uint8_t x2 = 59;
	uint8_t y1 = 36;
	uint8_t y2 = 46;
	uint8_t offset = 12*9;
	
	LEDBAR.old_sample = 0;
		
	// calc positv and negativ sample value -------------------------------
	for (uint8_t loop = 1; loop <=150; loop++)
	{
		LEDBAR.new_sample = Scope.adc_buffer[loop];				// load sample
		
		if (LEDBAR.new_sample > 127)
		{
			LEDBAR.new_sample = LEDBAR.new_sample - 127;
		}
		else
		{
			LEDBAR.new_sample = 127 - LEDBAR.new_sample;
		}
		
		if (LEDBAR.new_sample > LEDBAR.old_sample)
		{
			LEDBAR.old_sample = LEDBAR.new_sample;
		}
	}
	
	
	// set bar ------------------------------------------------------------	
	LEDBAR.new_Led_pos = 0;
	
	// set LED-Nr.
	if (LEDBAR.old_sample >= 8)   {LEDBAR.new_Led_pos = 1;}		// -24dB
	if (LEDBAR.old_sample >= 11)  {LEDBAR.new_Led_pos = 2;}		// -21dB  
	if (LEDBAR.old_sample >= 16)  {LEDBAR.new_Led_pos = 3;}		// -18dB  
	if (LEDBAR.old_sample >= 23)  {LEDBAR.new_Led_pos = 4;}		// -15dB 
	if (LEDBAR.old_sample >= 32)  {LEDBAR.new_Led_pos = 5;}		// -12dB
	if (LEDBAR.old_sample >= 45)  {LEDBAR.new_Led_pos = 6;}		//  -9dB
	if (LEDBAR.old_sample >= 64)  {LEDBAR.new_Led_pos = 7;}		//  -6dB
	if (LEDBAR.old_sample >= 91)  {LEDBAR.new_Led_pos = 8;}		//  -3dB
	if (LEDBAR.old_sample >= 113) {LEDBAR.new_Led_pos = 9;}		//  -1dB
	if (LEDBAR.old_sample >= 119) {LEDBAR.new_Led_pos = 10;}	//  -0dB
	
	// calc peek_pos ------------------------------------------------------
	if (LEDBAR.new_Led_pos >= LEDBAR.Peak_pos)
	{
		LEDBAR.Peak_pos = LEDBAR.new_Led_pos;
		LEDBAR.Peak_hold = 20;
	}
	else
	{
		if (LEDBAR.Peak_hold > 0)
		{
			LEDBAR.Peak_hold--;
		}
		else
		{
			LEDBAR.Peak_pos = 0;			
		}
	}
	
	if (LEDBAR.new_Led_pos >= LEDBAR.Led_pos)
	{
		LEDBAR.Led_pos = LEDBAR.new_Led_pos;
	}
	else
	{
		if (LEDBAR.Led_pos > 0)
		{
			LEDBAR.Led_pos--;
		}
	}
	
	// set LEDBAR ---------------------------------------------------------
	for (uint8_t count = 1; count <= 10; count++)
	{
		if (LEDBAR.Led_pos > count)
		{
			if (count <= 7)
			{
				fillrectangle(x1,y1+offset,x2,y2+offset,color_green);
			}
			else if (count == 8)
			{
				fillrectangle(x1,y1+offset,x2,y2+offset,color_yello);
			}
			else if (count == 9)
			{
				fillrectangle(x1,y1+offset,x2,y2+offset,color_orange);
			}
			else if (count == 10)
			{
				fillrectangle(x1,y1+offset,x2,y2+offset,color_red);
			}			
			offset -= 12;
		}
		else
		{
			if (count == LEDBAR.Peak_pos)
			{
				if (count <= 7)
				{
					fillrectangle(x1,y1+offset,x2,y2+offset,color_green);
				}
				else if (count == 8)
				{
					fillrectangle(x1,y1+offset,x2,y2+offset,color_yello);
				}
				else if (count == 9)
				{
					fillrectangle(x1,y1+offset,x2,y2+offset,color_orange);
				}
				else if (count == 10)
				{
					fillrectangle(x1,y1+offset,x2,y2+offset,color_red);
				}				
				offset -= 12;
			}
			
			else
			{
				fillrectangle(x1,y1+offset,x2,y2+offset,color_gray1);
				offset -= 12;
			}
			
		}
	}
	
			
	
	
	// draw sample curve and ram-address ----------------------------------
	
		if (Gui.Sam_rec_key_flag == 1 && Gui.Sam_rec_flag == 1)
		{

			// calc and print rec-time
			
			//uint8_t temp_vorkomma  = 0;//(sampling_wr_addr /40000);
			//uint8_t temp_nachkomma = 0;//((sampling_wr_addr % 40000) / 4000);
			
			//fillrectangle(177,50,219,63,color_black);
			//draw_3digit_without(0,176,50,6,color_white);
			//draw_char('.',199,50,6,color_white);
			//draw_3digit_without(0,190,50,6,color_white);
			// print KByte
			fillrectangle(175,140,199,152,color_black);
			uint16_t addr = (sampling_wr_addr >> 10);
			draw_3digit_with(addr,175,140,6,color_white);
			
			LEDBAR.sample_count++;
			if (LEDBAR.sample_count == 4)
			{
				
				if (LEDBAR.xpos > 156)
				{
					LEDBAR.xpos = 0;
				}
				
				uint8_t sample_temp = LEDBAR.old_sample;
				
				vline(LEDBAR.xpos+140,110-(sample_temp/3), sample_temp/3, color_yello);
				vline(LEDBAR.xpos+140,112, (sample_temp/8), color_orange);
				
				LEDBAR.sample_count = 0;
				LEDBAR.xpos += 2;
			}
		}
		else LEDBAR.sample_count = 0;

		
		// set Timer Flags for redrawing ----------------------------------
		soft_timer0 = 0;
		soft_timer0_flag = 0;
		Scope.adc_flag = 0;
		Scope.adc_trigger = 0;
		Scope.scope_wait_flag = 1;
		Scope.scope_wait_time = 75;
		
		Gui.time1_flag = 0;
		Gui.time1 = 75;
		
	}
}

//*************************************************************************
// init Menu Page
//*************************************************************************
void init_Menu_Page(void)
{
	fillrectangle(0,0,320,239,color_black);
	fillrectangle(0,168,320,240,color_gray4);
	
	// draw encoder knop 2-5
	draw_knop();	
	
	// init encoder
	Enc.maxval[0] = 14;			// max value for Encoder1 for Menu Page
	Enc.dynamic[0] = 0;

	Enc.maxval[1] = 127;
	Enc.dynamic[1] = 96;
	draw_encoder_position(2,0x00);
	// init encoder
	Enc.maxval[2] = 127;
	Enc.dynamic[2] = 96;
	draw_encoder_position(3,0x00);
	// init encoder
	Enc.maxval[3] = 127;
	Enc.dynamic[3] = 96;
	draw_encoder_position(4,0x00);
	// init encoder
	Enc.maxval[4] = 127;
	Enc.dynamic[4] = 96;
	draw_encoder_position(5,0x00);
	
}

//*************************************************************************
// print window header
//*************************************************************************
void print_window_header(char VH, char VL, char text_str[], char sub_nr[])
{
	//clear_WindowArea(color_black);
	fillrectangle(0,0,20,20,color_gray1);
	fillrectangle(21,0,320,20,VH,VL);
	print_string(text_str,27,4,6,color_white);
	uint8_t temp = page_nr +1;
	char str[4];
	itoa(temp, str, 10);
	uint8_t xpos;
	if (page_nr <= 8)
	{
		xpos = 7;
	}
	else xpos = 3;
	
	// print header text und Sub-Nr
	print_string(str,xpos,4,6,color_white);
	print_string(sub_nr,289,4,6,color_white);
	
	// draw sdcard symbol
	draw_sdcard_symbol(255,3);
	
	// draw gray Midi-Symbol
	draw_symbol(1,272,6,color_gray);
	rectangle(270,3,13,11,color_gray);
}

//*************************************************************************
// print window header
//*************************************************************************
void print_window_header_test(char VH, char VL, char text[], char sub_nr[])
{
	
	/*
	#include <avr/io.h><font></font>
	#include <avr/pgmspace.h><font></font>
	//menu items stored in flash<font></font>
	const uint8_t Menu1[] PROGMEM = "Menu1";<font></font>
	const uint8_t Menu2[] PROGMEM = "Menu2";<font></font>
	const uint8_t Menu3[] PROGMEM = "Menu3";<font></font>
	//array pointers to menu items<font></font>
	const uint8_t *Menu[] PROGMEM= {Menu1, Menu2, Menu3};<font></font>
	//menu item number<font></font>
	volatile uint8_t Item;<font></font>
	int main (void)<font></font>
	{<font></font>
		while (1) //Loop<font></font>
		{<font></font>
			//get menu Item value during some interrupt<font></font>
			LCDFstring((uint8_t*)pgm_read_word(&Menu[Item]));<font></font>
		}<font></font>
	}
	*/
	
	
	
	
	
	
	
	
	/*
	
	fillrectangle(0,0,20,20,color_gray1);
	fillrectangle(21,0,320,20,VH,VL);
	print_string(text,27,4,6,color_white);
	uint8_t temp = page_nr +1;
	char str[4];
	itoa(temp, str, 10);
	uint8_t xpos;
	if (page_nr <= 8)
	{
		xpos = 7;
	}
	else xpos = 3;
	
	// print header text und Sub-Nr
	print_string(str,xpos,4,6,color_white);
	print_string(sub_nr,289,4,6,color_white);
	
	// draw sdcard symbol
	draw_sdcard_symbol(255,3);
	
	// draw gray Midi-Symbol
	draw_symbol(1,272,6,color_gray);
	rectangle(270,3,13,11,color_gray);
	*/
}

//*************************************************************************
// print knop text
//*************************************************************************
void print_knop_text(char str1[], char str2[], char str3[], char str4[])
{
	fillrectangle(5,174,310,185,0x31, 0x86);		// clr knoptext
	
	uint16_t xpos1 = 37 - ((strlen(str1)*6)/2);
	uint16_t xpos2 = 117 - ((strlen(str2)*6)/2);
	uint16_t xpos3 = 197 - ((strlen(str3)*6)/2);
	uint16_t xpos4 = 277 - ((strlen(str4)*6)/2);
	print_string(str1,xpos1,174,2,color_white);
	print_string(str2,xpos2,174,2,color_white);
	print_string(str3,xpos3,174,2,color_white);
	print_string(str4,xpos4,174,2,color_white);
}



//*************************************************************************
// draw puple value frame in Osc Sub1 menu
//*************************************************************************
void draw_puple_frame(void)
{
	for (uint8_t row = 0; row < 3; row++)
	{
		for (uint8_t col = 0; col < 3; col++)
		{
			fillrectangle(68+(106*col),29+(29*row),94+(106*col),42+(29*row),color_purple);
			fillrectangle(67+(106*col),30+(29*row),95+(106*col),41+(29*row),color_purple);
		}
	}
}

//*************************************************************************
// draw orange selected frame in Osc Sub1 menu
//*************************************************************************
void draw_orange_frame(uint8_t par_nr_temp)
{
	// draw seleced frame
	if (par_nr_temp > 8)
	{
		par_nr_temp++;
	}
	uint8_t row = 106 * (par_nr_temp %3);
	uint8_t col = 29 * (par_nr_temp /3);
	rectangle(1 + row,22 + col,27,104,color_orange);
	rectangle(2 + row,23 + col,25,102,color_orange);
}





//*************************************************************************
// Menu load Sample-File
//*************************************************************************
void draw_load_wavefile(uint8_t osc_nr)
{
	/*
	// draw sample submenu 1
	Osc.wave_mode[osc_nr] = 1;
		
	// set parameter
	osc1_old_cursor_pos = 0;
	new_cursor_pos = 0;
	cursor_pos_new = 0;
	cursor_pos_old = 0;
	Enc.value_[1] = 0;
	Enc.value_old[1] = 0;
	Enc.status_[1] = 0;
	zoom_level = 0;
	loop_flag = 0;
	Osc1_ReTrig = 1;
	*/
	
	// stop sound_buffer Interrupt for fast loading
	TCC0.INTCTRLA = 0b00000000;
	// stop Osc_Render Task
	TCC1.INTCTRLA = 0b00000000;		
	
	// load wave file	
	load_wavefile(osc_nr);
		
	Gui.redraw_flag2 = FALSE;
	
	// run sound_buffer Interrupt
	TCC0.INTCTRLA = 0b00000011;
	// run Osc_Render Task
	TCC1.INTCTRLA = 0b00000010;
}

//*************************************************************************
// Menu save Preset-File
//*************************************************************************

void draw_save_presetfile(void)
{
	// exit save function
	if (save_flag == 1) 
	{
		save_preset_data();
		 current_preset = select_preset;
		 current_presetbank = select_presetbank;
		Patch.init_flag = 1;
		draw_menupage_soundprg();
		save_flag = 0;
		enable_encoder(1,1,1,1,1);
		enable_key(1,1,1,1,1,1,1,1,1,1,1,1,1);
	}
	else
	{
		// run save function and disaple enc+key
		enable_encoder(0,1,1,1,0);
		enable_key(0,0,0,0,0,0,0,1,0,0,0,1,0);
		
		// draw save info
		fillrectangle(30,38,280,145,color_blue1);
		rectangle(33,42,99,243,color_white);
		fillrectangle(139,42,180,65,color_blue1);
		print_string("Save",145,38,6,color_white);
		print_string("Select Name and Bank-No.",42,57,6,color_white);
		print_string("Name:",42,77,6,color_white);
		print_string("Bank:",42,97,6,color_white);
		print_string("Press save or cancel",62,120,6,color_white);
		// print old patch name
		fillrectangle(84,76,189,91,color_red);
		fillrectangle(83,77,190,90,color_red);
		get_preset_name();
		print_string(preset_name,86,77,6,color_white);
		// invert first char 
		fillrectangle(84,77,93,90,color_white);
		draw_char(preset_name[0],86,77,6,color_black);
		
		// print preset_bank
		fillrectangle(84,96,108,111,color_red);
		fillrectangle(83,97,109,110,color_red);
		draw_3digit_with(select_presetbank,85,97,6,color_white);
		
		print_knop_text("CHAR","POINT","BANK","");
		
		
		save_flag = 1;
		
		draw_encoder_position(2,0);
		draw_encoder_position(3,0);
		draw_encoder_position(4,select_presetbank);
		draw_encoder_position(5,0);
		Enc.value_[1] = 0;
		Enc.value_[2] = 0;
		Enc.value_[3] = select_presetbank;
		Enc.value_[4] = 0;
		Enc.value[1] = 0;
		Enc.value[2] = 0;
		Enc.value[3] = select_presetbank;
		Enc.value[4] = 0;
		Enc.value_old[1] = 0;
		Enc.value_old[2] = 0;
		Enc.value_old[3] = select_presetbank;
		Enc.value_old[4] = 0;
		Enc.maxval[1] = 45;
		Enc.maxval[2] = 12;
		Enc.maxval[3] = 98;
	}
}


//*************************************************************************
// get current preset name from Name.bin
//*************************************************************************
void get_preset_name(void)
{
	// set folders
	ffcd("");
	ffcd("PRESET");
	
	// convert sound_bank_nr in string
	char folder_temp[9];
	strcpy(folder_temp,"PRBANK");
	
	char xx[3];
	sound_bank = current_presetbank;

	itoa(sound_bank,xx,10);
	if (sound_bank > 9)
	{
		strcat(folder_temp,xx);
	}
	else
	{
		strcat(folder_temp,"0");
		strcat(folder_temp,xx);
	}
	
	ffcd(folder_temp);

	// read current preset-name from Namen.bin
	if( MMC_FILE_OPENED == ffopen("Name.bin",'r'))
	{
		ffseek(current_preset * 32);
		
		for (uint8_t i = 0; i < 13; i++)
		{
			char temp = ffread();
			if (temp == 0)
			{
				temp = ' ';
			}
			preset_name[i] = temp;			
		}
		ffclose();
	}
	// Error: File not found!
	else
	{
		fillrectangle(30,38,280,145,color_blue1);
		rectangle(33,42,99,243,color_white);
		fillrectangle(139,42,180,65,color_blue1);
		print_string("File 'Name.bin' not found!",42,85,6,color_white);
		_delay_ms(1000);
		
		ffclose();
		
		draw_menupage_soundprg();
	}
}

//*************************************************************************
// Menu delete Wave-File
//*************************************************************************
void draw_delete_wavefile(uint8_t osc_nr)
{
	if (delete_flag == 1)
	{
		fillrectangle(62,120,241,135,color_blue1);
		print_string("Delete File...",90,120,6,color_white);
		delete_wavefile(osc_nr);
		_delay_ms(350);
		delete_flag = 0;
		if (page_nr == 12)
		{
			draw_menupage_sampling();
		}
		else draw_menupage_osc(osc_nr);
	}
	else
	{
		// set delete flag
		delete_flag = 1;
		
		// draw delete info
		fillrectangle(30,38,280,145,color_blue1);
		rectangle(33,42,99,243,color_white);
		fillrectangle(139,42,180,65,color_blue1);
		print_string("Delete",145,38,6,color_white);
		print_string("Delete Wave-File on SDCARD",42,57,6,color_white);
		print_string("Name:             Bank:",42,85,6,color_white);
		print_string("Press delete or cancel",62,120,6,color_white);
		
		fillrectangle(229,84,255,98,color_red);
		fillrectangle(228,85,256,97,color_red);
		draw_3digit_with(Osc.wavebank[osc_nr],231,85,6,color_white);
		fillrectangle(84,84,151,99,color_red);
		fillrectangle(83,85,152,98,color_red);
		read_samplename(0);
		print_string(sample_name,86,85,6,color_white);
	}
	
}









//*************************************************************************
// set values in page "SOUND PRESET"
//*************************************************************************
// global var:
// preset_nr
void change_soundprg(void)
{	
	if (save_flag == 0)
	{
		// Encoder 2: select Sound Program --------------------------------
		if (Enc.status_[1] == 1)
		{
			select_preset = Enc.value_[1];
			draw_encoder_position(2,select_preset);

			// page scroll ? ----------------------------------------------
			static uint8_t old_page_nr = 0;
			uint8_t new_page_nr = select_preset / 7;
			if (new_page_nr != old_page_nr)
			{
				load_preset_list(new_page_nr);		// load new preset page
				checkmark_soundlist();				// set cursor
			}
			else
			{
				for (uint8_t i = 0; i <= 6; i++) // clear old rectangle
				{
					rectangle(3,26+(19*i),20,225,color_black);
					rectangle(4,27+(19*i),18,223,color_black);
				}
			}
			
			// set new orange rectangle in preset list --------------------
			uint8_t temp = ((select_preset % 7) * 19) + 26;
			rectangle(3,temp,19,225,color_orange);
			rectangle(4,temp+1,19,223,color_orange);
			
			// save page_nr
			old_page_nr = new_page_nr;
		}
			
		// Encoder 3: change presetbank -----------------------------------
		if (Enc.status_[2] == 1)
		{
			select_presetbank = Enc.value_[2];
			draw_encoder_position(3,select_presetbank);
			
			if (select_presetbank == current_presetbank)
			{
				select_preset = current_preset;
				Enc.value[1] = current_preset;
				Enc.value_old[1] = current_preset;
				sel_sound = current_preset;
				draw_encoder_position(2,current_preset);
			}
			// if another sound bank than select first sound prg
			else
			{
				select_preset = 0;
				Enc.value[1] = 0;
				Enc.value_old[1] = 0;
				Enc.status[1] = 0;
				sel_sound = 0;
				draw_encoder_position(2,0);
			}
			
			Sound_current_entry = 0;
			Sound_start_entry = 0;
			Sound_last_entry = 7;
			file_numbers = 0;
			
			// print preset-list in folder
			if (check_PRBANK_exist(select_presetbank) == TRUE)
			{
				load_preset_list(0);
				// draw cursor symbol in sound-program list
				uint8_t row = 26;
				uint8_t new_selEntry;
				new_selEntry = select_preset % 7;
				row = 26+((old_selEntry)*19);
				rectangle(3,row,19,225,color_black);
				rectangle(4,row+1,19,223,color_black);
				row = 26 + (new_selEntry*19);
				rectangle(3,row,19,225,color_orange);
				rectangle(4,row+1,19,223,color_orange);
				
				// set checkmark in soundprg list
				checkmark_soundlist();
				enable_encoder(1,1,1,1,1);
				Gui.file_not_found_flag = 0;
				
			}
			else
			{
				// print error..
				fillrectangle(3,25,230,163,color_black);
				print_info_box("No Folder..",40,75,1);
				Gui.file_not_found_flag = 1;
				enable_encoder(1,0,1,1,1);
			}
		}
		
		
	}
	else change_preset_save();
	
}

//*************************************************************************
// change encoder in save file function in Preset Menu
//*************************************************************************
void change_preset_save(void)
{
		uint8_t x_pos = Enc.value_[2];
		
		// select Char
		if (Enc.status_[1] == 1)
		{
			uint8_t ascii_char = pgm_read_byte (&(ascii_code[Enc.value_[1]]));
			draw_encoder_position(2, Enc.value_[1]);
			fillrectangle(85 + (x_pos*8),77,94 + (x_pos*8),90,color_white);
			draw_char(ascii_char,86 + (x_pos*8),77,6,color_black);
			
			preset_name[x_pos] = ascii_char;
		}
		
		// select point
		if (Enc.status_[2] == 1)
		{
			x_pos = Enc.value_[2];
			draw_encoder_position(3, x_pos);
			fillrectangle(84,76,189,91,color_red);
			fillrectangle(83,77,190,90,color_red);
			print_string(preset_name,86,77,6,color_white);
			fillrectangle(84 + (x_pos*8),77,93 + (x_pos*8),90,color_white);
			draw_char(preset_name[x_pos],86 + (x_pos*8),77,6,color_black);
		}
		
		// select Bank-No
		if (Enc.status_[3] == 1)
		{
			select_presetbank = Enc.value_[3];
			draw_encoder_position(4, select_presetbank);
			fillrectangle(84,96,108,111,color_red);
			fillrectangle(83,97,109,110,color_red);
			draw_3digit_with(select_presetbank,85,97,6,color_white);
		}

}















//*************************************************************************
// set Gloabal Parameters
//*************************************************************************
void change_system(void)
{
	if (Enc.status_[1] == 1)	// Select ---------------------------------
	{
		Gui.entry_system_page = Enc.value_[1];
		draw_encoder_position(2,Gui.entry_system_page);
		set_encoder_dyn(0,0,0,0);
		
		// redraw organge frame -------------------------------------------
		for (uint8_t col = 0; col < 5; col++)
		{
			for (uint8_t row = 0; row < 3; row++)
			{
				rectangle(1 + (106*row),22 + (29*col),27,104,color_black);
				rectangle(2 + (106*row),23 + (29*col),25,102,color_black);
			}
		}
		uint8_t row = 106 * (Gui.entry_system_page %3);
		uint8_t col = 29 * (Gui.entry_system_page /3);
		rectangle(1 + row,22 + col,27,104,color_orange);
		rectangle(2 + row,23 + col,25,102,color_orange);
		
		// set system values ----------------------------------------------
		switch(Gui.entry_system_page)
		{
			case(0): // Midi channel
			draw_encoder_position(3,Mid.channel_value);
			Enc.value_[2] = Mid.channel_value;
			set_encoder_max(14,16,0,0);
			set_encoder_dyn(96,96,0,0);
			break;
			
			case(1): // Encoder Dynamic
			draw_encoder_position(3,Enc.dynamic_);
			Enc.value_[2] = Enc.dynamic_;
			set_encoder_max(14,63,0,0);
			set_encoder_dyn(96,96,0,0);
			break;
			
			case(2): // Filtertyp
			draw_encoder_position(3,Gui.filtertyp);
			Enc.value_[2] = Gui.filtertyp;
			set_encoder_max(14,1,0,0);
			set_encoder_dyn(0,0,0,0);
			break;
			
			case(3): // Last Patch
			draw_encoder_position(3,Gui.last_patch);
			Enc.value_[2] = Gui.last_patch;
			set_encoder_max(14,1,0,0);
			set_encoder_dyn(0,0,0,0);
			break;
		}
	}
	
	
	if (Enc.status_[2] == 1)	// System values --------------------------
	{
		
		draw_encoder_position(3,Enc.value_[2]);
		
		uint8_t row; uint8_t col;
		
		switch(Gui.entry_system_page)
		{
			case(0): // Midi channel
			Mid.channel_value = Enc.value_[2];
			row = 0; col = 0;
			fillrectangle(68+(106*col),29+(29*row),94+(106*col),42+(29*row),color_purple);
			if (Mid.channel_value > 0)
			{
				draw_3digit_with(Mid.channel_value,70,29,6,color_white);
			}
			else print_string("Omni",68,32,2,color_white);
			break;
			
			case(1): // Encoder Dynamic
			Enc.dynamic_ = Enc.value_[2];
			row = 0; col = 1;
			fillrectangle(68+(106*col),29+(29*row),94+(106*col),42+(29*row),color_purple);
			draw_3digit_with(Enc.dynamic_,176,29,6,color_white);
			// save value into eeprom
			EEPWriteByte(ee_enc_dynamic, Enc.dynamic_);
			break;
			
			case(2): // Filtertyp
			Gui.filtertyp = Enc.value_[2];
			row = 0; col = 2;
			fillrectangle(68+(106*col),29+(29*row),94+(106*col),42+(29*row),color_purple);
			if (Gui.filtertyp == 0)
			{
				print_string("DEG",282,32,2,color_white);
			}
			else if (Gui.filtertyp == 1)
			{
				print_string("SMR4",279,32,2,color_white);
			}	
			// save value into eeprom
			EEPWriteByte(ee_filtertyp, Gui.filtertyp);		
			break;
			
			case(3): // last Patch
			Gui.last_patch = Enc.value_[2];
			row = 1; col = 0;
			fillrectangle(68+(106*col),29+(29*row),94+(106*col),42+(29*row),color_purple);
			if (Gui.last_patch > 0)
			{
				print_string("on",72,60,2,color_white);
			}
			else print_string("off",70,60,2,color_white);
			// save value into eeprom
			EEPWriteByte(ee_last_patch, Gui.last_patch);
			break;
			
			
		}
	}
	
}

//*************************************************************************
//	set marker in preset-list
//*************************************************************************
void checkmark_soundlist(void)
{
	if (Patch.init_flag != 0)
	{
		if ((select_preset /7) == (current_preset /7) && (select_presetbank == current_presetbank))
		{
			uint8_t ymarker = (current_preset % 7);
			uint8_t row = 35;
			row += (ymarker*19);
			fillrectangle(5,row-7,227,row+10,color_gray2);
			fill_circle(40,row,4,color_white);
			
			// print select preset name to yellow color
			set_presetname_folder();
			
			// redraw Presetlist ----------------------------------------------
			if( MMC_FILE_OPENED == ffopen("Name.bin",'r') )
			{
				char preset_name[14];
				uint16_t seek_pointer = (32 * current_preset);
				ffseek(seek_pointer);
				for (uint8_t i = 0; i < 13; i++)
				{
					preset_name[i] = ffread();
				}
				preset_name[13] = 0;
				draw_3digit_with(current_preset,8,row-5,6,color_white);
				print_string(preset_name,52,row-5,6,color_white);
				// print sine + wavefile symbols --------------------------
				ffseek(seek_pointer + 16);
				uint8_t symbol = ffread();
				if (symbol == 1)
				{
					draw_sine_symbol(168,row, color_white);
				}
				if(symbol == 2)
				{
					draw_wavefile_symbol(168,row, color_yello);
				}
				
				// print File length
				uint16_t file_len;
				file_len = ffread();
				file_len = file_len << 8;
				file_len = file_len + ffread();
				if (file_len > 0)
				{
					draw_4digit_without(file_len,183,row-3,2,color_white);
					print_string("KB",210,row-3,2,color_white);
				}
				
				ffclose();
			}
		}
	}
}









//*************************************************************************
// set sample input in sample menu
//*************************************************************************
void set_sample_input(void)
{
	switch (Sample.source)
	{
		case 0:
		draw_encoder_StrPos(5,"MIC",0);
		PORTB.OUT &= ~(1<<PIN3);
		PORTB.OUT |= (1<<PIN4);
		break;
		case 1:
		draw_encoder_StrPos(5,"EXT",23);
		PORTB.OUT |= (1<<PIN3);
		PORTB.OUT &= ~(1<<PIN4);
		break;
	}
}



//*************************************************************************
// Redraw sample-file
//*************************************************************************
void redraw_sample_file()
{
	zoom_level = 0;
	addoffset = 2855;
	
	//file_lenght = sampling_wr_addr;

	// init xram base addr ------------------------------------------------
	hugemem_ptr_t p = BOARD_EBI_SRAM_BASE;
	p += wave_offset;
	
	// init xy position ---------------------------------------------------
	int16_t x1 = 31;
		
	// draw shrinked wave -------------------------------------------------
	uint8_t hoehe_l;
	uint8_t hoehe_h;
	uint8_t hoehe_x;
	uint8_t yplot = 0;
	x1 = 0;
	
	for (uint8_t i1 = 0; i1 < 255; i1++)
	{
		uint8_t sample_h = 127;
		uint8_t sample_l = 127;
		
			//-----------------------------------------------------------------
			// key query: cancel loading
			// ----------------------------------------------------------------
			if (Key.query_flag == 1)
			{
				get_encoder();
				Key.query_flag = 0;
			}
			if (Enc.status_[0] == 1)
			{
				//return;
			}
				
		
		for(uint16_t i2 = 0; i2 < addoffset;i2++)
		{
			// read sample
			yplot = 255-hugemem_read8(p+i2);
			// if screen-plot over wave-end than draw zero-line
			if ((p+i2-BOARD_EBI_SRAM_BASE) > file_lenght-1)
			{
				yplot = 127;
			}
			// calculate positive and negative wave line
			if (yplot > 128)
			{
				if (sample_h < yplot)
				{
					sample_h = yplot;
				}
			}
			else
			{
				if (sample_l > yplot)
				{
					sample_l = yplot;
				}
			}
		}
		// calc Y-High
		hoehe_l = 65-(sample_l >>1);
		hoehe_h = (sample_h-128) >>1;
		hoehe_x = hoehe_l+hoehe_h;
		// save line for restore cursor background
		hugemem_write8(BOARD_EBI_SRAM_BASE + x1 + 0xFFE00,hoehe_h);
		hugemem_write8(BOARD_EBI_SRAM_BASE + x1 + 0xFFF00,hoehe_x);
		// inc x-position on screen
		x1++;
		// next sample_adr.
		p = p + addoffset;
	}
}

//*************************************************************************
// draw rec/play symbol for sampling
//*************************************************************************
void draw_rec_symbol(void)
{
	if (Gui.Sam_rec_flag == 0)
	{
		fillrectangle(269,48,295,65,color_green);
		print_string("play",269,52,2,color_white);
	}
	else
	{
		fillrectangle(269,48,295,65,color_red);
		print_string("rec",271,52,2,color_white);
	}
	
	pixel(270,48,color_black);
	pixel(295,48,color_black);
	pixel(270,64,color_black);
	pixel(295,64,color_black);
}

//*************************************************************************
// draw gray frame
//*************************************************************************
void draw_gray_frame(void)
{
	// grau frames
	rectangle(0,21,145,319,color_gray2);
	hline(0,50,318,color_gray2);
	hline(0,79,318,color_gray2);
	hline(0,108,318,color_gray2);
	if (page_nr == 14)
	{
		hline(0,137,318,color_gray2);
		vline(106,21,145,color_gray2);
		vline(212,21,145,color_gray2);
	}
	else
	{
		vline(106,21,86,color_gray2);
		vline(212,21,145,color_gray2);
	}
	
}



