/*
 * envelope.c
 *
 * Created: 26.06.2016 11:08:23
 *  Author: rolfd
 */ 

#include "globals.h"
#include "envelope.h"
#include "lfo.h"
#include "modulation.h"
#include "wavetable.h"
#include "display/displ_funktion.h"
#include "gui.h"
#include "oscillators.h"
#include "midi.h"
#include "op.h"


//*************************************************************************
// Envelope main routine
//*************************************************************************
void Envelope_Task(void)
{
	if (Gui.Sam_monitor_flag == 1)	// return if monitoring for sampling
	{
		return;
	}
	
	// Sync Envelope with audio buffer (max 5msec delay)
	if (Mid.gate_flag == 1 && Mid.trigger_flag == 1)
	{
		Env.start_count++;
	}
	
	// Midi Sync ----------------------------------------------------------
	//if (midi_sync_flag == 1 && Env.start_flag == 3)
	if (Mid.gate_flag == 1 && Mid.trigger_flag == 1 && Env.start_count == 3)
	{
		Mid.trigger_flag = 0;
		Env.start_count = 0;
		
		// start attack phase in envelope ---------------------------------
		Env.stage[0] = ATTACK;
		Env.stage[1] = ATTACK;
		Env.phase[0] = 0;
		Env.phase[1] = 0;
		Env.value_old[0] = Env.value[0];
		Env.value_old[1] = Env.value[1];
		
		// ReTrigger Osc1+2 in sample mode --------------------------------
		if (Osc.wave_mode[0] == 1 && Osc.sync_function[0] == 1)
		{			
			Osc1.phase_x = 0;
			Osc1.phase_carry2 = 0;
		}
		if (Osc.wave_mode[1] == 1 && Osc.sync_function[1] == 1)
		{
			Osc2.phase_x = 0;
			Osc2.phase_carry2 = 0;
		}
		
		// set Osc.loop_flag ----------------------------------------------
		Osc.loop_flag[0] = 1;
		Osc.loop_flag[1] = 1;
		
		// set Note Frequency ---------------------------------------------
		//frequency_tune();					// set note-frequency to osc1+2
		loop_end_flag1 = 0;
		loop_end_flag2 = 0;
		
		// LFOs sync ------------------------------------------------------
		LFO.run_flag[0] = 1;
		if (LFO.sync_flag[0] == 1)
		{
			LFO.phase[0] = 0;
		}
		LFO.run_flag[1] = 1;
		if (LFO.sync_flag[1] == 1)
		{
			LFO.phase[1] = 0;
		}
		LFO.run_flag[2] = 1;
		if (LFO.sync_flag[2] == 1)
		{
			LFO.phase[2] = 0;
		}
	}
	
	envelope(0);
	envelope(1);
}

//*************************************************************************
// calc Envelope
//*************************************************************************
void envelope (uint8_t env_nr)
{
	uint8_t ramp_a = 0;
	uint8_t ramp_b = 0;
	uint16_t ramp_start = 0;
	uint16_t ramp_end = 0;
	
	if (Env.stage[env_nr] == DEAD)
	{
		return;
	}
	
	if (Mid.gate_flag == 0 && Mid.trigger_flag == 1)
	{
		Mid.trigger_flag = 0;
		Env.stage[0] = RELEASE;
		Env.stage[1] = RELEASE;
		Env.phase[0] = 0;
		Env.phase[1] = 0;
	}
	
	if (Env.stage[env_nr] == ATTACK)
	{
		// load attack_value
		Env.phase_increment[env_nr] = pgm_read_word (&(lut_res_env_portamento_increments[Env.attack[env_nr]]));
		ramp_start = Env.value_old[env_nr];
		ramp_end = 65535;
		ramp_a = 0;
		ramp_b = 255;
	}
	else if (Env.stage[env_nr] == DECAY)
	{
		// load decay_value
		Env.phase_increment[env_nr] = pgm_read_word (&(lut_res_env_portamento_increments[Env.decay[env_nr]]));
		// load sustain_level
		ramp_start = pgm_read_word (&(sustain_rates[Env.sustain[env_nr]]));
		ramp_end = 65535;
		ramp_a = 255;
		ramp_b = 0;
	}
	
	else if (Env.stage[env_nr] == SUSTAIN)
	{
		return;
	}
	
	else if (Env.stage[env_nr] == RELEASE)
	{
		// load release_value
		Env.phase_increment[env_nr] = pgm_read_word (&(lut_res_env_portamento_increments[Env.release[env_nr]]));
		ramp_start = 0;
		ramp_end = Env.release_16[env_nr];
		ramp_a = 255;
		ramp_b = 0;
	}
	
	else if (Env.stage[env_nr] == DEAD)
	{
		Env.value[env_nr] = 0;
		Env.value_8[env_nr] = 0;
		return;
	}
	
	// increment 16Bit env_ramp
	Env.phase[env_nr] += Env.phase_increment[env_nr];

	// check end of attack phase
	if (Env.phase[env_nr] < Env.phase_increment[env_nr])
	{
		// set next envelope phase
		Env.stage[env_nr]++;
		Env.phase[env_nr] = 0;
	}
	else
	{
		uint8_t step = InterpolateSample(wav_res_env_expo, Env.phase[env_nr]);
		uint8_t ramp_expo =  U8MixU16(ramp_a, ramp_b, step);
		Env.value[env_nr] = ramp_start + (((uint32_t)(ramp_end - ramp_start) * ramp_expo) >> 8);
		Env.value_8[env_nr] = Env.value[env_nr] >> 8;
	}
	
	if (Env.stage[env_nr] < 3)
	{
		Env.release_16[env_nr] = Env.value[env_nr];
	}
}	


//*************************************************************************
// set values to Envelope 1+2
//*************************************************************************
void change_env(uint8_t env_nr)
{
	
	if (page_nr == 6){env_nr = 0;}
	if (page_nr == 7){env_nr = 1;}
			
	// change attack level
	if (Enc.status_[1] == 1)
	{
		Env.attack[env_nr] = Enc.value_[1];
		draw_encoder_position(2,Env.attack[env_nr]);
	}
	
	// change decay level
	if (Enc.status_[2] == 1)
	{
		Env.decay[env_nr] = Enc.value_[2];
		draw_encoder_position(3,Env.decay[env_nr]);
	}
	
	// change sustain level
	if (Enc.status_[3] == 1)
	{
		Env.sustain[env_nr] = Enc.value_[3];
		draw_encoder_position(4,Env.sustain[env_nr]);
		//Env.value[env_nr] = pgm_read_word (&(sustain_rates[Env.sustain[env_nr]]));		// read sustain-value from Log-Table
	}
	
	// change relaese level
	if (Enc.status_[4] == 1)
	{
		Env.release[env_nr] = Enc.value_[4];
		draw_encoder_position(5,Env.release[env_nr]);
	}
	// draw ADSR2 curve
	draw_adsr_line();
	
}



