/*
 * filter.c
 *
 * Created: 26.06.2016 19:06:58
 *  Author: rolfd
 */ 


#include "filter.h"
#include "display/displ_funktion.h"
#include "gui.h"
#include "keyquery.h"


//*************************************************************************
// Menu Page 8: Filter
//*************************************************************************
void draw_menupage_filter1(void)
{
	clear_WindowArea(color_black);
		
	// print windows header
	print_window_header(color_olive,"FILTER","1/1");
	print_knop_text("CUTOFF","RESONANCE","INPUT","MODE");
	
	// draw gray frame
	rectangle(1,22,143,317,color_gray2);
		
	// set encoder values
	set_encoder_val(VCF.cutoff_value,VCF.reso_value,VCF.ext_input,VCF.filter_mode);
	
	// Filtertyp ?
	if (Gui.filtertyp == 0)
	{
		set_encoder_max(127,127,1,2); // DEG
	}
	if (Gui.filtertyp == 1)
	{
		set_encoder_max(127,127,1,6); // SMR4
	}
	
	// draw extern filter input
	draw_filter_input();
	
	set_encoder_dyn(Enc.dynamic_,Enc.dynamic_,0,0);
	
	// draw mini-scope window
	fillrectangle(249,48,300,83,color_gray2);
	print_string("MiniScope",251,39,1,color_blue1);
	
	// set filter switch
	set_filter_switch();
	
	// draw filter curve
	VCF.old_cutoff = VCF.cutoff_value;
	VCF.old_reso = VCF.reso_value;
	print_filter_info();
	draw_filter_curve();
	
}

//*************************************************************************
// set filter switch
//*************************************************************************
void set_filter_switch(void)
{
	// Filtertyp: DEG
	if (Gui.filtertyp == 0)
	{
		// set filter-switch
		uint8_t mode_temp;
		mode_temp = 255-(VCF.filter_mode <<5);
		// Filter Bits lschen
		PORTB.OUT &= 0b00011111;
		// Filter Bits neu setzen
		PORTB.OUT |= mode_temp;
	}
	
	// Filtertyp SMR4
	if (Gui.filtertyp == 1)
	{
		uint8_t mode_temp;
		switch (VCF.filter_mode)
		{
			case 0: 
			{	// LP 6dB
				mode_temp =  0b01100000;
				PORTB.OUT &= 0b00011111;
				PORTB.OUT |= mode_temp;
				break;
			}
			case 1:
			{	// LP 12dB
				mode_temp =  0b11100000;
				PORTB.OUT &= 0b00011111;
				PORTB.OUT |= mode_temp;
				break;
			}
			case 2:
			{	// LP 18dB
				mode_temp =  0b00100000;
				PORTB.OUT &= 0b00011111;
				PORTB.OUT |= mode_temp;
				break;
			}
			case 3:
			{	// LP 24dB
				mode_temp =  0b10100000;
				PORTB.OUT &= 0b00011111;
				PORTB.OUT |= mode_temp;
				break;
			}
			case 4:
			{	// BP 12dB
				mode_temp =  0b11000000;
				PORTB.OUT &= 0b00011111;
				PORTB.OUT |= mode_temp;
				break;
			}
			case 5:
			{	// HP 12dB
				mode_temp =  0b00000000;
				PORTB.OUT &= 0b00011111;
				PORTB.OUT |= mode_temp;
				break;
			}
			case 6:
			{	// LP-HP 12dB
				mode_temp =  0b10000000;
				PORTB.OUT &= 0b00011111;
				PORTB.OUT |= mode_temp;
				break;
			}
		}
	}
}

//*************************************************************************
// set Filter values
//*************************************************************************
void change_filter1(void)
{
	// change cutoff
	if (Enc.status_[1] == 1)
	{
		VCF.cutoff_value = Enc.value_[1];
		draw_encoder_position(2,VCF.cutoff_value);
		draw_filter_curve();
	}
	
	// change resonance
	if (Enc.status_[2] == 1)
	{
		VCF.reso_value = Enc.value_[2];
		draw_encoder_position(3,VCF.reso_value);
		draw_filter_curve();
	}
	
	// ext Filter-Input
	if (Enc.status_[3] == 1)
	{
		VCF.ext_input = Enc.value_[3];
		if (VCF.ext_input == 0)
		{
			// set PortB for Filter switch
			PORTD.OUTCLR = PIN7_bm;
		}
		else 
		{
			PORTD.OUTSET = PIN7_bm;
		}
		draw_filter_input();		
	}
	
	// change filter mode
	if (Enc.status_[4] == 1)
	{
		VCF.filter_mode = Enc.value_[4];
		set_filter_switch();
		print_filter_info();
		draw_filter_curve();
	}	
}

//*************************************************************************
// print filter info
//*************************************************************************
void print_filter_info(void)
{
	fillrectangle(15,35,220,155,color_black);
	
	// draw filter curve
	print_string("100",56,70,2,color_yello);
	print_string("1K",119,70,2,color_yello);
	print_string("10K",176,70,2,color_yello);
	vline(17,85,65,color_white);
	vline(213,85,65,color_white);
	hline(17,150,195,color_white);
	
	if (Gui.filtertyp == 0)
	{
		switch (VCF.filter_mode)
		{
			case 0: {print_string("DEG LP 12dB",80,35,2,color_white);
			draw_encoder_StrPos(5,"LP",0);}
			break;
			case 1: {print_string("DEG HP 12dB",80,35,2,color_white);
			draw_encoder_StrPos(5,"HP",22);}
			break;
			case 2: {print_string("DEG BP 12dB",80,35,2,color_white);
			draw_encoder_StrPos(5,"BP",46);}
			break;
		}
		
		set_filter_switch();
	}
	
	if (Gui.filtertyp == 1)
	{
		switch (VCF.filter_mode)
		{
			case 0: {print_string("SMR4 LP 6dB",80,35,2,color_white);
			draw_encoder_StrPos(5,"LP6",0);}
			break;
			case 1: {print_string("SMR4 LP 12dB",80,35,2,color_white);
			draw_encoder_StrPos(5,"LP12",21);}
			break;
			case 2: {print_string("SMR4 LP 18dB",80,35,2,color_white);
			draw_encoder_StrPos(5,"LP18",42);}
			break;
			case 3: {print_string("SMR4 LP 24dB",80,35,2,color_white);
			draw_encoder_StrPos(5,"LP24",63);}
			break;
			case 4: {print_string("SMR4 BP 12dB",80,35,2,color_white);
			draw_encoder_StrPos(5,"BP12",84);}
			break;
			case 5: {print_string("SMR4 HP 12dB",80,35,2,color_white);
			draw_encoder_StrPos(5,"HP12",105);}
			break;
			case 6: {print_string("SMR4 LP-HP 12dB",80,35,2,color_white);
			draw_encoder_StrPos(5,"LPHP",127);}
			break;
			
		}
		
		set_filter_switch();
	}
	
	
	

}

//*************************************************************************
// draw filter curve
//*************************************************************************
void draw_filter_curve(void)
{
	
	uint8_t ypos_a;
	uint8_t ypos_b;
	uint8_t res_value;
	
	// draw lowpass filter curve ------------------------------------------
	if (VCF.filter_mode <= 3 )
	{
		// calc xpos
		uint8_t old_xpos = 19 + ((181 * VCF.old_cutoff) / 128);
		uint8_t new_xpos = 19 + ((181 * VCF.cutoff_value) / 128);
		
		// clear old xline
		draw_line(19,119,211,119,color_black);
		
		// calc half left curves
		res_value = (VCF.old_reso/2)+64;
		ypos_a = PGM_READ_BYTE(&(resonance_curve2[0]));
		ypos_a = (ypos_a * res_value)/69;
		ypos_a = 149 - ypos_a;
		
		// clear old curve
		for (uint8_t i = 0; i < 16;i++)
		{
			ypos_b = PGM_READ_BYTE(&(resonance_curve2[i]));
			ypos_b = (ypos_b * res_value)/69;
			ypos_b = 149 - ypos_b;
			if (ypos_a > 119){ypos_a = 119;}
			if (ypos_b > 119){ypos_b = 119;}
			if (old_xpos > (20 + i))
			{
				draw_line(old_xpos-i,ypos_a,old_xpos-i,ypos_b,color_black);
			}
			ypos_a = ypos_b;
		}
		
		// draw new curve
		res_value = (VCF.reso_value/2)+64;
		ypos_a = PGM_READ_BYTE(&(resonance_curve2[0]));
		ypos_a = (ypos_a * res_value)/69;
		ypos_a = 149 - ypos_a;
		
		for (uint8_t i = 0; i < 16;i++)
		{
			ypos_b = PGM_READ_BYTE(&(resonance_curve2[i]));
			ypos_b = (ypos_b * res_value)/69;
			ypos_b = 149 - ypos_b;
			if (ypos_a > 119){ypos_a = 119;}
			if (ypos_b > 119){ypos_b = 119;}
			if (new_xpos > (20+i))
			{
				draw_line(new_xpos-i,ypos_a,new_xpos-i,ypos_b,color_yello);
			}
			ypos_a = ypos_b;
		}
		
		// calc half right curves
		res_value = (VCF.old_reso/2)+64;
		ypos_a = PGM_READ_BYTE(&(resonance_curve[0]));
		ypos_a = (ypos_a * res_value)/69;
		ypos_a = 149 - ypos_a;
		
		// clear old curve
		for (uint8_t i = 0; i < 16;i++)
		{
			ypos_b = PGM_READ_BYTE(&(resonance_curve[i]));
			ypos_b = (ypos_b * res_value)/69;
			ypos_b = 149 - ypos_b;
			draw_line(old_xpos+i,ypos_a,old_xpos+i,ypos_b,color_black);
			ypos_a = ypos_b;
		}

		// draw new curve
		res_value = (VCF.reso_value/2)+64;
		ypos_a = PGM_READ_BYTE(&(resonance_curve[0]));
		ypos_a = (ypos_a * res_value)/69;
		ypos_a = 149 - ypos_a;
		
		for (uint8_t i = 0; i < 16;i++)
		{
			ypos_b = PGM_READ_BYTE(&(resonance_curve[i]));
			ypos_b = (ypos_b * res_value)/69;
			ypos_b = 149 - ypos_b;
			draw_line(new_xpos+i,ypos_a,new_xpos+i,ypos_b,color_yello);
			ypos_a = ypos_b;
		}
		
		// draw new xline
		if (new_xpos > 20+14)
		{
			draw_line(19,119,new_xpos-14,119,color_yello);
		}
	}
	
	// draw highpass filter curve -----------------------------------------
	if (VCF.filter_mode >= 5)
	{
		// calc xpos
		uint8_t old_xpos = 213 - ((181 * (127-VCF.old_cutoff)) / 128);
		uint8_t new_xpos = 213 - ((181 * (127-VCF.cutoff_value)) / 128);
	
		// clear old xline
		draw_line(19,119,211,119,color_black);
		
		// calc half left curves
		res_value = (VCF.old_reso/2)+64;
		ypos_a = PGM_READ_BYTE(&(resonance_curve2[0]));
		ypos_a = (ypos_a * res_value)/69;
		ypos_a = 149 - ypos_a;
		
		// clear old curve
		for (uint8_t i = 0; i < 16;i++)
		{
			ypos_b = PGM_READ_BYTE(&(resonance_curve2[i]));
			ypos_b = (ypos_b * res_value)/69;
			ypos_b = 149 - ypos_b;
			if (ypos_a > 119){ypos_a = 119;}
			if (ypos_b > 119){ypos_b = 119;}
			if (old_xpos < (213 - i))
			{
				draw_line(old_xpos+i,ypos_a,old_xpos+i,ypos_b,color_black);
			}
			ypos_a = ypos_b;
		}
		
		// draw new curve
		res_value = (VCF.reso_value/2)+64;
		ypos_a = PGM_READ_BYTE(&(resonance_curve2[0]));
		ypos_a = (ypos_a * res_value)/69;
		ypos_a = 149 - ypos_a;
		
		for (uint8_t i = 0; i < 16;i++)
		{
			ypos_b = PGM_READ_BYTE(&(resonance_curve2[i]));
			ypos_b = (ypos_b * res_value)/69;
			ypos_b = 149 - ypos_b;
			if (ypos_a > 119){ypos_a = 119;}
			if (ypos_b > 119){ypos_b = 119;}
			if (new_xpos < (213-i))
			{
				draw_line(new_xpos+i,ypos_a,new_xpos+i,ypos_b,color_yello);
			}
			ypos_a = ypos_b;
		}
	
		
		
		// calc half right curves
		res_value = (VCF.old_reso/2)+64;
		ypos_a = PGM_READ_BYTE(&(resonance_curve[0]));
		ypos_a = (ypos_a * res_value)/69;
		ypos_a = 149 - ypos_a;
		
		// clear old curve
		for (uint8_t i = 0; i < 16;i++)
		{
			ypos_b = PGM_READ_BYTE(&(resonance_curve[i]));
			ypos_b = (ypos_b * res_value)/69;
			ypos_b = 149 - ypos_b;
			draw_line(old_xpos-i,ypos_a,old_xpos-i,ypos_b,color_black);
			ypos_a = ypos_b;
		}
		
		// draw new curve
		res_value = (VCF.reso_value/2)+64;
		ypos_a = PGM_READ_BYTE(&(resonance_curve[0]));
		ypos_a = (ypos_a * res_value)/69;
		ypos_a = 149 - ypos_a;
		
		for (uint8_t i = 0; i < 16;i++)
		{
			ypos_b = PGM_READ_BYTE(&(resonance_curve[i]));
			ypos_b = (ypos_b * res_value)/69;
			ypos_b = 149 - ypos_b;
			draw_line(new_xpos-i,ypos_a,new_xpos-i,ypos_b,color_yello);
			ypos_a = ypos_b;
		}
		
		// draw new xline
		if (new_xpos < (213-14))
		{
			draw_line(new_xpos+14,119,212,119,color_yello);
		}
	}
	
	// draw bandpass filter curve -----------------------------------------
	if (VCF.filter_mode == 4)
	{
		// calc xpos
		uint8_t old_xpos = 19 + ((181 * VCF.old_cutoff) / 128);
		uint8_t new_xpos = 19 + ((181 * VCF.cutoff_value) / 128);

		// calc half left curves
		res_value = (VCF.old_reso/2)+64;
		ypos_a = PGM_READ_BYTE(&(resonance_curve[0]));
		ypos_a = (ypos_a * res_value)/69;
		ypos_a = 149 - ypos_a;
		
		// clear old curve
		for (uint8_t i = 0; i < 16;i++)
		{
			ypos_b = PGM_READ_BYTE(&(resonance_curve[i]));
			ypos_b = (ypos_b * res_value)/69;
			ypos_b = 149 - ypos_b;
			if (old_xpos > (20 + i))
			{
				draw_line(old_xpos-i,ypos_a,old_xpos-i,ypos_b,color_black);
			}
			ypos_a = ypos_b;
		}
		
		// draw new curve
		res_value = (VCF.reso_value/2)+64;
		ypos_a = PGM_READ_BYTE(&(resonance_curve[0]));
		ypos_a = (ypos_a * res_value)/69;
		ypos_a = 149 - ypos_a;
		
		for (uint8_t i = 0; i < 16;i++)
		{
			ypos_b = PGM_READ_BYTE(&(resonance_curve[i]));
			ypos_b = (ypos_b * res_value)/69;
			ypos_b = 149 - ypos_b;
			if (new_xpos > (20+i))
			{
				draw_line(new_xpos-i,ypos_a,new_xpos-i,ypos_b,color_yello);
			}
			ypos_a = ypos_b;
		}
	
		
		// calc half right curves
		res_value = (VCF.old_reso/2)+64;
		ypos_a = PGM_READ_BYTE(&(resonance_curve[0]));
		ypos_a = (ypos_a * res_value)/69;
		ypos_a = 149 - ypos_a;
		
		// clear old curve
		for (uint8_t i = 0; i < 16;i++)
		{
			ypos_b = PGM_READ_BYTE(&(resonance_curve[i]));
			ypos_b = (ypos_b * res_value)/69;
			ypos_b = 149 - ypos_b;
			draw_line(old_xpos+i,ypos_a,old_xpos+i,ypos_b,color_black);
			ypos_a = ypos_b;
		}

		// draw new curve
		res_value = (VCF.reso_value/2)+64;
		ypos_a = PGM_READ_BYTE(&(resonance_curve[0]));
		ypos_a = (ypos_a * res_value)/69;
		ypos_a = 149 - ypos_a;
		
		for (uint8_t i = 0; i < 16;i++)
		{
			ypos_b = PGM_READ_BYTE(&(resonance_curve[i]));
			ypos_b = (ypos_b * res_value)/69;
			ypos_b = 149 - ypos_b;
			draw_line(new_xpos+i,ypos_a,new_xpos+i,ypos_b,color_yello);
			ypos_a = ypos_b;
		}
	}
	
	
	// save new_resonance value into old_resonance value ------------------
	VCF.old_reso = VCF.reso_value;
	VCF.old_cutoff = VCF.cutoff_value;
		
	
	// draw filter grid ---------------------------------------------------
	for (uint8_t i = 0; i < 70;)
	{
		pixel(67,83+i, color_white);
		i += 10;
	}
	for (uint8_t i = 0; i < 70;)
	{
		pixel(127,83+i, color_white);
		i += 10;
	}
	for (uint8_t i = 0; i < 70;)
	{
		pixel(187,83+i, color_white);
		i += 10;
	}

}

//*************************************************************************
// draw extern filter input (encoder 4)
//*************************************************************************
void draw_filter_input(void)
{
	// value of extern filter input
	if (VCF.ext_input == 0)
	{
		draw_encoder_StrPos(4,"OFF",0);
	}
	else draw_encoder_StrPos(4,"ON",24);
}