//*************************************************************************
//
//                 define global variable
//
//*************************************************************************


#ifndef globals_H_
#define globals_H_



#define  F_CPU 32000000UL		// Prozessor-Clock 32MHz
#include <avr/interrupt.h>
#include <asf.h>

#include <util/delay.h>			// Bibliothek mit Warteroutinen (es wird kein Timer im ATmega verwendet)
#include <avr/pgmspace.h>		// Zugriff auf int. Programmspeicher
#include <avr/eeprom.h>			// Zugriff auf int. EEPROM
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <stdbool.h>

#include <hugemem.h>
#include <ebi.h>
#include <ebi_port.h>
#include <avr/eeprom.h>

//*************************************************************************
// BOOL DEFINITIONEN
//*************************************************************************
#define TRUE 	1
#define FALSE 	0


//*************************************************************************
// SainSmart 3.2" Display
//*************************************************************************

// Vitual Port: Signal invert without Reset
#define	SET_LE			VPORT1.OUT &= ~(PIN0_bm);	// LCD_LE
#define	CLR_LE			VPORT1.OUT |=  (PIN0_bm);
#define	SET_WR			VPORT1.OUT &= ~(PIN1_bm);	// LCD_WR
#define	CLR_WR			VPORT1.OUT |=  (PIN1_bm);
#define	LCD_SET_RS		VPORT1.OUT &= ~(PIN2_bm);	// LCD_RD
#define	LCD_CLR_RS		VPORT1.OUT |=  (PIN2_bm);
#define	SET_CS			VPORT1.OUT &= ~(PIN3_bm);	// LCD_RD
#define	CLR_CS			VPORT1.OUT |=  (PIN3_bm);
//#define CLR_RESET		PORTD.OUTCLR = PIN6_bm;
//#define SET_RESET		PORTD.OUTSET = PIN6_bm;
#define CLR_RESET		PORTD.OUT &= ~(PIN6_bm);
#define SET_RESET		PORTD.OUT |=  (PIN6_bm);

/*
// LCD Pin
#define LCD_RES     PIN6
#define LCD_LE		PIN0
#define LCD_WR      PIN1
#define LCD_RS      PIN2
*/

// LCD Colors
#define color_black		0x00, 0x00
#define color_white		0xFF, 0xFF
#define color_red		0xF8, 0x00
#define color_red_light 0xC0, 0x00
#define color_green		0x04, 0x00
#define color_blue		0x00, 0x1F
#define color_blue1     0x1C, 0x1C
#define color_silver	0xC6, 0x18
#define color_gray		0x84, 0x10
#define color_gray1     0x10, 0xA2
#define color_gray2     0x29, 0x65
#define color_maroon	0x01, 0x00
#define color_yello     0xFF, 0xE0
#define color_olive		0x84, 0x00
#define color_lime		0x07, 0xE0
#define color_aqua		0x07, 0xFF
#define color_teal		0x04, 0x10
#define color_navy		0x00, 0x10
#define color_fuchsia	0xF8, 0x1F
#define color_purple	0x80, 0x10
#define color_orange    0xFD, 0x00
#define color_gray4     0x31, 0x86
#define color_yello2    0xFD, 0x80

// XY swap
#define swap(type, i, j) {type t = i; i = j; j = t;}
	
  

//-------------------------------------------------------------------------
// KeyQuery SPI PortE
//-------------------------------------------------------------------------
#define SPI_SS      4     // PC4 = SPI SS
#define SPI_MOSI    5     // PC5 = SPI MOSI
#define SPI_MISO    6     // PC6 = SPI MISO
#define LCD_A0		2     // PC2 = LCD A0
#define SPI_SCK     7     // PC7 = SPI SCK

#define	DOGCommand		      PORTC.OUTCLR = (1 << LCD_A0 )
#define	DOGData				  PORTC.OUTSET = (1 << LCD_A0 )
#define	DOGEnable			  PORTE.OUTCLR = (1 << SPI_SS)
#define	DOGDisable			  PORTE.OUTSET = (1 << SPI_SS)



//*************************************************************************
// SDCARD
//
#define SPI2_SS      4		// PF4 = SPI SS
#define SPI2_MOSI    5		// PF5 = SPI MOSI
#define SPI2_MISO    6		// PF6 = SPI MISO
#define SPI2_Clock   7		// PF7 = SPI SCK

#define SDCARD_detect       PORTF.IN & PIN1_bm


//*************************************************************************
// define EEPROM addr
#define ee_patch_nr		0x0000
#define ee_patchbank	0x0001
#define ee_last_patch	0x0002
#define ee_enc_dynamic  0x0003
#define ee_filtertyp    0x0004


//-------------------------------------------------------------------------
// Channel Switch for Scope and Sample Recording
//
#define cha1_enable	 PORTA.OUTSET = (1 << PIN0)
#define cha2_enable	 PORTA.OUTCLR = (1 << PIN0)

//-------------------------------------------------------------------------
// Voltage Cotrol Line for VCF + VCA + Delay
//
#define CV_VCF_Cha1			TCD0_CCABUF			// 12Bit Value VCF Cha1
#define CV_VCF_Cha2			TCD0_CCBBUF			// 12Bit Value VCF Cha2
#define CV_VCA_Cha1			TCD0_CCCBUF			// 12Bit Value for VCA Cha1
#define CV_VCA_Cha2			TCD0_CCDBUF			// 12Bit Value for VCA Cha2
#define CV_Reso1			TCD1_CCABUF			// 8Bit Value Resonanc
#define CV_Reso2			TCD1_CCBBUF			// 8Bit Value Resonanc
#define CV_delay_value1		TCE0_CCABUF			// 8Bit Value
#define CV_delay_value2		TCE0_CCBBUF			// 8Bit Value
#define CV_delay_feedback	TCE0_CCCBUF			// 8Bit Value
#define CV_delay_time		TCE0_CCDBUF			// 8Bit Value

//-------------------------------------------------------------------------
// LED's
//
#define Test_LED_on		PORTB_OUT &= ~(1<<PIN3)		// Test-LED on
#define Test_LED_off	PORTB_OUT |=(1<<PIN3)		// Test_LED off
#define Test_LED_toggle PORTB_OUTTGL = (1<<PIN3)	// Test_LED toggle

#define Midi_LED_on		LED.Port_A |=  (1 << 0);	// Midi LED on
#define Midi_LED_off	LED.Port_A &= ~(1 << 0);	// Midi LED off
#define LFO1_LED_on		LED.Port_A |=  (1 << 1);	// LFO1 LED on
#define LFO1_LED_off	LED.Port_A &= ~(1 << 1);	// LFO1 LED off
#define LFO2_LED_on		LED.Port_A |=  (1 << 2);	// LFO2 LED on
#define LFO2_LED_off	LED.Port_A &= ~(1 << 2);	// LFO2 LED off
#define LFO3_LED_on		LED.Port_A |=  (1 << 3);	// LFO3 LED on
#define LFO3_LED_off	LED.Port_A &= ~(1 << 3);	// LFO3 LED off
#define SEQ_LED_on		LED.Port_A |=  (1 << 4);	// SEQ LED on
#define SEQ_LED_off		LED.Port_A &= ~(1 << 4);	// SEQ LED off
#define REC_LED_on		LED.Port_A |=  (1 << 5);	// REC LED on
#define REC_LED_off		LED.Port_A &= ~(1 << 5);	// REC LED off
#define Poti2_LED_on	LED.Port_A |=  (1 << 6);	// Poti_1 LED on
#define Poti2_LED_off	LED.Port_A &= ~(1 << 6);	// Poti_1 LED off
#define Poti1_LED_on	LED.Port_A |=  (1 << 7);	// Poti_2 LED on
#define Poti1_LED_off	LED.Port_A &= ~(1 << 7);	// Poti_2 LED off

struct Led
{
	uint8_t Port_A;
	
} LED;

//-------------------------------------------------------------------------
// lcd menu pages
//-------------------------------------------------------------------------
//extern uint8_t inverse;				
extern uint16_t test_timer;
extern uint8_t test_timer_flag;
extern uint16_t time_flag1_count;
extern uint8_t time_flag1;
extern uint8_t old_x;
extern uint8_t page_nr;
extern uint8_t old_page_nr;
extern uint8_t last_pagenr;
extern uint8_t vca_submenu;
extern uint8_t osc1_upage;
extern uint8_t osc2_upage;
extern uint8_t vcf_env_upage;
extern uint8_t enc2_upage;
extern uint8_t osc1_submenu;
extern uint8_t osc2_submenu;
extern uint8_t lfo1_submenu;
extern uint8_t lfo2_submenu;
extern uint8_t lfo3_submenu;
extern uint8_t vcf1_submenu;
extern uint8_t env1_submenu;
extern uint8_t env2_submenu;
extern uint8_t env3_submenu;
extern uint8_t env4_submenu;
extern uint8_t seq_submenu;
extern uint8_t mod_submenu;
extern uint8_t menu_selection;			// 1 = Menu selection disabled	
extern uint8_t char_name;
extern uint8_t char_pos;
extern uint8_t lcd_env_point;
extern uint8_t env_x1;
extern uint8_t env_x2;
extern uint8_t clr_env_curve_flag;
extern uint8_t cursor_pos_old;
extern uint8_t cursor_pos_new;
extern uint8_t cursor_loopA;
extern uint8_t cursor_loopB;
extern uint8_t zoom_level;
extern uint8_t cursorbuffer[];
extern uint8_t pixel_data1a;
extern uint8_t pixel_data1b;
extern uint8_t pixel_data1c;
extern uint8_t pixel_data1d;
extern uint8_t pixel_data2a;
extern uint8_t pixel_data2b;
extern uint8_t pixel_data2c;
extern uint8_t pixel_data2d;
extern uint32_t cursor_time_code;
extern uint32_t cursor_wave_address;
extern uint32_t screen_startaddr;
extern uint32_t screen_endaddr;
extern uint32_t loop_startaddr;
extern uint32_t loop_endaddr;
extern uint8_t old_selEntry;
extern uint16_t new_cursor_pos;
extern uint8_t osc1_old_selEntry;
extern uint8_t osc2_old_selEntry;
extern uint16_t osc1_old_cursor_pos;
extern uint16_t osc2_old_cursor_pos;
extern uint32_t cursor_addr_offset;
extern char preset_name[17];

struct LCDMenu
{
	uint8_t current_preset;
	uint8_t current_presetbank;
	uint8_t select_preset;
	uint8_t select_presetbank;
	uint8_t miniscope_flag;
	uint8_t osc1_submenu;
	uint8_t osc2_submenu;
	uint8_t osc1_par_nr;
	uint8_t osc2_par_nr;
	
} Menu;

struct Cursor
{
	uint8_t old_cursor_pos_loopA;
	uint8_t old_cursor_pos_loopB;
	uint16_t zoom_addr_offset;
	
}Cursor;
extern uint8_t Osc1_current_entry;
extern uint8_t Osc1_start_entry;
extern uint8_t Osc1_last_entry;
extern uint8_t Osc2_current_entry;
extern uint8_t Osc2_start_entry;
extern uint8_t Osc2_last_entry;
extern uint8_t Sound_current_entry;
extern uint8_t Sound_start_entry;
extern uint8_t Sound_last_entry;

extern uint8_t colorH;
extern uint8_t colorL;


/*
char note_C0[] PROGMEM = "C";
char note_C1[] PROGMEM = "C#";
char note_D0[] PROGMEM = "D";
char note_D1[] PROGMEM = "D#";
char note_E0[] PROGMEM = "E";
char note_F0[] PROGMEM = "F";
char note_F1[] PROGMEM = "F#";
char note_G0[] PROGMEM = "G";
char note_G1[] PROGMEM = "G#";
char note_A0[] PROGMEM = "A";
char note_A1[] PROGMEM = "A#";
char note_H0[] PROGMEM = "H";

PGM_P note_table[12] PROGMEM = {note_C0,note_C1,note_D0,note_D1,note_E0,note_F0,note_F1,
	note_G0,note_G1,note_A0,note_A1,note_H0};
	*/




//-------------------------------------------------------------------------
// Oscillator
//-------------------------------------------------------------------------

extern uint16_t Osc1_integral;
extern uint8_t Osc1_fractional;
extern uint8_t Osc1_carry;

//extern uint8_t SBufChA[];			// Sample Buffer
extern volatile uint8_t SBufChA_rdpointer;
extern volatile uint8_t SBufChA_wrpointer;
extern uint8_t sample_indexChA;
extern volatile uint8_t sample_adr;

extern uint16_t wave_adr;
extern uint8_t wave_frq;
extern uint32_t wave_len;

extern uint8_t wave_nr1;
extern uint8_t wave_nr2;
extern uint8_t wavebank1;
extern uint8_t wavebank2;
extern uint8_t sound_nr;
extern uint8_t bank_nr;
extern uint8_t oktave_value1;
extern uint8_t oktave_value2;
extern uint8_t transpose_value1;
extern uint8_t transpose_value2;
extern uint8_t detune_value1;
extern uint8_t detune_value2;
extern int8_t pitch_oct1;
extern int8_t pitch_oct2;
extern uint8_t osc1_mode;
extern uint8_t osc2_mode;
extern uint8_t osc_mode;
extern volatile uint64_t phaccu1;
extern volatile uint64_t phaccu2;
extern uint32_t phaccu_stepsize1;
extern uint32_t phaccu_stepsize2;
extern volatile uint16_t wavetable_dco1;
extern volatile uint16_t wavetable_dco2;
extern volatile uint8_t old_wave;
extern volatile uint8_t new_wave;
extern volatile uint8_t pwm1_level;
extern volatile uint8_t pwm2_level;
extern int8_t pitch_semi1;
extern int8_t pitch_semi2;
extern int8_t pitch_fine1;
extern int8_t pitch_fine2;
extern volatile uint8_t osc1_value;
extern volatile uint8_t osc2_value;
extern uint8_t temp_osc1_value;
extern uint8_t temp_osc2_value;
extern volatile uint8_t osc1_volume;
extern volatile uint8_t osc1_pan;
extern volatile uint8_t osc2_volume;
extern volatile uint8_t osc2_pan;
extern volatile int8_t osc1_balance;
extern volatile int8_t osc2_balance;
extern volatile uint8_t Bitcrush;		// Bit reduction
extern uint8_t Bitcrush_Bits;
extern int32_t osc1_stepsize;
extern int32_t osc2_stepsize;

extern uint16_t wave_shrink;
extern uint8_t cursor1_xpos;
extern uint8_t cursor2_xpos;
extern uint32_t wave_offset;
extern uint16_t addoffset;
extern uint8_t enc_waveoffset;
extern uint8_t enc_addoffset;
extern uint32_t wave_adr1;
extern uint32_t wave_adr2;
extern volatile uint32_t wave_adr3;
extern volatile uint32_t wave_adr4;
extern uint8_t sel_sample;
extern volatile uint8_t loop_flag;
extern volatile uint8_t loop_end_flag1;
extern volatile uint8_t loop_end_flag2;
extern uint8_t bandlim_octave_osc1;
extern uint8_t bandlim_octave_osc2;
extern int8_t midi_trans;
extern uint8_t Osc1_ReTrig;

struct Oscillator
{
	uint8_t osc1_typ;
	uint8_t osc2_typ;
	uint8_t osc1_wavenumber;
	uint8_t osc2_wavenumber;
	uint8_t osc1_banknumber;
	uint8_t osc2_banknumber;
	uint8_t osc1_semitone;
	uint8_t osc2_semitone;
	uint8_t osc1_fine;
	uint8_t osc2_fine;
	uint16_t osc1_fpitch;
	uint16_t osc2_fpitch;
	uint16_t osc1_pitch;
	uint16_t osc2_pitch;
	uint16_t osc12_fpitch;
	uint16_t osc12_pitch;
	
} Osci;






struct DigiOsc
{
	uint8_t wave_typ1;
	uint8_t wave_typ2;	
} DCO;

// for use in assembly code without struct
extern uint8_t Osci_osc_mix;
extern uint8_t Osci_noise_mix;
extern uint8_t Osci_operator;
extern uint8_t Osci_operator2;
extern uint8_t Osci_bitcrush;
extern uint8_t Osci_puls;
extern uint8_t Osci_puls2;
extern uint8_t Osci_pwm_value;
extern uint8_t Osci_pwm_value2;
extern uint16_t Osci_sin_wav;
extern uint16_t Osci_fm_ratio;
extern uint32_t Osci_phase2;
extern uint8_t wavefile_flag;
extern uint8_t Osc1_wavefile_flag;
extern uint8_t Osc2_wavefile_flag;


//-------------------------------------------------------------------------
// sampling
//-------------------------------------------------------------------------
volatile uint8_t sampling_flag;
extern __uint24 sampling_wr_addr;
extern __uint24 sampling_rd_addr;
extern uint8_t Sampling_rec_mode;
extern uint8_t Sampling_recording;
extern uint8_t Sampling_threslev;
extern uint8_t Sampling_thresflag;
volatile uint8_t Sampling_recflag;
extern uint8_t redraw_sample;
volatile uint8_t Sampling_monitor_flag;

struct Sampling
{
	uint8_t change_flag;
	uint8_t source;
	
} Sample;


//-------------------------------------------------------------------------
// ADC Input
//-------------------------------------------------------------------------
extern uint8_t poti1_value;
extern uint8_t poti2_value;
extern uint8_t poti_mux;


//-------------------------------------------------------------------------
// Amplifier (VCA)
//
extern uint8_t vca1_value;
extern uint8_t vca2_value;
extern uint16_t vca1_out;
extern uint16_t vca2_out;
volatile extern uint8_t pan_mix;


//*************************************************************************
// Modulation Matrix
//*************************************************************************

// sources
enum
{
	MOD_SRC_ENV1	= 0,
	MOD_SRC_ENV2	= 1,
	MOD_SRC_LFO1	= 2,
	MOD_SRC_LFO2	= 3,
	MOD_SRC_LFO3    = 4,
	MOD_SRC_VELO	= 5,
	MOD_SRC_NOTE    = 6,
	MOD_SRC_NOISE   = 7,
	MOD_SRC_SEQ     = 8,
	MOD_SRC_MWHEEL  = 9,
	MOD_SRC_PBEND	= 10,
	MOD_SRC_ATOUCH  = 11,
	MOD_SRC_POTI1   = 12,
	MOD_SRC_POTI2   = 13,
	MOD_SRC_OFFSET  = 14
};
// destinations
enum
{
	MOD_DST_VCA			= 0,
	MOD_DST_VCA_BAL		= 1,
	MOD_DST_VCF			= 2,
	MOD_DST_VCF_BAL		= 3,
	MOD_DST_RESO		= 4,
	MOD_DST_LFO1		= 5,
	MOD_DST_LFO2		= 6,
	MOD_DST_LFO3		= 7,
	MOD_DST_OSC1_f		= 8,
	MOD_DST_OSC2_f		= 9,
	MOD_DST_OSC1		= 10,
	MOD_DST_OSC2		= 11,
	MOD_DST_OSC12_f		= 12,
	MOD_DST_OSC12		= 13,
	MOD_DST_Osc1_PRM    = 14,
	MOD_DST_Osc2_PRM    = 15,
	MOD_DST_OSC_MIX     = 16,
	MOD_DST_Fx_Time     = 17,
	MOD_DST_Fx_PAN		= 18
};



extern uint8_t lfo1_depth_vca;
extern uint8_t lfo2_depth_vca;
extern uint8_t env1_depth_vca;
extern uint8_t env2_depth_vca;
extern uint8_t lfo1_depth_vcf;
extern uint8_t lfo2_depth_vcf;
extern uint8_t env1_depth_vcf;
extern uint8_t env2_depth_vcf;
extern uint8_t lfo1_depth_osc1;
extern uint8_t lfo2_depth_osc1;
extern uint8_t env1_depth_osc1;
extern uint8_t env2_depth_osc1;
extern uint8_t lfo1_depth_osc2;
extern uint8_t lfo2_depth_osc2;
extern uint8_t env1_depth_osc2;
extern uint8_t env2_depth_osc2;
extern uint8_t lfo1_depth_fx;
extern uint8_t lfo2_depth_fx;
extern uint8_t env1_depth_fx;
extern uint8_t env2_depth_fx;
//-----------------------------------
extern uint8_t slot_array[12][3];
extern uint16_t source_addr[12];
extern uint16_t destination_addr[12];
extern uint16_t dummy_addr;
extern uint8_t slot_nr;
extern uint8_t source_nr;
extern uint8_t destination_nr;
extern uint8_t mod_amount;
extern uint16_t VCA_gain;
extern uint16_t VCF_cutoff;
extern uint16_t VCF_resonanc;
extern uint16_t Fx_gain;


// Sequencer
struct ModMatrix
{
	uint8_t slot_number;
	uint8_t old_slot_number;
	
} Matrix;

extern uint32_t xxxx;  // Test Variable


//-------------------------------------------------------------------------
// Midi
//


extern uint8_t midibuf[];
extern uint8_t midibuf_wr_pointer;
extern uint8_t midibuf_rd_pointer;
extern uint8_t midi_byte;
extern uint8_t midi_statusbyte;
extern uint8_t midi_data1;
extern uint8_t midi_data2;
extern uint8_t midi_datasize;
extern uint8_t midi_datapointer;
extern uint8_t midi_note;
extern uint8_t last_midi_note;
extern uint8_t first_midi_note;
extern uint8_t velocity;
extern uint8_t velo_temp;
extern uint8_t NoteOn_icon;
extern uint8_t midi_icon_set;
extern uint8_t midi_receive_chanal;
extern uint16_t midi_pitchbend;
extern uint8_t led_timer;
extern uint8_t ctrl_data0;
extern uint8_t ctrl_data1;
extern uint8_t prgchange_nr;
extern uint8_t arpeggio_flag;
extern uint16_t arpeggio_timer0;
extern uint16_t arpeggio_timer1;
extern uint8_t arpeggio_step;
extern uint8_t midictrl_data1;
extern uint8_t midictrl_data2;
extern uint8_t midictrl_data3;
extern uint8_t midictrl_data4;
extern uint8_t midictrl_data5;
extern uint8_t midictrl_data6;
extern uint8_t midictrl_data7;
extern uint8_t midictrl_data8;
extern uint8_t midictrl_flag;

extern uint8_t midi_sync_flag;
extern uint8_t midi_gate_flag;

extern uint8_t midi_data[];
extern uint8_t NoteStack_pool[];
extern uint8_t VeloStack_pool[];
extern uint8_t NoteStack_ptr;
extern uint8_t NoteStack_size;
extern uint8_t data_size;
extern uint8_t expected_data_size;
extern uint8_t note_count;
extern uint8_t midi_velocity;
extern uint8_t midi_channel;
extern uint8_t midi_note_trigger;
extern uint8_t midi_note_gate;

// Sequencer
struct MIDI
{
	uint16_t Pitchbend;
	uint8_t  Modwheel;
	uint8_t  Atouch;
		
} Midi;



//*************************************************************************
// Fx Delay
//*************************************************************************
struct Fx_Delay
{
	uint8_t Time;
	uint8_t Time_value;
	uint8_t Feedback;
	uint8_t Value;
	uint8_t Pan;
} Fx;

//*************************************************************************
// LED BARGRAPH
//*************************************************************************
struct LED_BARGRAPH
{
	uint8_t new_sample;
	uint8_t old_sample;
	uint8_t Led_pos;
	uint8_t new_Led_pos;
	uint8_t Peak_pos;
	uint8_t Peak_hold;
	uint8_t sample_count;
	volatile uint16_t xpos;
	
} LEDBAR;




//-------------------------------------------------------------------------
// SDCARD
//
extern uint8_t SDCARD_status;
extern uint32_t freebytes;
extern uint8_t selected_file;
extern uint8_t file_numbers;
extern char filename_buffer[];
extern char sound_bank_nr;
extern uint8_t sound_prg_nr;
extern uint8_t sample_bank_nr;
extern char sample_file_name[];
extern char sample_name[];
extern uint8_t file_len_h;
extern uint8_t file_len_l;
extern uint8_t file_info;
extern uint8_t sd_disable;
extern __uint24 file_lenght;
extern char file_name[];
extern uint8_t save_flag;
extern uint8_t load_flag;
extern uint8_t delete_flag;
extern uint8_t cancle_flag;
extern uint8_t load_wavefile_flag;
extern uint16_t seek_pointer16;

struct SD_CARD
{
	uint8_t delay_time;
	
} SDCARD;

extern uint8_t sdcard_time;

extern uint8_t file_current_entry;
extern uint8_t file_last_entry;
extern uint8_t file_temp_entry;


//*************************************************************************
// Sound
//*************************************************************************
extern char soundname_x[17];
extern uint16_t sdata_len;
extern uint8_t sound;
extern uint8_t sel_sound;
extern uint8_t sound_bank;
extern uint8_t sel_sound_bank;
extern uint8_t wavebank;
extern uint8_t samplebank;
extern uint32_t quantization_error; // when 0 noise in sound
extern __int24 quantization_error2;
extern uint32_t quant_error32;
extern uint32_t quant_error24;
extern uint16_t quant_error16;
extern uint8_t test_value;
extern uint8_t current_preset;
extern uint8_t select_preset;
extern uint8_t current_presetbank;
extern uint8_t select_presetbank;




//-------------------------------------------------------------------------
// Global Parameter
//-------------------------------------------------------------------------
uint8_t global_select;
uint8_t global_value;

//-------------------------------------------------------------------------
// Sonstiges
//
extern uint8_t soft_timer0;
extern uint16_t soft_timer1;
extern uint8_t soft_timer2;
extern uint8_t soft_timer3;
extern uint8_t soft_timer4;
extern uint16_t soft_timer11;
extern uint8_t soft_timer11_flag;

extern volatile uint8_t soft_timer0_flag;
extern uint8_t soft_timer1_flag;
extern uint8_t soft_timer4_flag;

extern uint8_t timer0_flag;
extern uint8_t timer3_flag;

extern uint8_t soft_timer1_flag;
extern uint8_t soft_timer2_flag;
extern uint8_t soft_timer3_flag;
extern uint8_t led0_timer;
extern uint8_t led1_timer;
extern uint8_t led2_timer;
extern uint8_t led3_timer;
extern uint8_t led4_timer;
extern uint8_t led5_timer;
extern uint8_t led6_timer;
extern uint8_t led7_timer;


// timer variable ( 10 ms intervall)
extern volatile uint16_t TimingDelay;

extern volatile uint16_t stackpointer;





#endif /* globals_H_ */