/*
 * init_system.c
 *
 * Created: 05.07.2016 19:15:46
 *  Author: Rolf Degen
 */

#include "globals.h"
#include "init_system.h"
#include "display/displ_funktion.h"
#include "gui.h"
#include "oscillators.h"
#include "xram.h"
#include "SDCARD/sd_card.h"
#include "sequencer.h"
#include "midi.h"


//*************************************************************************
// init Main system clock 32MHz
//*************************************************************************
void init_clock(void)
{

	// set startuptime for extern Crystal Oscillator
	OSC_XOSCCTRL = OSC_XOSCSEL_XTAL_16KCLK_gc | OSC_FRQRANGE_12TO16_gc;
	
	// enable ext. 16MHz Crystal Oscillator
	OSC.CTRL |= OSC_XOSCEN_bm;
	
	// wait oscillator is stable
	while(!(OSC.STATUS & OSC_XOSCRDY_bm));
	
	// set PLL  16MHz * 2
	OSC.PLLCTRL = OSC_PLLSRC_XOSC_gc | 0x02; // 0x02 = 32MHz, 0x03 = 48MHz
	
	// enable PLL
	OSC.CTRL |= OSC_PLLEN_bm;
	
	// wait PLL is stable
	while(!(OSC.STATUS & OSC_PLLRDY_bm));
	
	// protect I/O register
	CCP = CCP_IOREG_gc;
	
	// enable PLL-Clock
	CLK.CTRL = CLK_SCLKSEL_PLL_gc;
	
	// Peripheral Clock output: Disabled
	PORTCFG.CLKEVOUT=(PORTCFG.CLKEVOUT & (~PORTCFG_CLKOUT_gm)) |
	PORTCFG_CLKOUT_OFF_gc;
	
	// Disable the unused oscillators: 2 MHz, 32 MHz, internal 32 kHz, PLL
	OSC.CTRL&= ~(OSC_RC2MEN_bm | OSC_RC32MEN_bm | OSC_RC32KEN_bm |
	OSC_PLLEN_bm);

}

//*************************************************************************
// init Midi Usart-Interface
//*************************************************************************
void init_USART(void)
{
	// Midi serial baud rates 31250
	USARTF0.BAUDCTRLA = 0xC0;						// BSCALE = -6, BSEL = 4032
	USARTF0.BAUDCTRLB = 0xAF;						// ==> 31.250 Midi-Baudrate
	USARTF0.CTRLC = 0x03;							// Async, No Parity, 1 stop bit, 8 data bits
	USARTF0.CTRLB = 0x10;							// Enable RX
	USARTF0.CTRLA = 0x20;							// Enable RX interrupt
	USARTF0.CTRLA = USART_RXCINTLVL_HI_gc;			// RX interrupt high level
} 


//*************************************************************************
// init Timer
//*************************************************************************
void init_TIMER(void)
{
	// init Timer_0 25usec 44.1KHz (Audio_buffer out) ---------------------
	// 40KHz = 799   44.1KHz = 735
	TCC0.CTRLA = TC_CLKSEL_DIV1_gc;		// Presacler 1
	TCC0.PER = 799;						// Counter Top-Value 799 = 40KHz
	TCC0.INTCTRLA = 0b00000000;			// Interrupt priority high
	
	// init Timer_1	1msec 1KHz Envelope generators ------------------------
	TCC1.CTRLA |= TC_CLKSEL_DIV4_gc;
	TCC1.PER = 7999;
	TCC1.INTCTRLA = 0b00000000;			// Interrupt priority low
	
	// init Timer_2 interrupt (render osc buffer)--------------------------
	TCE1.CTRLA |= TC_CLKSEL_DIV1_gc;
	TCE1.PER = 0;
	TCE1.INTCTRLA = 0b00000010;			// Interrupt priority middle
	
	/*
	// init Timer_3 interrupt ---------------------------------------------
	TCF0.CTRLA |= TC_CLKSEL_DIV2_gc;
	TCF0.PER = 9500;
	TCF0.INTCTRLA = 0b00000010;			// Interrupt priority low
	*/
}

//*************************************************************************
// init I/O Ports
//*************************************************************************
void init_PORT(void)
{
	// Yello Test-LED on MotherBoard
	PORTB.DIR |= (1<<PIN3);			// Portpin B.3 Direction Out (Led1 Power/Status)
		
	//LED PortA-C into Frontpannel off;
	LED.Port_A = 0x00;
	
	// ADC Inputs
	PORTA.DIR &= ~(1<<PIN0);		// PortA0 ext URef 2.5V
	PORTA.DIR &= ~(1<<PIN1);		// ext. Audio Input -
	PORTA.DIR &= ~(1<<PIN3);		// ext. AudioIn + 
	PORTA.DIR &= ~(1<<PIN4);		// ext. analog Comparator Input
	PORTB.DIR &= ~(1<<PIN0);		// Poti_1
	PORTB.DIR &= ~(1<<PIN1);		// Poti_2
	
	// PortB Audio/Filter Switch
	/*
	PORTA.DIR |= (1<<PIN5);
	PORTA.DIR |= (1<<PIN6);
	PORTA.DIR |= (1<<PIN7);
	PORTB.DIR |= (1<<PIN3);
	PORTB.DIR |= (1<<PIN4);
	PORTB.DIR |= (1<<PIN5);			// free for Filter-Board
	PORTB.DIR |= (1<<PIN6);			// free for Filter-Board
	PORTB.DIR |= (1<<PIN7);			// free for Filter-Board
	*/
	PORTA.DIRSET = PIN5_bm;
	PORTA.DIRSET = PIN6_bm;
	PORTA.DIRSET = PIN7_bm;
	PORTB.DIRSET = PIN3_bm;
	PORTB.DIRSET = PIN4_bm;
	PORTB.DIRSET = PIN5_bm;			// free for Filter-Board
	PORTB.DIRSET = PIN6_bm;			// free for Filter-Board
	PORTB.DIRSET = PIN7_bm;			// free for Filter-Board
	PORTD.DIRSET = PIN7_bm;			// Filter Input Switch
	
	// Midi
	PORTF.DIR &= ~(1<<PIN2);		// PortF Pin1 In
	PORTF.PIN2CTRL = 0x18;			// enabled Pullup on Midi-IN (RXD-Pin PortF2)
	
	// lcd ----------------------------------------------------------------
			
	// Vitueller Port0 = PORTC (faster) LCD DATA
	PORTCFG.VPCTRLA |= PORTCFG_VP0MAP_PORTC_gc;
	VPORT0.DIR = 0xFF;
	
	// Vitueller Port1 = PortQ (faster) WR/LE
	PORTCFG.VPCTRLA |= PORTCFG_VP1MAP_PORTQ_gc;
	VPORT1.DIR = 0x0F;
	
	PORTD.DIR |= (1<<PIN6);	// RESET
	
	// SD Card Detect / Write Protect -------------------------------------
	PORTF.DIR &= ~(1<<PIN0);		// PortF Pin0 In (Write Protect)
	PORTF.DIR &= ~(1<<PIN1);		// PortF Pin1 In (Card Detect)
	PORTF.PIN0CTRL = 0x18;			// enabled Pullup Pin PortF0 
	PORTF.PIN1CTRL = 0x18;			// enabled Pullup Pin PortF1

}

//*************************************************************************
// init. PWM for Filter, VCA and Fx
// PWM Resolution and Frequency 8Bit/125KHz
//*************************************************************************
void init_PWM(void)
{
	PORTD.DIR = 0b11111111;
	PORTD.OUT = 0x00;
	TCD0.CTRLA = 0x01;		// divide clock by 2
	TCD0.PER = 0x03FF;		// Resolutions: 16Bit = 0xFFFF/12Bit = 0x0FFF/10Bit = 0x3FF/8Bit = 0x00FF
	TCD0.CCA = 0x00FF;		// A  VCF CH1  8Bit/125KHz
	TCD0.CCB = 0x00FF;		// B  VCF CH2
	TCD0.CCC = 0x00FF;		// C  VCA CH1
	TCD0.CCD = 0x00FF;		// D  VCA CH2
	TCD0.CTRLB = 0xF3;		// enable frequency generation for all 4 Channels
	
	TCD1.CTRLA = 0x01;		// divide clock by 2
	TCD1.PER = 0x00FF;		// 0xFFFF = 16Bit Resolution / 0x03FF = 10Bit / 0x00FF = 8Bit Resolution
	TCD1.CCA = 0x00FF;		// A	Reso CH1  125KHz
	TCD1.CCB = 0x00FF;		// B	Reso CH2
	TCD1.CTRLB = 0xF3;		// enable frequency generation for all 4 Channels
	

	PORTE.DIR = 0b00111111;
	PORTE.OUT = 0x00;
	TCE0.CTRLA = 0x01;		// divide clock by 2
	TCE0.PER = 0x00FF;		// Resolutions: 16Bit = 0xFFFF / 12Bit = 0x0FFF / 8Bit = 0x00FF
	TCE0.CCA = 0x00FF;		// A  125KHz
	TCE0.CCB = 0x00FF;		// B
	TCE0.CCC = 0x00FF;		// C
	TCE0.CCD = 0x00FF;		// D  Delay Time
	TCE0.CTRLB = 0xF3;		// enable frequency generation for all 4 Channels
	
	CV_VCA_Cha1 = 0;
	CV_VCA_Cha2 = 0;
	CV_VCF_Cha1 = 0;
	CV_VCF_Cha2 = 0;
	CV_Reso1 = 0;
	CV_Reso2 = 0;
	CV_delay_feedback = 0;
	CV_delay_value1 = 0;
	CV_delay_value2 = 0;
}

//*************************************************************************
// init SPI PortE for Encoder and KeyQuery 
//*************************************************************************
void init_SPI(void)
{
	// init SPI-PortE for KeyQuery
	PORTE.DIRSET  = (1 << SPI_MOSI) | (1 << SPI_SCK) | (1<<SPI_SS);
	
	// SPI-Clock 16MHz	(SPI Dobble Clockspeed)
	SPIE.CTRL =  SPI_MODE_0_gc | SPI_PRESCALER_DIV4_gc | (1<< SPI_ENABLE_bp) | (1<< SPI_MASTER_bp) | (1<< SPI_CLK2X_bp); // SPI Double Speed 16MHz
	SPIE.INTCTRL = SPI_INTLVL_OFF_gc; // SPI-Interrupt disabled
}

//*************************************************************************
// init DACA Ch1+Ch2 for Soundout
//*************************************************************************
void init_DAC(void)
{
	
	// DAC-Ch2 8Bit
	DACA.CTRLB = ~(DAC_CHSEL_gm | DAC_CH0TRIG_bm)| DAC_CHSEL_SINGLE_gc;
	DACA.CTRLC |= DAC_REFSEL_AREFA_gc | DAC_LEFTADJ_bm; // 8Bit
	//DACA.CTRLC |= DAC_REFSEL_AREFA_gc;				// 12Bit
	DACA.OFFSETCAL	= 0b11111111;
	DACA.GAINCAL	= 0b11111111;
	DACA.CTRLA = (DAC_CH0EN_bm | DAC_ENABLE_bm);
	
	
	// DAC-Ch1 8Bit
	DACB.CTRLB = ~(DAC_CHSEL_gm | DAC_CH0TRIG_bm) | DAC_CHSEL_SINGLE_gc;
	DACB.CTRLC |= DAC_REFSEL_AREFA_gc | DAC_LEFTADJ_bm; // 8Bit
	//DACB.CTRLC |= DAC_REFSEL_AREFA_gc;				// 12Bit
	DACB.OFFSETCAL  = 0b11111111;
	DACB.GAINCAL	= 0b11111111;
	DACB.CTRLA = (DAC_CH0EN_bm | DAC_ENABLE_bm);
}

//*************************************************************************
// init ADC
//*************************************************************************
void init_ADC(void)
{
	// intit ADC_A 12Bit (Sample Input) -----------------------------------
	// 8Bit / differencial mode / ext.VRef 2.5V PortA0 / Prescaler 64 / positiv Input PA3 / negativ Input PA1
	ADCA.CTRLB = ADC_RESOLUTION_12BIT_gc | ADC_FREERUN_bm;
	ADCA.REFCTRL = ADC_REFSEL_AREFA_gc;
	ADCA.PRESCALER = ADC_PRESCALER_DIV64_gc;
	ADCA.CH0.CTRL = ADC_CH_INPUTMODE_DIFF_gc;
	//ADCA.CH0.CTRL = ADC_CH_INPUTMODE_SINGLEENDED_gc;
	ADCA.CH0.MUXCTRL |= ADC_CH_MUXPOS_PIN3_gc | ADC_CH_MUXNEG_PIN1_gc;
	//ADCA.CH0.MUXCTRL |= ADC_CH_MUXPOS_PIN3_gc;
	ADCA.INTFLAGS = 0x00;
	ADCA.CALH = 0x00;
	ADCA.CTRLA = ADC_ENABLE_bm;		// ADC enabled
	
	
	// int ADC_B 8Bit (Pot1 + Pot2) ---------------------------------------
	// 8Bit / unsigned mode / ext.VRef 2.5V PortA0 / Prescaler 256 / singleended / input PB0+PB1
	ADCB.CTRLB = ADC_RESOLUTION_8BIT_gc | ADC_FREERUN_bm;
	ADCB.REFCTRL = ADC_REFSEL_AREFA_gc;
	ADCB.PRESCALER = ADC_PRESCALER_DIV256_gc;
	ADCB.CH0.CTRL = ADC_CH_INPUTMODE_SINGLEENDED_gc;
	ADCB.CH0.MUXCTRL = ADC_CH_MUXPOS_PIN0_gc;
	ADCB.INTFLAGS = 0x00;
	ADCB.CTRLA = ADC_ENABLE_bm;		// ADC enabled

	// configure Analog Comparator AC0 to PORTA Pin3 ----------------------
	ACA.AC0MUXCTRL = AC_MUXPOS_PIN4_gc | AC_MUXNEG_SCALER_gc;
	ACA.AC0CTRL = AC_INTMODE_FALLING_gc | AC_INTLVL_MED_gc | AC_HYSMODE_SMALL_gc | AC_ENABLE_bm;
	ACA.AC1CTRL = AC_INTLVL_MED_gc;
}

//*************************************************************************
// init extern 1MByte SRAM
//*************************************************************************
void init_XRAM(void)
{
	static struct ebi_cs_config     cs_config;

	// Set signals which are active-low to high value
	PORTH.OUT = 0xFF;
	
	// Configure bus pins as outputs(except for data lines). */
	PORTH.DIR = 0xFF;
	PORTK.DIR = 0xFF;
	
	// EBI-Mode: Addressbits, Nr. of Addresslatch, LPC-Mode
	ebi_setup_port(18, 2, 0, EBI_PORT_SRAM | EBI_PORT_3PORT
	| EBI_PORT_CS0);											// CS must be on
	ebi_cs_set_mode(&cs_config, EBI_CS_MODE_SRAM_gc);
	ebi_cs_set_address_size(&cs_config, EBI_CS_ASPACE_1MB_gc);
	ebi_cs_set_base_address(&cs_config, BOARD_EBI_SRAM_BASE);   // 0x800000UL
	ebi_cs_set_sram_wait_states(&cs_config, EBI_CS_SRWS_1CLK_gc);
	ebi_cs_write_config(3, &cs_config);
	ebi_enable_cs(3, &cs_config);
	Xram.max_space  = 0xEF1FF;
	Xram.free_space = 0xEF1FF;
	Xram.sample_length[0] = 0;
	Xram.sample_length[1] = 0;
}

//*************************************************************************
// init SainSmart 3.2 LCD
//*************************************************************************
void init_LCD(void)
{
	
	// set LCD Reset and CS
	SET_RESET;
	_delay_ms(10);
	CLR_RESET;
	_delay_ms(30);
	SET_RESET;
	_delay_ms(30);
	
	CLR_CS;
	
	// display off
	write_com_data(0x07, 0x0231); // Display Control (display on)
	_delay_us(1);
	
	// write command and data
	write_com_data(0x00, 0x0001); // OSCEN
	_delay_us(1);
	write_com_data(0x03, 0xA8A4); // Powercontrol
	_delay_us(1);
	write_com_data(0x0C, 0x0000); // Powercontrol 2 // 5,1V
	_delay_us(1);
	write_com_data(0x0D, 0x080C); // Powercontrol 3 // Vref 2.0 * 2.5
	_delay_us(1);
	write_com_data(0x0E, 0x2B00); // Powercontrol 4
	_delay_us(1);
	write_com_data(0x1E, 0x00B7); // Powercontrol 5
	_delay_us(1);
	
	
	
	// cange X side
	//write_com_data(0x01, 0x293F);	// 240x320 portrait mode
	write_com_data(0x01, 0x2B3F);	// 320x240 landscape mode
	_delay_us(1);
	
	write_com_data(0x02, 0x0600); // LCD Driving Waveform Control
	_delay_us(1);
	write_com_data(0x10, 0x0000); // Sleepmode
	_delay_us(1);
	
	// cange Y side 
	//write_com_data(0x11, 0x6070); // 240x320 portrt mode
	write_com_data(0x11, 0x6838);	// 320x240 landscape mode
	_delay_us(1);
	
	write_com_data(0x05, 0x0000); // Compare register
	_delay_us(1);
	write_com_data(0x06, 0x0000); // Compare register
	_delay_us(1);
	write_com_data(0x16, 0xEF1C); // Horizontal Porch
	_delay_us(1);
	write_com_data(0x17, 0x0003); // Vertical Porch
	_delay_us(1);
	write_com_data(0x07, 0x0230); // Display Control (display off)
	_delay_us(1);
	write_com_data(0x0B, 0x0000); // Frame Cycle Control
	_delay_us(1);
	write_com_data(0x0F, 0x0000); // Gate Scan Position
	_delay_us(1);
	write_com_data(0x41, 0x0000);
	_delay_us(1);
	write_com_data(0x42, 0x0000);
	_delay_us(1);
	write_com_data(0x48, 0x0000);
	_delay_us(1);
	write_com_data(0x49, 0x013F);
	_delay_us(1);
	write_com_data(0x4A, 0x0000);
	_delay_us(1);
	write_com_data(0x4B, 0x0000);
	_delay_us(1);
	write_com_data(0x44, 0xEF00);
	_delay_us(1);
	write_com_data(0x45, 0x0000);
	_delay_us(1);
	write_com_data(0x46, 0x013F);
	_delay_us(1);	
	write_com_data(0x30, 0x0707);	// Gama
	_delay_us(1);
	write_com_data(0x31, 0x0204);
	_delay_us(1);
	write_com_data(0x32, 0x0204);
	_delay_us(1);
	write_com_data(0x33, 0x0502);
	_delay_us(1);
	write_com_data(0x34, 0x0507);
	_delay_us(1);
	write_com_data(0x35, 0x0204);
	_delay_us(1);
	write_com_data(0x36, 0x0204);
	_delay_us(1);
	write_com_data(0x37, 0x0502);
	_delay_us(1);
	write_com_data(0x3A, 0x0302);
	_delay_us(1);
	write_com_data(0x3B, 0x0302);
	_delay_us(1);
	write_com_data(0x23, 0x0000);
	_delay_us(1);
	write_com_data(0x24, 0x0000);
	_delay_us(1);
	write_com_data(0x25, 0x8000);
	_delay_us(1);
	write_com_data(0x4F, 0x0000); // set GDDRAM Y address
	_delay_us(1);
	write_com_data(0x4E, 0x0000); // set GDDRAM X address
	_delay_us(10);
	write_com(0x22);
	_delay_us(10);
	
	// draw black screen
	fillrectangle(0,0,320,240,color_black);
	
	// display on
	write_com_data(0x07, 0x0233); // Display Control (display on)
	_delay_us(10);
	write_com(0x22);
	_delay_us(10);
	
}

//*********************************************************************
// init noise generator
//*********************************************************************
void init_noise(void)
{
	if (Noise.data_rng_state == 0)
	{
		Noise.data_rng_state++;
	}
}

//*****************************
// init Interrupt
//*****************************
void init_Interrupt(void)
{
	PMIC.CTRL |= PMIC_LOLVLEN_bm | PMIC_MEDLVLEN_bm | PMIC_HILVLEN_bm | PMIC_RREN_bm;     //  Interruptlevel aktiviert
	sei();
}

//*************************************************************************
// init parameter
//*************************************************************************
void init_parameter(void)
{
	// System
	
	// GUI
	Gui.max_nr_pages = 14;
	Gui.real_env_time = 0;
	Gui.real_env_flag = 0;
	
	// Sequncer Arpeggiator
	SEQ.seq_mode = 0;
	SEQ.seq_bpm =  120;
	SEQ.seq_div_value = 5;
	SEQ.seq_dir = 0;
	SEQ.seq_hold = 0;
	SEQ.seq_hold_flag = 0;
	SEQ.seq_oct = 0;
	SEQ.arp_step = 0;
	SEQ.arp_note_size = 0;
	NoteStack_ptr = 0;
	SEQ.draw_note_flag = 0;
	SEQ.draw_step_flag = 0;
	
	// Sample editor
	Osc.loop_A[0] = 0;
	Osc.loop_B[0] = 0;
	Osc.loop_A[1] = 0;
	Osc.loop_B[1] = 0;
	Gui.CurA_addr = 0;
	Gui.CurB_addr = 0;
	Gui.CurA_value = 0;
	Gui.CurB_value = 0;
	Gui.CurA_old_value = 0;
	Gui.CurB_old_value = 0;
	
	Gui.Sam_rec_mode = 0;
	Gui.Sam_rec_key_flag = 0;
	Gui.Sam_rec_flag = 0;
	Gui.Sam_monitor_flag = 0;
	Gui.Sam_threshold_flag = 0;
	Gui.cut_flag = 0;
	Gui.del_flag = 0;
	Gui.Sam_rec_mode = 2;			// Sample record_mode: Key
	Gui.filtertyp = EEPReadByte(ee_filtertyp);
	
	// Midi Flags
	Mid.trigger_flag = 0;
	midi_gate_flag = 0;
	
	// Envelope
	Env.stage[0] = DEAD;
	Env.stage[1] = DEAD;
	Env.value[0] = 0;
	Env.value[1] = 0;
	Env.value_8[0] = 0;
	Env.value_8[1] = 0;

}

//*********************************************************************
// write and read EEPROM
//*********************************************************************

// Byte in das EEPROM schreiben
void EEPWriteByte(uint16_t addr, uint8_t val)
{
	eeprom_write_byte((uint8_t *)addr, val);
}

// Byte aus dem EEPROM lesen
uint8_t EEPReadByte(uint16_t addr)
{
	return eeprom_read_byte((uint8_t *)addr);
}