/*
 * keyquery.c
 *
 * Created: 27.06.2016 20:01:34
 *  Author: Rolf Degen
 */ 

#include "globals.h"
#include "sequencer.h"
#include "keyquery.h"
#include "filter.h"
#include "lfo.h"
#include "display/displ_funktion.h"
#include "SDCARD/sd_card.h"
#include "oscillators.h"
#include "gui.h"
#include "miniscope.h"
#include "init_system.h"
#include "midi.h"




//*************************************************************************
// KEY QUERY
//*************************************************************************

const int8_t table[16] PROGMEM = {0,0,-1,0,0,0,0,1,1,0,0,0,0,-1,0,0};		//Encoder table

//-------------------------------------------------------------------------
// read Key Chips 74HC165
//-------------------------------------------------------------------------
void read_74hc165(void)
{
	DOGDisable;
	//KEYshift;
	uint8_t led_status = LED.Port_A;
	SPIE.DATA = led_status;
	while(!(SPIE.STATUS & (1 << SPI_IF_bp))){}
	Key.KEY_DATA1 = SPIE.DATA;
	SPIE.DATA = led_status;
	while(!(SPIE.STATUS & (1 << SPI_IF_bp))){}
	Key.KEY_DATA2 = SPIE.DATA;
	SPIE.DATA = led_status;
	while(!(SPIE.STATUS & (1 << SPI_IF_bp))){}
	Key.KEY_DATA3 = SPIE.DATA;
	//KEYload;
	DOGEnable;
	// clr Flag
	Key.ReadShiftReg_Flag = 0;	
} 

//*************************************************************************
// read encoder status on I/O Port 
//*************************************************************************
void read_enc_status(void)
{
	// set variable for encoder's
	int8_t new_x = 0;
	int8_t diff_x = 0;
	uint8_t enc_phaseA = 0;
	uint8_t enc_phaseB = 0;
	
	for (uint8_t enc_id = 0; enc_id < 5; enc_id++)
	{
		// encoder enabled ?
		if (Enc.enabled[enc_id] == 1)
		{
			// read encoder 1-4 I/O Pin
			switch (enc_id)
			{
				case 0:
				enc_phaseA = Key.KEY_DATA1;
				enc_phaseA &= 0b00000001;		// mask enc A Bit
				enc_phaseA ^= 0b00000001;
				enc_phaseB = Key.KEY_DATA1;
				enc_phaseB &= 0b00000010;		// mask enc B Bit
				enc_phaseB ^= 0b00000010;
				enc_phaseB = enc_phaseB >>1;
				break;
				case 1:
				enc_phaseA = Key.KEY_DATA1;
				enc_phaseA &= 0b00000100;
				enc_phaseA ^= 0b00000100;
				enc_phaseA = enc_phaseA >>2;
				enc_phaseB = Key.KEY_DATA1;
				enc_phaseB &= 0b00001000;
				enc_phaseB ^= 0b00001000;
				enc_phaseB = enc_phaseB >>3;
				break;
				case 2:
				enc_phaseA = Key.KEY_DATA1;
				enc_phaseA &= 0b00010000;
				enc_phaseA ^= 0b00010000;		// inv. enc A bit
				enc_phaseA = enc_phaseA >>4;
				enc_phaseB = Key.KEY_DATA1;
				enc_phaseB &= 0b00100000;
				enc_phaseB ^= 0b00100000;		// inv. enc B Bit
				enc_phaseB = enc_phaseB >>5;
				break;
				case 3:
				enc_phaseA = Key.KEY_DATA1;
				enc_phaseA &= 0b01000000;
				enc_phaseA ^= 0b01000000;
				enc_phaseA = enc_phaseA >>6;
				enc_phaseB = Key.KEY_DATA1;
				enc_phaseB &= 0b10000000;
				enc_phaseB ^= 0b10000000;
				enc_phaseB = enc_phaseB >>7;
				break;
				case 4:
				enc_phaseA = Key.KEY_DATA2;
				enc_phaseA &= 0b00000001;
				enc_phaseA ^= 0b00000001;
				enc_phaseB = Key.KEY_DATA2;
				enc_phaseB &= 0b00000010;
				enc_phaseB ^= 0b00000010;
				enc_phaseB = enc_phaseB >>1;
				break;
			}
			// slowing (Entschleunigung)
			Enc.Accel[enc_id] -= ENCODER_ACCEL_DEC;		// wird hier zuviel subtrahiert,
			if (Enc.Accel[enc_id] & 0b10000000)			// wird EncAccel negativ, und
			Enc.Accel[enc_id] = 0;
			
			// convert gray to binary
			new_x = 0;
			Enc.last[enc_id] = (Enc.last[enc_id] << 2)  & 0x0F;
			if (enc_phaseA) Enc.last[enc_id] |=2;
			if (enc_phaseB) Enc.last[enc_id] |=3;
			new_x += pgm_read_byte(&table[Enc.last[enc_id]]);
			
			// difference last - new
			diff_x = Enc.last_[enc_id] - new_x;					// difference last - new
			Enc.last_[enc_id] = new_x;								// save new to last
			if (diff_x > 0)
			{
				// direction
				if (Enc.last[enc_id] > 0)
				Enc.delta[enc_id] -= 1+(Enc.Accel[enc_id] >> 2);  // diff_1: xxxxxx11
				else
				Enc.delta[enc_id] += 1+(Enc.Accel[enc_id] >> 2);  // diff_1: xxxxxx01
				
				// acceleration (Beschleunigung)
				if (Enc.Accel[enc_id] <= (ENCODER_ACCEL_TOP - Enc.dynamic[enc_id]))
				Enc.Accel[enc_id] += Enc.dynamic[enc_id];
			}
		}
	}	
}

//*************************************************************************
// get encoder_values and call function
//*************************************************************************
void get_encoder(void)
{
	for (uint8_t enc_id = 0; enc_id < 5; enc_id++)
	{
		
		int16_t enc_temp;
		enc_temp = Enc.value[enc_id];
		
		if ((enc_temp + Enc.delta[enc_id]) < 0)
		{
			enc_temp = 0;
		}
		// max value
		else if ((enc_temp + Enc.delta[enc_id]) > 255)
		{
			enc_temp = 255;
		}
		// inc value
		else {enc_temp += Enc.delta[enc_id];}
		
		Enc.value[enc_id] = enc_temp;
		Enc.delta[enc_id] = 0;
		
		if (Enc.value[enc_id] != Enc.value_old[enc_id])
		{
			
			
			if (Enc.value[enc_id] <= Enc.maxval[enc_id])
			{
				wr_EncBuffer(enc_id,Enc.value[enc_id]);
				Enc.value_old[enc_id] = Enc.value[enc_id];
			}
			else
			{
				Enc.value[enc_id] = Enc.maxval[enc_id];
			}
			
		}
	}
}

//*************************************************************************
// write data to enc_buffer
//*************************************************************************
void wr_EncBuffer(uint8_t enc_id, uint8_t enc_value)
{
	Enc.Buffer[Enc.Buf_wrpointer++] = enc_id;
	Enc.Buffer[Enc.Buf_wrpointer++] = enc_value;
	Enc.Buf_counter += 2;
	if (Enc.Buf_wrpointer >= 0x20)
	{
		Enc.Buf_wrpointer = 0;
	}	
}

//*************************************************************************
// read data from enc_buffer
//*************************************************************************
uint8_t rd_EncBuffer(void)
{
	uint8_t data = (Enc.Buffer[Enc.Buf_rdpointer++]);
	if (Enc.Buf_rdpointer >= 0x20)
	{
		Enc.Buf_rdpointer = 0;
	}
	return data;
}

//*************************************************************************
// Encoder query
//*************************************************************************
void encoder_query (void)
{
	uint8_t enc_id = rd_EncBuffer();
	uint8_t enc_value = rd_EncBuffer();
	Enc.Buf_counter -= 2;
		
	switch (enc_id)
	{
		case 0: Enc.value[0] = enc_value;
		Enc.status[0] = 1;
		page_nr = Enc.value[0];
		switch (page_nr)
		{
			case 0:draw_menupage_soundprg();
			break;
			case 1:draw_menupage_osc(0);
			break;
			case 2:draw_menupage_osc(1);
			break;
			case 3:draw_menupage_lfo(0);
			break;
			case 4:draw_menupage_lfo(1);
			break;
			case 5:draw_menupage_lfo(2);
			break;
			case 6:draw_menupage_env(0);
			break;
			case 7:draw_menupage_env(1);
			break;
			case 8:draw_menupage_filter1();
			break;
			case 9:draw_menupage_Fx();
			break;
			case 10:draw_menupage_seq();
			break;
			case 11:draw_menupage_scope();
			break;
			case 12:draw_menupage_sampling();
			break;
			case 13:draw_menupage_modulationmatrix();
			break;
			case 14:draw_menupage_system();
			break;
		}
		Enc.status[0] = 0;
		break;
		
		case 1: Enc.value_[1] = enc_value;
		Enc.status_[1] = 1;
		update_values_menupage();
		Enc.status_[1] = 0;
		break;
		
		case 2:	Enc.value_[2] = enc_value;
		Enc.status_[2] = 1;
		update_values_menupage();
		Enc.status_[2] = 0;
		break;
		
		case 3: Enc.value_[3] = enc_value;
		Enc.status_[3] = 1;
		update_values_menupage();
		Enc.status_[3] = 0;
		break;
		
		case 4: Enc.value_[4] = enc_value;
		Enc.status_[4] = 1;
		update_values_menupage();
		Enc.status_[4] = 0;
		break;
	}
}

//*************************************************************************
// select menupages
//*************************************************************************
 void update_values_menupage(void)
 {
	 // Page 1: Preset ----------------------------------------------------
	 if (page_nr == 0)
	 {
		 change_soundprg();
	 }
	 // Page 2: Oscillaor 1 -----------------------------------------------
	 if (page_nr == 1)
	 {
		 change_osc(0);
	 }
	 
	 // Page 3: Oscillator 2 -------------------------------------------
	 if (page_nr == 2)
	 {
		change_osc(1);
	 }
	 // Page 4: LFO 1 --------------------------------------------------
	 if (page_nr == 3)
	 {
		 change_lfo(0);
	 }
	 // Page 5: LFO2 ---------------------------------------------------
	 if (page_nr == 4)
	 {
		 change_lfo(1);
	 }
	 // Page 6: LFO3 ------------------------------------------------------
	 if (page_nr == 5)
	 {
		 change_lfo(2);
	 }
	 // Page 7: ENV1 ------------------------------------------------------
	 if (page_nr == 6)
	 {
		 change_env(0);
	 }
	 // Page 8: ENV2 ------------------------------------------------------
	 if (page_nr == 7)
	 {
		 change_env(1);
	 }
	 // Page 9: VCF -------------------------------------------------------
	 if (page_nr == 8)
	 {
		 change_filter1();
	 }
	 
	 // Page 10: FX Delay -------------------------------------------------
	 if (page_nr == 9)
	 {
		 change_Fx();
	 }
	 
	 // Page 11: SEQUENCER ------------------------------------------------
	 if (page_nr == 10)
	 {
		 change_seq();
	 }
	 
	 // Page 12: Scope ----------------------------------------------------
	 if (page_nr == 11)
	 {
		 change_scope();
	 }
	 
	 // Page 13: Sampling -------------------------------------------------
	 if (page_nr == 12)
	 {
		 if (save_flag == 0)
		 {
			 change_sampling();
			 
		 }
		 else
		 {
			 change_sample_save(0);
			 return;
		 }
	 }
	 
	 // Page 14: Modulationmatrix -----------------------------------------
	 if (page_nr == 13)
	 {
		 change_modulationmatrix();
	 }
	 
	 // Page 15: SYSTEM Parameters ----------------------------------------
	 if (page_nr == 14)
	 {
		 change_system();
	 }
	 
 }
	
//*************************************************************************
// key_query
//*************************************************************************
void get_keys(void)
{
	uint8_t temp_data;
	uint8_t key_mask;
	
	// read KEY1 ----------------------------------------------------------
	if (Key.key1_enable == 1)
	{
		temp_data = Key.KEY_DATA2;			// load shiftregister for Key-Group1
		key_mask = 0b11111011;				// set bitmask for Key1 "Sub-Menu"
		temp_data |= key_mask;
		if (temp_data == key_mask)
		{
			Key.key_nr = Key_SUB;			// Key1 (SUB)
			key_press();
		}
		else key_less(Key_SUB);
	}
	
	// read KEY2 ----------------------------------------------------------
	if (Key.key2_enable == 1)
	{
		temp_data = Key.KEY_DATA2;
		key_mask = 0b11110111;				// set bitmask for Key2
		temp_data |= key_mask;
		if (temp_data == key_mask)
		{
			Key.key_nr = Key_OSC;			// Key2 (Osc)
			key_press();
		}
		else key_less(Key_OSC);
	}
	
	// read KEY3 ----------------------------------------------------------
	if (Key.key3_enable == 1)
	{
		temp_data = Key.KEY_DATA2;
		key_mask = 0b11101111;				// set bitmask for Key3
		temp_data |= key_mask;
		if (temp_data == key_mask)
		{
			Key.key_nr = Key_VCF;			// Key3 is press (VCF)
			key_press();
		}
		else key_less(Key_VCF);
	}
	
	// read KEY4 ----------------------------------------------------------
	if (Key.key4_enable == 1)
	{
		temp_data = Key.KEY_DATA2;
		key_mask = 0b11011111;				// set bitmask for Key4
		temp_data |= key_mask;
		if (temp_data == key_mask)
		{
			Key.key_nr = Key_ENV;			// Key4 is press (Env)
			key_press();
		}
		else key_less(Key_ENV);
	}
	
	// read KEY5 ----------------------------------------------------------
	if (Key.key5_enable == 1)
	{
		temp_data = Key.KEY_DATA2;
		key_mask = 0b10111111;				// set bitmask for Key5
		temp_data |= key_mask;
		if (temp_data == key_mask)
		{
			Key.key_nr = Key_LFO;			// Key5 is press (LFO)
			key_press();
		}
		else key_less(Key_LFO);
	}
	
	// read KEY6 ----------------------------------------------------------
	if (Key.key6_enable == 1)
	{
		temp_data = Key.KEY_DATA2;
		key_mask = 0b01111111;				// set bitmask for Key6
		temp_data |= key_mask;
		if (temp_data == key_mask)
		{
			Key.key_nr = Key_MOD;			// Key6 is press (Modulationmatrix)
			key_press();
		}
		else key_less(Key_MOD);
	}
	
	// read KEY7 ----------------------------------------------------------
	if (Key.key7_enable == 1)
	{
		temp_data = Key.KEY_DATA3;
		key_mask = 0b11111110;				// set bitmask for Key7
		temp_data |= key_mask;
		if (temp_data == key_mask)
		{
			Key.key_nr = Key_LOAD;			// Key7 is press (Load)
			key_press();
		}
		else key_less(Key_LOAD);
	}
	
	// read KEY8 ----------------------------------------------------------
	if (Key.key8_enable == 1)
	{
		temp_data = Key.KEY_DATA3;
		key_mask = 0b11111101;				// set bitmask for Key8
		temp_data |= key_mask;
		if (temp_data == key_mask)
		{
			Key.key_nr = Key_SAVE;			// Key8 is press (Save)
			key_press();
		}
		else key_less(Key_SAVE);
	}
	// read KEY9 ----------------------------------------------------------
	if (Key.key9_enable == 1)
	{
		temp_data = Key.KEY_DATA3;
		key_mask = 0b11111011;				// set bitmask for Key9
		temp_data |= key_mask;
		if (temp_data == key_mask)
		{
			Key.key_nr = Key_DEL;			// Key9 is press (Delete)
			key_press();
		}
		else key_less(Key_DEL);
	}
	// read KEY10 ---------------------------------------------------------
	if (Key.key10_enable == 1)
	{
		temp_data = Key.KEY_DATA3;
		key_mask = 0b11110111;				// set bitmask for Key10
		temp_data |= key_mask;
		if (temp_data == key_mask)
		{
			Key.key_nr = Key_SEQ;			// Key10 is press (SEQ Start/Stop)
			key_press();
		}
		else key_less(Key_SEQ);
	}
	// read KEY11 ----------------------------------------------------------
	if (Key.key11_enable == 1)
	{
		temp_data = Key.KEY_DATA3;
		key_mask = 0b11101111;				// set bitmask for Key11
		temp_data |= key_mask;
		if (temp_data == key_mask)
		{
			Key.key_nr = Key_REC;			// Key11 is press (Rec)
			key_press();
		}
		else key_less(Key_REC);
	}
	// read KEY12 ----------------------------------------------------------
	if (Key.key12_enable == 1)
	{
		temp_data = Key.KEY_DATA3;
		key_mask = 0b11011111;				// set bitmask for Key12
		temp_data |= key_mask;
		if (temp_data == key_mask)
		{
			Key.key_nr = Key_ESC;			// Key12 is press (Esc)
			key_press();
		}
		else key_less(Key_ESC);
	}

}



//*************************************************************************
// key-press
//*************************************************************************
 void key_press(void)
 {
	 if (Key.key_debounce_time == 0)
	 {
		 if (Key.key_nr != Key.key_nr_old)
		 {
			 Key.key_nr_old = Key.key_nr;
			 Key.key_debounce_time = 0x80;
			 key_function();
		 }
	 }
 }
 
//*************************************************************************
// key-less
//*************************************************************************
 void key_less(uint8_t key_number)
 {
	 if (Key.key_nr_old == key_number)
	 {
		 if (Key.key_debounce_time == 0)
		 {
			Key.key_debounce_time = 0x80;
			Key. key_nr_old = 0;
			Key.key_nr = 0;
		 }
	 }
 }
 
 //************************************************************************
 // KEY-Function
 //************************************************************************
 void key_function(void)
 {
	//---------------------------------------------------------------------
	// save current Sample from Osc_Waveform
	//---------------------------------------------------------------------
	/*
	if (page_nr == 1 && Key.key_nr == Key_SAVE)
	{
		if (save_flag == 0)
		{
			Save_osc_waveform();
			save_flag = 1;
			_delay_ms(500);
			save_flag = 0;
		}
		
	}
	*/
	
	//---------------------------------------------------------------------
	// KEY 2 - 5   OSC/LFO/ENV/VCF
	//---------------------------------------------------------------------
	if (Key.key_nr == Key_OSC)
	{
		if (page_nr == 1)
		{
			page_nr = 2; Enc.value[0] = 2; Enc.status[0] = 1;
		}
		else
		{
			page_nr = 1; Enc.value[0] = 1; Enc.status[0] = 1;
		}	
	}
	if (Key.key_nr == Key_LFO)
	{
		if (page_nr == 3)
		{
			page_nr = 4; Enc.value[0] = 4; Enc.status[0] = 1;
		}
		else if (page_nr == 4)
		{
			page_nr = 5; Enc.value[0] = 5; Enc.status[0] = 1;
		}
		
		else
		{
			page_nr = 3; Enc.value[0] = 3; Enc.status[0] = 1;
		}
	}
	if (Key.key_nr == Key_ENV)
	{
		if (page_nr == 6)
		{
			page_nr = 7; Enc.value[0] = 7; Enc.status[0] = 1;
		}
		else
		{
			page_nr = 6; Enc.value[0] = 6; Enc.status[0] = 1;
		}
	}
	if (Key.key_nr == Key_VCF)
	{
		page_nr = 8; Enc.value[0] = 8; Enc.status[0] = 1;
	}	
	
	if (Key.key_nr == Key_MOD)
	{
		page_nr = 13; Enc.value[0] = 13; Enc.status[0] = 1;
	}

	
	  //*******************************************************************
	  // Page 1 "SOUND PRESET"
	  if (page_nr == 0)
	  {
		  // Key6 "load Preset" -------------------------------------------
		  if (Key.key_nr == Key_LOAD)
		  {
			  load_flag = 1;
			  SEQ.seq_stop_flag = 1;
			  load_preset_data();
			  load_flag = 0;
			  Sample.change_flag = 0;
			
		  }  
	 
		  // Key7 "Save Preset" -------------------------------------------
		  if (Key.key_nr == Key_SAVE)
		  {
			  SEQ.seq_stop_flag = 1;
			  draw_save_presetfile();
		  }
	   
	     // Key8 "Delete Preset" ------------------------------------------
		  else if (Key.key_nr == Key_DEL)
		  {
			   if (delete_flag == 1)
			   {
				   SEQ.seq_stop_flag = 1;
				   fillrectangle(70,70,231,111,color_white);
				   rectangle(72,72,36,156,color_black);
				   print_string("Delete File...",98,83,6,color_black);
				   delete_preset_data();
				   draw_menupage_soundprg();
				   delete_flag = 0;
				   enable_encoder(1,1,1,1,1);
				   enable_key(1,1,1,1,1,1,1,1,1,1,1,1,1);
			   }
			   else
			   {
				   fillrectangle(30,38,280,145,color_blue1);
				   rectangle(33,42,99,243,color_white);
				   fillrectangle(130,42,190,65,color_blue1);
				   print_string("Delete",138,39,6,color_white);
				   print_string("Delete PRG-File",90,62,6,color_white);
				   print_string("PRG:      BANK:    ",75,86,6,color_white);
				   fillrectangle(108,85,134,99,color_red);
				   fillrectangle(198,85,224,99,color_red);
				   draw_3digit_with(select_preset,110,86,6,color_white);
				   draw_3digit_with(select_presetbank,200,86,6,color_white);
				   print_string("Press delete or cancel",65,108,6,color_white);
				   delete_flag = 1;
				   Enc.enabled[0] = 1;
				   enable_encoder(0,0,0,0,0);
				   enable_key(0,0,0,0,0,0,0,0,1,0,0,1,0);
			   }			 
		  }
		
		// Key 2 "ESC function
		else if (Key.key_nr == Key_ESC)
		{
			if (cancle_flag == 0)
			{
				cancle_flag = 1;
			}
			
			if (delete_flag == 1 || save_flag == 1)
			{
				Enc.enabled[0] = 0;
				Enc.status_[0] = 0;
				delete_flag = 0;
				save_flag = 0;
				enable_encoder(1,1,1,1,1);
				enable_key(1,1,1,1,1,1,1,1,1,1,1,1,1);
				draw_menupage_soundprg();
			}
		}	
	 }
	  
	 //********************************************************************
	 // Page 2+3 "Oscillator1+2"
	 //********************************************************************
	 if (page_nr == 1)
	 {
		 oscillator_key_function(0);
	 }
	 
	 if (page_nr == 2)
	 {
		 oscillator_key_function(1);
	 }
	 	 
	 //********************************************************************
	 // page 7 "FILTER 1"
	 if (page_nr == 7)
	 {
		 // draw submenu
		 if (Key.key_nr == Key_SUB)
		 {
			 // next submenu
			 vcf1_submenu++;
			 if (vcf1_submenu > 1)
			 {
				 vcf1_submenu = 0;
			 }
			 //draw_menupage_filter1();
		 }
	 }
	 
	 //********************************************************************
	 // page 11 Sequencer
	 if (page_nr == 10)
	 {
		 /*
		 // draw sequencer submenu
		 if (Key.key_nr == Key_SUB)
		 {
			 // next submenu
			 seq_submenu++;
			 if (seq_submenu > 2)
			 {
				 seq_submenu = 0;
			 }
			 draw_menupage_seq();
		 }
		 
		 // load sequenz function
		 if (Key.key_nr == Key_LOAD)
		 {
			if (load_flag == 0)
			{
				SEQ.seq_run = 0;
				midi_gate_flag = 0;			// note off
				enable_encoder(0,1,0,0,0);
				enable_key(0,0,0,0,0,1,0,0,1,0,0,0,0);
				load_flag = 1;
				save_flag = 0;
				draw_seq_load_save_query();
			}
			else
			{
				load_save_seq();
				enable_encoder(1,1,1,1,1);
				enable_key(1,1,1,1,1,1,1,1,1,1,1,1,1);
				load_flag = 0;
				save_flag = 0;
				SEQ.seq_run = 1;
				SEQ.step_nr = 0;
				SEQ.running_step_nr = 0;
			}
			
		 }
		 
		 // save sequenz function
		 if (Key.key_nr == Key_SAVE)
		 {
			 if (save_flag == 0)
			 {
				SEQ.seq_run = 0;
				Enc.enc1_enabled = 1;
				load_flag = 0;
				save_flag = 1;
				draw_seq_load_save_query();
			 }
			 else
			 {
				 load_save_seq();
				 Enc.enc1_enabled = 0;
				 load_flag = 0;
				 save_flag = 0;
				 SEQ.seq_run = 0;
			 }
		 }
		 
		 // cancel function
		 if (Key.key_nr == Key_ESC)
		 {
			 if (save_flag == 1 || load_flag == 1)
			 {
				 draw_menupage_seq();
				 Enc.enc1_enabled = 1;
				 load_flag = 0;
				 save_flag = 0;
				 SEQ.seq_run = 0;
			 }
			  else // start/stop sequenzer
			  {

				  if (SEQ.seq_run == 0)
				  {
					  if (SEQ.seq_dir == 0 || SEQ.seq_dir == 2)
					  {
						  SEQ.step_nr = 0;				// set step_nr 0 for start
						  SEQ.running_step_nr = 0;
					  }
					  else if (SEQ.seq_dir == 1)
					  {
						  SEQ.step_nr = 15;				// set step_nr 15 for start
						  SEQ.running_step_nr = 15;
					  }
					  SEQ.seq_speed_temp = 0;
					  SEQ.seq_run = 1;					// start sequencer
					  midi_trans = 0;
					  SEQ.selected_step_nr = 0;
					  if (SEQ.seq_dir == 1)
					  {
						  SEQ.step_nr = 15;
					  }
					  else SEQ.step_nr = 0;
				  }
				  
			  }
		 }
	*/
	 }
	  // Start/Stop sequencer
	  if (Key.key_nr == Key_SEQ)
	  {
		  if (SEQ.seq_run == 0)
		  {
			  //SEQ.seq_pulswidth_temp = 0;
			 // midi_gate_flag = 0;
			  SEQ.seq_run = 1;
			  SEQ.seq_stop_flag = 0;
			  SEQ.arp_step = 0;
			  SEQ.arp_octave_ = 0;
		  }
		  else
		  {
			  SEQ.seq_stop_flag = 1;
		  }
	  }
	 
	  //*******************************************************************
	  // page 13 "Sampling"
	  if (page_nr == 12)
	  {
		  // draw submenu -------------------------------------------------
		  if (Key.key_nr == Key_SUB)
		  {
			  // next submenu
			  Gui.sam_submenu++;
			  if (Gui.sam_submenu > 2)
			  {
				  Gui.sam_submenu = 0;
			  }
			  draw_menupage_sampling();
		  }
		  
		  // cut wave-file ------------------------------------------------
		  else if (Gui.sam_submenu == 1 && Gui.sample_edit_function == 1 && Key.key_nr == Key_DEL)
		  {
			  sample_edit_cut(0);
		  }
		  
		  // esc in sample editor -----------------------------------------
		  else if (Gui.sam_submenu == 1 && Key.key_nr == Key_ESC)
		  {
				Gui.cut_flag = 0;
				Gui.del_flag = 0;
				init_keys();
				draw_zoom_sample(Gui.Zoom_value);
				draw_cursor_A();
				draw_cursor_B();
				vline(Gui.CurA_value + 50, 29, 128, color_white);
				vline(Gui.CurB_value + 50, 29, 128, color_white);
		  }
		  
		  // Quit wave-file -----------------------------------------------
		  else if (Gui.sam_submenu == 1 && Gui.sample_edit_function == 0 && Key.key_nr == Key_DEL)
		  {
			  sample_edit_quit();
		  }
		  
		   // Reverse wave-file -------------------------------------------
		   else if (Gui.sam_submenu == 1 && Gui.sample_edit_function == 3 && Key.key_nr == Key_DEL)
		   {
			   sample_edit_reverse();
		   }
		  
		  // load wave-file -----------------------------------------------
		  else if (Gui.sam_submenu == 2 && Key.key_nr == Key_LOAD)
		  {
			  Osc.wave_mode[0] = 1;	// Osc1 sample mode
			  Osc.wave_mode[1] = 2; // Osc2 off mode
			  Osc.mix_value = 0;
			  Osc.op_value = 0;
			  Env.attack[0] = 0;
			  Env.attack[1] = 0;
			  Env.decay[0] = 0;
			  Env.decay[1] = 0;
			  Env.sustain[0] = 127;
			  Env.sustain[1] = 127;
			  Env.release[0] = 35;
			  Env.release[1] = 35;
			  Osc.loop_function[0] = 0;
			  Osc.sync_function[0] = 1;
			  Fx.Value = 0;
			  Xram.sample_length[0] = 0;
			  Xram.sample_length[1] = 0;
			  Gui.CurA_addr = 0;
			  Gui.CurB_addr = 0;
			  Osc.loop_A[0] = 0;
			  Osc.loop_B[0] = 0;
			  Gui.sample_edit_function = 1;
			  wavefile_flag = 1;
			  load_wavefile(0);
			  Gui.sam_submenu = 1;
			  draw_menupage_sampling();
			  Gui.redraw_flag2 = TRUE;
			  
		  }
		  // delete wave-file ---------------------------------------------
		  else if (Gui.sam_submenu == 2 && Key.key_nr == Key_DEL)
		  {
			  enable_encoder(0,0,0,0,0);
			  enable_key(0,0,0,0,0,0,0,0,1,0,0,1,0);
			  draw_delete_wavefile(0);
		  }
		  else if (Gui.sam_submenu == 2 && Key.key_nr == Key_ESC)
		  {
			  draw_menupage_sampling();
			  enable_encoder(1,1,1,1,1);
			  enable_key(1,1,1,1,1,1,1,1,1,1,1,1,1);
			  save_flag = 0;
			  delete_flag = 0;
		  }
		  
		  // sample_record start/stop/ ------------------------------------
		  else if (Gui.sam_submenu == 0 && Key.key_nr == Key_REC)
		  {	  
			// Recmode: Key
			if (Gui.Sam_rec_mode == 2 && Gui.Sam_rec_key_flag == 0)
			{
				init_sample_recording();
				Gui.Sam_rec_flag = 1;
				draw_rec_symbol();
				Gui.Sam_rec_key_flag = 1;				
			}
			// Recmode: Threshold
			else if (Gui.Sam_rec_mode == 1 && Gui.Sam_rec_key_flag == 0)
			{
				init_sample_recording();
				Gui.Sam_rec_flag = 0;
				Gui.Sam_rec_key_flag = 1;
			}
			// Recmode: Midi Note
			else if (Gui.Sam_rec_mode == 0 && Gui.Sam_rec_key_flag == 0)
			{
				init_sample_recording();
				Gui.Sam_rec_key_flag = 1;
			}
			
			else if (Gui.Sam_rec_key_flag == 1 || Gui.Sam_rec_flag == 1)
			{
				stop_sampling();			
			}  
		  }
		  
		  // save sample --------------------------------------------------
		  if (Key.key_nr == Key_SAVE)
		  {			
			if (save_flag == 0)
			{
				Gui.sam_submenu = 2;
				draw_save_file();
			}
			else
			{
				// stop sound_buffer Interrupt for fast loading
				TCC0.INTCTRLA = 0b00000000;
				// stop Osc_Render Task
				TCC1.INTCTRLA = 0b00000000;
				save_wave_file(0);
				save_flag = 0;
				init_keys();
				Gui.Sam_old_entry = 0;
				draw_menupage_sampling();
				// run sound_buffer Interrupt
				TCC0.INTCTRLA = 0b00000011;
				// run Osc_Render Task
				TCC1.INTCTRLA = 0b00000010;
			}
					
		  }
	  }
	 
	 //********************************************************************
	 // page 13 "MODULATIONS-MATRIX"
	 if (page_nr == 13)
	 {
		 if (Key.key_nr == Key_SUB)
		 {
			nop();
		 }
	 }
	 
 }
 

//*************************************************************************
//	Sample cut function
//*************************************************************************
void sample_edit_cut(uint8_t osc_nr)
{
	if (Gui.cut_flag == 1)
	{
		cut_sample();
		Gui.CurB_addr = Gui.CurA_addr;
		Gui.CurB_value = Gui.CurA_value;
		Enc.value[1] = Gui.CurA_value;
		Enc.value[2] = Gui.CurA_value;
		draw_zoom_sample(Gui.Zoom_value);
		// calc sample block_size
		/*
		Gui.block_size = (__uint24)(Xram.sample_length[osc_nr] >> 8) >> Gui.Zoom_value;
		Gui.CurA_value = (__uint24)(Gui.CurA_addr / Gui.block_size);
		Gui.CurB_value = Gui.CurA_value;
		Enc.value[1] = Gui.CurA_value;
		Enc.value[2] = Gui.CurA_value;
		*/
		set_encoder_val(Gui.CurA_value,Gui.CurA_value,Gui.Zoom_value,Gui.sample_edit_function);
		draw_6digit_with(Gui.CurA_addr,24,175,2,color_white);
		draw_6digit_with(Gui.CurB_addr,104,175,2,color_white);
		sample_editor_func();
		//draw_cursor_A();
		//draw_cursor_B();
		//vline(Gui.CurA_value + 50, 29, 128, color_white);
		//hline(43,158,6,color_black); // delete line pixel
		draw_char('A',42+Gui.CurA_value,158,2,color_white);
		draw_char('B',49+Gui.CurA_value,158,2,color_white);
		init_keys();
		Gui.cut_flag = 0;
	}
	else
	{
		enable_encoder(0,0,0,0,0);
		enable_key(0,0,0,0,0,0,0,0,1,0,0,1,0);
		fillrectangle(49,49,270,137,color_red);
		rectangle(51,51,83,216,color_white);
		print_string("Cut Sample",112,58,6,color_white);
		print_string("Press delete for cutting",63,90,6,color_white);
		print_string("or cancel for abort.",63,105,6,color_white);
		Gui.cut_flag = 1;
	}
}

//*************************************************************************
//	Sample edit quit function
//*************************************************************************
void sample_edit_quit(void)
{
	if (Gui.del_flag == 1)
	{
		Gui.del_flag = 0;
		silence_sample();
		draw_zoom_sample(Gui.Zoom_value);
		draw_cursor_A();
		draw_cursor_B();
		vline(Gui.CurA_value + 50, 29, 128, color_white);
		vline(Gui.CurB_value + 50, 29, 128, color_white);
		draw_char('A',42+Gui.CurA_value,158,2,color_white);
		draw_char('B',49+Gui.CurB_value,158,2,color_white);
		init_keys();
	}
	else
	{
		Gui.del_flag = 1;
		enable_encoder(0,0,0,0,0);
		enable_key(0,0,0,0,0,0,0,0,1,0,0,1,0);
		// info
		fillrectangle(49,49,270,137,color_red);
		rectangle(51,51,83,216,color_white);
		print_string("Mute sample",112,58,6,color_white);
		print_string("Press delete for quit",63,90,6,color_white);
		print_string("or cancel for abort.",63,105,6,color_white);
	}
}

//*************************************************************************
// Reverse sample
//*************************************************************************
void sample_edit_reverse(void)
{
	reverse_sample();
	draw_zoom_sample(Gui.Zoom_value);
	draw_cursor_A();
	draw_cursor_B();
	vline(Gui.CurA_value + 50, 29, 128, color_white);
	vline(Gui.CurB_value + 50, 29, 128, color_white);
	draw_char('A',42+Gui.CurA_value,158,2,color_white);
	draw_char('B',49+Gui.CurB_value,158,2,color_white);
}


//*************************************************************************
//	wait after start
//*************************************************************************
void init_keys(void)
{
	enable_encoder(1,1,1,1,1);
	enable_key(1,1,1,1,1,1,1,1,1,1,1,1,1);
	
	// load encoder dynamic
	Enc.dynamic_ = EEPReadByte(ee_enc_dynamic);
}

//*************************************************************************
// enable ore disabled encoder   enabled=1  disabled=0
//*************************************************************************
void enable_encoder(uint8_t enc1, uint8_t enc2, uint8_t enc3, uint8_t enc4, uint8_t enc5)
{
	Enc.enabled[0] = enc1;
	Enc.enabled[1] = enc2;
	Enc.enabled[2] = enc3;
	Enc.enabled[3] = enc4;
	Enc.enabled[4] = enc5;
}

//*************************************************************************
// enable ore disabled key   enabled=1  disabled=0
//*************************************************************************
void enable_key(uint8_t key1,uint8_t key2,uint8_t key3,uint8_t key4,uint8_t key5,uint8_t key6,
uint8_t key7,uint8_t key8,uint8_t key9,uint8_t key10,uint8_t key11,uint8_t key12,uint8_t key13)
{
	Key.key1_enable = key1;
	Key.key2_enable = key2;
	Key.key3_enable = key3;
	Key.key4_enable = key4;
	Key.key5_enable = key5;
	Key.key6_enable = key6;
	Key.key7_enable = key7;
	Key.key8_enable = key8;
	Key.key9_enable = key9;
	Key.key10_enable = key10;
	Key.key11_enable = key11;
	Key.key12_enable = key12;
	Key.key13_enable = key13;
}

//*************************************************************************
// init Sample recording
//*************************************************************************
void init_sample_recording(void)
{
	quantization_error2 = 0;
	REC_LED_on;
	enable_encoder(0,0,0,0,0);
	enable_key(0,0,0,0,0,0,0,0,0,0,1,0,0);
	fillrectangle(140,68,297,130,color_black);	// clear Wave line
	hline(140, 110,157, color_orange);
	fillrectangle(175,140,199,152,color_black);	// clear Ram Address
	draw_3digit_with(0,175,140,6,color_white);
	LEDBAR.xpos = 0;
	sampling_wr_addr = 0;
	Xram.sample_length[0] = 0;
	Xram.sample_length[1] = 0;
	Osc.loop_A[0] = 0;
	Osc.loop_B[0] = 0;
	Osc.loop_A[1] = 0;
	Osc.loop_B[1] = 0;
	Gui.CurA_addr = 0;
	Gui.CurB_addr = 0;
	Osc.wave_mode[0] = 1;
	Osc.wave_mode[1] = 2; // Osc2 off
	Gui.sample_edit_function = 1;
	wavefile_flag = 1;
}

//*************************************************************************
// Oscillator key_function
//*************************************************************************
void oscillator_key_function(uint8_t osc_nr)
{
	if (Key.key_nr == Key_SUB)
	{
		if (Osc.wave_mode[osc_nr] != 2)
		{
			// next submenu
			Gui.osc_submenu[osc_nr]++;
			if (Osc.wave_mode[osc_nr] == 0)
			{
				if (Gui.osc_submenu[osc_nr] > 1)
				{
					Gui.osc_submenu[osc_nr] = 0;
				}
			}
			else if (Osc.wave_mode[osc_nr] == 1)
			{
				if (Gui.osc_submenu[osc_nr] > 2)
				{
					Gui.osc_submenu[osc_nr] = 0;
				}
			}
			else if (Osc.wave_mode[osc_nr] == 2)
			{
				Gui.osc_submenu[osc_nr] = 0;
			}
			draw_menupage_osc(osc_nr);
		}
	}
	
	
	// load sample-file ----------------------------------------------
	else if (Gui.osc_submenu[osc_nr] == 1 && Osc.wave_mode[osc_nr] == 1 && Key.key_nr == Key_LOAD)
	{
		draw_load_wavefile(osc_nr);
		Gui.osc_submenu[osc_nr] = 0;
		draw_menupage_osc(osc_nr);
	}
	
	// delete wave-file ----------------------------------------------
	else if (Gui.osc_submenu[osc_nr] == 1 && Osc.wave_mode[osc_nr] == 1 && Key.key_nr == Key_DEL)
	{
		enable_encoder(0,0,0,0,0);
		enable_key(0,0,0,0,0,0,0,0,1,0,0,1,0);
		draw_delete_wavefile(osc_nr);
	}
	
	// save wave-file -----------------------------------------------------
	else if (Gui.osc_submenu[osc_nr] == 1 && Osc.wave_mode[osc_nr] == 1 && Key.key_nr == Key_SAVE)
	{
		
			if (save_flag == 0)
			{
				//Gui.osc_submenu = 2;
				draw_save_file();
			}
			else
			{
				// stop sound_buffer Interrupt for fast loading
				TCC0.INTCTRLA = 0b00000000;
				// stop Osc_Render Task
				TCC1.INTCTRLA = 0b00000000;
				save_wave_file(osc_nr);
				save_flag = 0;
				Gui.Sam_old_entry = 0;
				draw_menupage_osc(osc_nr);
				// run sound_buffer Interrupt
				TCC0.INTCTRLA = 0b00000011;
				// run Osc_Render Task
				TCC1.INTCTRLA = 0b00000010;
			}		
	}
	
	// cancel save/delete function
	else if (Osc.wave_mode[osc_nr] == 1 && Key.key_nr == Key_ESC)
	{
		draw_menupage_osc(osc_nr);
		save_flag = 0;
		delete_flag = 0;
		Gui.cut_flag = 0;
		Gui.del_flag = 0;
	}
	
	// Sampel editor: cut wave-file ---------------------------------------
	else if (Gui.osc_submenu[osc_nr] == 2 && Gui.sample_edit_function == 1 && Key.key_nr == Key_DEL)
	{
		sample_edit_cut(osc_nr);
	}
	// Sample editor: quit wave-file --------------------------------------
	else if (Gui.osc_submenu[osc_nr] == 2 && Gui.sample_edit_function == 0 && Key.key_nr == Key_DEL)
	{
		sample_edit_quit();
	}
	// Sample editor reverse wave-file ------------------------------------
	else if (Gui.osc_submenu[osc_nr] == 2 && Gui.sample_edit_function == 3 && Key.key_nr == Key_DEL)
	{
		sample_edit_reverse();
	}
}


