//*************************************************************************
//  LFO.c
//	timer0 interrupt 25usec
//
//  Created: 23.06.2016 20:34:16
//  Author: Rolf Degen
//*************************************************************************

#include "globals.h"
#include "lfo.h"
#include "wavetable.h"
#include "oscillators.h"
#include "gui.h"




//*************************************************************************
// LFO-Task
//*************************************************************************
void LFO_Task (void)
{
		
	// LFO1-3 Task --------------------------------------------------------
	uint8_t shape;
	uint16_t phase;
	uint16_t phase_inc;
	uint16_t addr;
	//uint8_t lfo_sample;
	uint8_t run_status;
	uint8_t oneshoot_status;
	
	for (uint8_t lfo_nr = 0; lfo_nr < 3; lfo_nr++)
	{
		shape = LFO.shape[lfo_nr];
		phase = LFO.phase[lfo_nr];
		phase_inc = LFO.rate[lfo_nr];
		run_status = LFO.run_flag[lfo_nr];
		oneshoot_status = LFO.oneshoot_flag[lfo_nr];
			
		if (run_status == 1)
		{
			phase += phase_inc;
			
			if ((oneshoot_status == 1) && (phase < phase_inc))
			{
				run_status = 0;
				LFO.run_flag[lfo_nr] = run_status;
			}
			else
			{
				// load wave and set lfo_out
				if (LFO.shape[lfo_nr] == 17) // S&H function
				{
					if (phase < phase_inc)
					{
						LFO_sample_out(lfo_nr, Noise.sample);
					}
				}
				else
				{
					addr = (shape << 8) | (phase >> 8);
					LFO_sample_out(lfo_nr, pgm_read_byte(&LFO_waves[(addr)]));
				}
				
				// update LFO.phase and LFO-Led
				LFO.phase[lfo_nr] = phase;
				LFO.run_flag[lfo_nr] = run_status;
			}
		}
		else
		{
			switch (lfo_nr)
			{
				
				case 0:	LFO1_LED_off; Gui.Led1_time = 0;
				break;
				case 1:	LFO2_LED_off; Gui.Led2_time = 0;
				break;
				case 2:	LFO3_LED_off; Gui.Led3_time = 0;
				break;
				
			}
		}
	}		

}

void LFO_sample_out(uint8_t lfo_nr, uint8_t sample)
{
	switch (lfo_nr)
	{
		case 0:	LFO.out[0] = sample;
		Gui.Led1_time = sample;
		break;
		case 1:	LFO.out[1] = sample;
		Gui.Led2_time = sample;
		break;
		case 2:	LFO.out[2] = sample;
		Gui.Led3_time = sample;
		break;
	}
}

