//*************************************************************************
//  AVR-Synthesizer "DE-GENERATOR" Version 3.012   16.02.2017
//  -----------------------------------------------------------------------
//  2x DCO / 2x VCF / 2x VCA / 3x LFO / 2x Envelope / 1MByte ext.SRAM
//  and TFT 3.2" SainSmart 320x240  Controller-Typ: SSD1289
//
//	ATxmega128A1 with 32MHz
//
//  Created: 12.2012
//  Author:  Rolf Degen  email: rolfdegen@hotmail.com
//
//*************************************************************************





// include files
#include "globals.h"
#include "init_globals.h"			
#include "display/displ_funktion.h"
#include "wavetable.h"
#include "SDCARD/fat.h"
#include "SDCARD/file.h"
#include "SDCARD/mmc.h"
#include "SDCARD/mmc_config.h"
#include "SDCARD/sd_card.h"


// new include files
#include "lfo.h"
#include "sequencer.h"
#include "miniscope.h"
#include "oscillators.h"
#include "voice.h"
#include "init_system.h"
#include "gui.h"
#include "modulation.h"
#include "patch.h"
#include "midi.h"
#include "envelope.h"
#include "op.h"


//*************************************************************************
// Midi-Receive Interrupt
//*************************************************************************
ISR(USARTF0_RXC_vect)
{
	 midibuf[midibuf_wr_pointer] = USARTF0.DATA;	// Save Midi-Data to Buffer (Buffersize: 128 Byte)
	 midibuf_wr_pointer++;							// inc Buffer-Pointer
	 if (midibuf_wr_pointer > 0x80)
	 {
		 midibuf_wr_pointer = 0x00;
	 }
}

//*************************************************************************
// Timer1-Interrupt 1msec
// Midi, Envelopes, LFOs, Key_query and Leds
//*************************************************************************
ISR(TCC1_OVF_vect)
{
	TCC1.INTCTRLA = 0b00000000;			// stop timer1 interrupt
	
	//PORTA.OUTSET = PIN7_bm;
	
	//Midi data avaible ---------------------------------------------------
	if(mididata_avail() == 1)
	{
		midi_handle();
	}
	
	// Sequencer / ARP Task -----------------------------------------------
	sequencer_task();
	
	// Envelope Task ------------------------------------------------------
	Envelope_Task();
	
	// Modulation Matrix Task ---------------------------------------------
	if (Gui.Sam_monitor_flag == 0)
	{
		modulation_matrix();
	}
	
	// read Encoder and Key Port ------------------------------------------
	static uint8_t hc165_time = 2;
	hc165_time--;
	if (hc165_time == 0)
	{
		read_74hc165();
		hc165_time = 2;
	}
	
	// read encoder status ------------------------------------------------
	read_enc_status();
	
	// LFO-Task -----------------------------------------------------------
	LFO_Task();
	
	// LED-Task -----------------------------------------------------------
	LED_Task();	
	
	//-----------------------------------------------------------------
	// Debounce_Timer for Key-Query
	//-----------------------------------------------------------------	
	if (Key.key_debounce_time >0)
	{
		Key.key_debounce_time--;
	}

	Key.query_flag = 1;
	
	//---------------------------------------------------------------------
	// Status Check and waittime for SDCARD 10msec 
	//---------------------------------------------------------------------
	SDCARD.delay_time++;
	if (SDCARD.delay_time == 10)
	{
		TimingDelay = (TimingDelay==0) ? 0 : TimingDelay-1;
		SDCARD.delay_time = 0;
		sdcard_status();
	}
	
	//---------------------------------------------------------------------
	// soft-timer0 (screen refresh for mini scope function)
	//---------------------------------------------------------------------
	if (soft_timer0 == 5)
	{
		soft_timer0_flag = 1;
	}
	soft_timer0++;
	
	//---------------------------------------------------------------------
	// Counter for Real-Env. curve on page7+8
	//---------------------------------------------------------------------
	if (Gui.real_env_flag == 1)
	{
		if (Gui.real_env_time == 32)
		{
			Gui.real_env_time = 0;
			Gui.real_env_flag = 0;
		}
		Gui.real_env_time++;
	}
	
	//---------------------------------------------------------------------
	// soft_timer 10 (Led graph in Sample Editor)
	//---------------------------------------------------------------------
	if (Gui.time1 > 0)
	{
		Gui.time1--;
	}
	else Gui.time1_flag = 1;
	
	//---------------------------------------------------------------------
	// clear midi input_symbol
	//---------------------------------------------------------------------
	if (Mid.symbol_time > 0)
	{
		Mid.symbol_time--;
		if (Mid.symbol_time == 1)
		{
			Mid.symbol_off_flag = 1;
		}
	}
	
	//---------------------------------------------------------------------
	// scope wait timer
	//---------------------------------------------------------------------
	if (Scope.scope_wait_time > 0)
	{
		Scope.scope_wait_time--;
	}
	else Scope.scope_wait_flag = 0;
	
	if (Scope.trigger_out_time < 900)
	{
		Scope.trigger_out_time++;
	}
	
	
	
	//---------------------------------------------------------------------
	// update poti1+2
	//---------------------------------------------------------------------	
	if (poti_mux == 0)
	{
		poti1_value = ADCB_CH0RES;
		poti_mux = 1;
		ADCB.CH0.MUXCTRL = ADC_CH_MUXPOS_PIN1_gc;
	}
	else
	{
		poti2_value = ADCB_CH0RES;
		poti_mux = 0;
		ADCB.CH0.MUXCTRL = ADC_CH_MUXPOS_PIN0_gc;
	}
	
	//PORTA.OUTCLR = PIN7_bm;
		
	TCC1.INTCTRLA = 0b00000010;			// set timer1 interrupt
}

//*************************************************************************
// Software interrupt (fill voice_buffer)
//*************************************************************************
ISR(TCE1_OVF_vect)
{
	// set measuring point
	//PORTA.OUTSET = PIN6_bm;
	
	// stop timer interrupt	
	TCE1.INTCTRLA = 0b00000000;			// stop interrupt
	
	Osc_Render(0);
	Osc_Render(1);
	Voice_Buffer();
	
	// clear measuring point
	//PORTA.OUTCLR = PIN6_bm;
}

//*************************************************************************
// Timer0 Interrupt: Audio_Out Task 40KHz/25usec
//*************************************************************************
ISR(TCC0_OVF_vect)
{
	// set measuring point
	//PORTA.OUTSET = PIN7_bm;
	
	// stop global interrupt
	cli();
	
	// BPM clock ----------------------------------------------------------
	if (SEQ.seq_run == 1)
	{
		if (SEQ.seq_speed_temp > 0)
		{
			SEQ.seq_speed_temp--;
		}
		else
		{
			SEQ.seq_speed_temp = ((uint32_t)(19200000/(SEQ.seq_bpm * SEQ.seq_div))-1);
			SEQ.seq_clock_flag = 1;
		}
	}
	
	// noise generator ----------------------------------------------------
	static uint8_t noise_task = 2;
	noise_task--;
	if (noise_task == 0)
	{
		Noise.data_rng_state = (Noise.data_rng_state >> 1) ^ (-(Noise.data_rng_state & 1) & 0xb400);
		Noise.sample = Noise.data_rng_state >> 8;
		noise_task = 2;
	}	
	
	// recording with dithering (noise reduction) -------------------------
	int16_t sample = ADCA_CH0RES;
	__int24 sample_16 = (sample * 16);
	sample_16 += quantization_error2;
	int8_t sample_8 = sample_16 >> 8;
	quantization_error2 = sample_16 - ((__int24)sample_8 << 8);
	
	
	// Sample recording ---------------------------------------------------
	if (page_nr == 12 && Gui.sam_submenu == 0)
	{
		// sampling monitor -----------------------------------------------
		if (Gui.Sam_monitor_flag == 1)
		{
			DACA.CH0DATAH = sample_8;
			DACB.CH0DATAH = sample_8;
		}
		
		// sampling rec-mode: "Threshold" -------------------------------------
		if (Gui.Sam_rec_mode == 1 && Gui.Sam_rec_key_flag == 1 && Gui.Sam_rec_flag == 0)
		{
			uint8_t temp = (uint8_t)sample_8;
			if (temp > (Sampling_threslev + 128))
			{
				Gui.Sam_rec_flag = 1;
				Gui.rec_led_flag = 1;
			}
		}
		
		// write Sample into 1MByte XRAM --------------------------------------
		if (Gui.Sam_rec_flag == 1)
		{
			hugemem_ptr_t p = BOARD_EBI_SRAM_BASE;
			hugemem_write8(p+sampling_wr_addr, sample_8);
			sampling_wr_addr++;
			
			if (sampling_wr_addr > 0xef1df) // //Xram.max_space = 0xef1df;
			{
				stop_sampling();
			}
		}
	}
	
	// play sound from buffer if sampling-mode disabled -------------------
	if (Gui.Sam_monitor_flag == 0 && Gui.Sam_rec_flag == 0 && Gui.Sam_rec_key_flag == 0)
	{
		// read samples from buffers
		if (Voice.buffer_nr == 0)
		{
			// 8Bit (DAC Channel is change for smr4 filter)
			DACB.CH0DATAH = Voice.Buffer1a[Voice.buf_read];
			DACA.CH0DATAH = Voice.Buffer1b[Voice.buf_read];
			Voice.Buffer3a[Voice.buf_read] = Noise.sample;
		}
		else
		{
			// 8Bit (DAC Channel is change for smr4 filter)
			DACB.CH0DATAH = Voice.Buffer2a[Voice.buf_read];
			DACA.CH0DATAH = Voice.Buffer2b[Voice.buf_read];
			Voice.Buffer3b[Voice.buf_read] = Noise.sample;
		}
		
		// inc buffer and change buffer_nr --------------------------------
		Voice.buf_read++;
		
		if (Voice.buf_read >= 40)
		{
			Voice.buf_read = 0;
			
			// toggle buffer_nr
			Voice.buffer_nr ^= 0b00000001;
			
			// set buffer_fill interrupt
			TCE1.INTCTRLA = 0b00000010;
		}
	}


	// MiniScope with trigger function ------------------------------------
	uint8_t temp = Scope.timebase_reg + Scope.timebase_level;
	
	if (temp < Scope.timebase_reg && Scope.adc_trigger <= 1)
	{
		// trigger off_automatic
		if (Scope.adc_trigger_level == 0 || Scope.adc_trigger == 1 || page_nr == 12 || Scope.trigger_out_time == 900)	// if trigger off  ore sampling page ?
		{																	// than recording scope-samples
			Scope.adc_buffer[Scope.adc_wr_pointer] = sample_8;
			Scope.adc_wr_pointer++;
			if (Scope.adc_wr_pointer == 151)
			{
				Scope.adc_wr_pointer = 0;
				Scope.adc_trigger = 99;
			}
		}
	}
	Scope.timebase_reg = temp;

	// set global interrupt

	sei();
	
	// clear measuring point
	//PORTA.OUTCLR = PIN7_bm;

}

//*************************************************************************
// Analog Comparator0 Interrupt (priority low)
// for MiniScope funktion
//*************************************************************************
ISR(ACA_AC0_vect)
{
	if (Scope.adc_trigger == 0)
	{
		Scope.adc_trigger = 1;
	}
	Scope.trigger_out_time = 0;

}


//*************************************************************************
// init System
//*************************************************************************
int main(void)
{
	init_clock();				// init CPU-Clock to 32MHz
	init_USART();				// init usart for midi
	init_TIMER();				// init Timer0 Audio
	init_PORT();				// init I/O Ports
	init_PWM();					// init PWM_CV Control
	init_SPI();					// init SPI-Register
	init_DAC();					// init DAC's
	init_ADC();					// init ADC
	init_XRAM();				// init 1MByte XRAM
	init_LCD();					// init GLCD
	init_noise();				// init noise/randum generator
	init_Interrupt();			// init all Interrupt's
	init_SDCARD();				// init SDCard
	clear_midi_notestack();		// init Midi notestack
	//draw_startscreen();			// draw Start Screen
	init_parameter();			// init Parameter
	load_shape_bin();			// load shape-curves for screen
	init_Menu_Page();			// load clear menu page
	init_Patch();				// load first/current patch
	init_Scope();				// init scope function
	init_Voice();				// init Voice Buffer
	init_keys();				// enable all keys and encoders
	
							
//*************************************************************************
//                             Main Loop
//*************************************************************************
	while(1)
	{		
		//-----------------------------------------------------------------
		// encoder query
		// ----------------------------------------------------------------
		if(Enc.Buf_counter >= 2)
		{
			encoder_query();
		}
		
		//-----------------------------------------------------------------
		// key query
		// ----------------------------------------------------------------
		if (Key.query_flag == 1)
		{
			Key.KeyQueryFlag = 1;
			get_encoder();
			get_keys();
			Key.KeyQueryFlag = 0;
			Key.query_flag = 0;
		}
		
		//-----------------------------------------------------------------
		// plot wave scope/mini-scope funktion
		//-----------------------------------------------------------------
		if (soft_timer0_flag == 1 && sampling_flag == 0)
		{
			if (page_nr == 0 || page_nr == 8 || page_nr == 11 )
			{
				draw_scope_line();
			}
			
			if (page_nr == 1 && Gui.osc_submenu[0] == 0)
			{
				draw_scope_line();
			}
			
			if (page_nr == 2 && Gui.osc_submenu[1] == 0)
			{
				draw_scope_line();
			}
		}
			
		//-----------------------------------------------------------------
		// draw recording sample
		//-----------------------------------------------------------------
		if (page_nr == 12 && Gui.sam_submenu == 0 && Gui.time1_flag == 1)
		{
			draw_sampling_curve();
			if (Gui.rec_led_flag == 1)
			{
				draw_rec_symbol();
				Gui.rec_led_flag = 0;
			}
		}	
					
		//-----------------------------------------------------------------
		// draw real enveloop curve
		//-----------------------------------------------------------------
		if (page_nr == 6 || page_nr == 7)
		{
			
			
			//if (sampling_flag == 0)
			//{
				draw_real_env();
			//}
		}
		
		//-----------------------------------------------------------------
		// draw midi symbol
		//-----------------------------------------------------------------
		if (Mid.input_flag != 0 && Mid.symbol_time == 0)
		{
			draw_symbol(1,272,6,color_white);
			rectangle(270,3,13,11,color_white);
			Mid.input_flag = 0;
			Mid.symbol_time = 200;
		}
		
		if (Mid.symbol_off_flag != 0)
		{
			draw_symbol(1,272,6,color_gray);
			rectangle(270,3,13,11,color_gray);
			Mid.symbol_off_flag = 0;
		}
		
		//-----------------------------------------------------------------
		// draw notes in rectangle
		//-----------------------------------------------------------------
		if (page_nr == 10 && SEQ.draw_note_flag != 0)
		{
			draw_sequencer_notes();
		}
		if (page_nr == 10 && SEQ.draw_step_flag != 0)
		{
			draw_sequencer_rectangle();
		}
		if (page_nr == 10 && SEQ.arp_sort_flag != 0)
		{
			sort_notes(SEQ.arp_note_size);
		}
				
	}
}
 
//******************************* END *************************************


//-------------------------------------------------------------------------
//Test Routines
//-------------------------------------------------------------------------
/*
fillrectangle(0,0,100,20,color_black);
draw_5digit_with(note_,0,0,2,color_white);
*/

