/*
 * midi.c
 *
 * Created: 23.07.2016 09:34:33
 *  Author: rolfd
 */ 

#include "midi.h"

//-------------------------------------------------------------------------
// clear midi note_stack
//-------------------------------------------------------------------------
void clear_midi_notestack(void)
{
	// clear midi note_stack_buffer
	for (uint8_t i = 0; i <= NoteStack_size; i++)
	{
		NoteStack_pool[i] = 0xFF;
		VeloStack_pool[i] = 0xFF;
	}
	NoteStack_ptr = 0;
}

//-------------------------------------------------------------------------
// Check data available in midi buffer
// This function is call in Timer0 Interrupt-Routine (all 1msec)
// is available = 1 then is call midi_handle routine
//-------------------------------------------------------------------------
uint8_t mididata_avail(void)
{
	uint8_t available;
	if(midibuf_wr_pointer != midibuf_rd_pointer)
	{
		available = 1;				// data in buffer
		Midi_LED_on;
	}
	else
	{
		available = 0;				// no data in buffer
		Midi_LED_off;
	}
	return available;
}

//-------------------------------------------------------------------------
// read midi data from buffer (size: 128 Byte)
//-------------------------------------------------------------------------
uint8_t read_mididata(void)
{
	uint8_t data;
	while(midibuf_wr_pointer == midibuf_rd_pointer);
	data =  midibuf[midibuf_rd_pointer];
	midibuf_rd_pointer++;
	if (midibuf_rd_pointer > 0x80)
	{
		midibuf_rd_pointer = 0x00;
	}
	return data;
}

//-------------------------------------------------------------------------
// Midi Handle
// this function is call in timer interrupt 1msec
//-------------------------------------------------------------------------
void midi_handle(void)
{
	uint8_t midi_byte = read_mididata();
	static uint8_t hi;
	static uint8_t lo;
	static uint8_t running_status;
	
	if (midi_byte >= 0xf8)
	{
		//MessageReceived(byte);
	}
	else
	{
		if (midi_byte >= 0x80)
		{
			hi = midi_byte & 0xf0;
			lo = midi_byte & 0x0f;
			data_size = 0;
			expected_data_size = 1;
			switch (hi)
			{
				case 0x80:
				case 0x90:
				case 0xa0:
				case 0xb0:
				expected_data_size = 2;
				break;
				
				case 0xc0:
				case 0xd0:
				break;  // default data size of 1.
				
				case 0xe0:
				expected_data_size = 2;
				break;
				
				case 0xf0:
				if (lo > 0 && lo < 3)
				{
					expected_data_size = 2;
				}
				else if (lo >= 4)
				{
					expected_data_size = 0;
				}
				break;
			}
			
			if (midi_byte == 0xf7)
			{
				if (running_status == 0xf0)
				{
					//Device::SysExEnd();
				}
				running_status = 0;
			}
			
			else if (midi_byte == 0xf0)
			{
				running_status = 0xf0;
				//Device::SysExStart();
			}
			else
			{
				running_status = midi_byte;
			}
		}
		else
		{
			midi_data[data_size++] = midi_byte;
		}
		if (data_size >= expected_data_size)
		{
			midi_function(running_status);
			
			data_size = 0;
			if (running_status > 0xf0)
			{
				expected_data_size = 0;
				running_status = 0;
			}
		}
	}
}

//-------------------------------------------------------------------------
// midi function
//-------------------------------------------------------------------------
void midi_function(uint8_t status)
{
	uint8_t hi = status & 0xf0;
	uint8_t lo = status & 0x0f;
	
	// Midi receive channel
	if (Mid.channel_value > 0)
	{
		if (lo != Mid.channel_value-1)
		{
			return;
		}
	}
	
	
	switch (hi)
	{
		case 0x80:			// receive NoteOff
		midi_NoteOff();
		break;

		case 0x90:			// receive NoteOn
		if (midi_data[1])				// if velocity = 0 than NoteOff
		{
			midi_NoteOn();
		}
		else
		{
			midi_NoteOff();
		}
		break;
		
		// Pitch_bend
		case 0xE0:
		pitch_bend();
		break;
		
		// Mod_wheel
		//case 0xB0:
		//mod_wheel();
		//break;
		
		// After_touch
		case 0xd0:
		after_touch();
		break;
		
		case 0xb0:
		mod_wheel();
		switch (midi_data[0])
		{
			case 0x7B:
			midi_AllNotesOff();
			break;
		}
		break;
	}
}

//-------------------------------------------------------------------------
// NoteOn
//-------------------------------------------------------------------------
void midi_NoteOn(void)
{
	Mid.input_flag = 1; // note symbol
	
	
	// sampling rec-mode: "Midi"
	if ((page_nr == 12) && (Gui.Sam_rec_mode == 0) && (Gui.Sam_rec_key_flag == 1))
	{
		Gui.rec_led_flag = 1;
		Gui.Sam_rec_flag = 1;
	}

	// if ARP Mode on than write midi_note into notestack_pool ------------
	if (SEQ.seq_run == 1 && SEQ.seq_mode == 1)
	{
		if (NoteStack_ptr < NoteStack_size)
		{
			uint8_t note_ = midi_data[0];
			midi_velocity = midi_data[1];
			
			NoteStack_pool[NoteStack_ptr] = note_;			// write midi note into notestack
			VeloStack_pool[NoteStack_ptr] = midi_velocity;	// write velocity into notestack
			NoteStack_ptr++;
			
			SEQ.note_activity_flag = 1;
			
			
			if (NoteStack_ptr == 1 && SEQ.seq_hold == 0)
			{
				//SEQ.seq_pulswidth_temp = 1;
				//SEQ.seq_speed_temp = 0;
				SEQ.arp_note_size = 0;
				SEQ.arp_step = 0;
				//SEQ.seq_clock_flag = 0;
				SEQ.arp_step_down = 0;
				SEQ.arp_octave_back_ = 0;
				SEQ.arp_octave_ = 0;
			}
			SEQ.seq_hold_flag = 0;	
		}
	}
	
	// write midi_note into notestack_pool and play note ------------------
	else
	{
		if (NoteStack_ptr < NoteStack_size)
		{
			midi_note = midi_data[0];
			midi_velocity = midi_data[1];
			NoteStack_pool[NoteStack_ptr] = midi_note;		// write current note into notestack_pool
			VeloStack_pool[NoteStack_ptr] = midi_velocity;
			NoteStack_ptr++;
			
			Mid.gate_flag = 1;		// key press
			Mid.trigger_flag = 1;
			
		}
	}
}

//-------------------------------------------------------------------------
// NoteOff
//-------------------------------------------------------------------------
void midi_NoteOff(void)
{
	uint8_t last;
	
	// search current note into notestack_pool
	for (uint8_t i = 0; i < NoteStack_size; i++)
	{
		if (NoteStack_pool[i] == midi_data[0])
		{
			// clear current note into notestack_pool
			NoteStack_pool[i] = 0xFF;
			VeloStack_pool[i] = 0xFF;
			NoteStack_ptr--;
			last = i;
			
			// sort notes into notestack_pool
			for (; last < NoteStack_size;last++)
			{
				NoteStack_pool[last] = NoteStack_pool[last+1];
				VeloStack_pool[last] = VeloStack_pool[last+1];
			}
			
			// playing previous note
			if (NoteStack_ptr > 0)
			{
				if (SEQ.seq_run == 0)
				{
					last = NoteStack_ptr-1;
					midi_note = NoteStack_pool[last];
					midi_velocity = VeloStack_pool[last];
					frequency_tune();
				}
			}
			else
			{
				// stop envelope if no note into notestack_pool
				if (SEQ.seq_run == 0)
				{
					Mid.gate_flag = 0;	// key released
					Mid.trigger_flag = 1;
				}
				
				// stop sampling in rec-mode ------------------------------------------
				if ((page_nr == 12) && (Gui.Sam_rec_flag == 1))
				{
					stop_sampling();
				}
			}
			break;
		}
	}
	
	if (SEQ.seq_run != 0)
	{
		SEQ.note_activity_flag = 1;
		SEQ.draw_note_flag = 1;
		if (NoteStack_ptr == 0 && SEQ.seq_hold == 0)
		{
			SEQ.arp_note_size = 0;
			SEQ.arp_step = 0;
		}	
	}
	
}


//-------------------------------------------------------------------------
// Midi Controller
//
void midi_Controller(void)
{
	Mid.input_flag = 1;
	
	switch (midi_data1)
	{
		case 0x01: modulation();
		break;
		case 0x7B: midi_AllNotesOff();
		break;
		
		
	}
}

//-------------------------------------------------------------------------
// set all notes off
//
void midi_AllNotesOff(void)
{
	midi_gate_flag = 0;
	clear_midi_notestack();
}

//-------------------------------------------------------------------------
// set midi mod.wheel
//-------------------------------------------------------------------------
void modulation(void)
{
	//Mid.input_flag = 1;
}

//-------------------------------------------------------------------------
// set midi pitchbend
//-------------------------------------------------------------------------
void pitch_bend(void)
{
	Mid.input_flag = 1;
	
	// calc pitchband data
	Midi.Pitchbend = ((uint16_t)(midi_data[1] << 7) + midi_data[0]);
	midi_pitchbend = Midi.Pitchbend;
}

//-------------------------------------------------------------------------
// set midi pitch_weel (modulation)
//-------------------------------------------------------------------------
void mod_wheel(void)
{
	Mid.input_flag = 1;
	
	// load modwheel data
	Midi.Modwheel = midi_data[1];
}

//-------------------------------------------------------------------------
// set midi aftertouch
//-------------------------------------------------------------------------
void after_touch(void)
{
	//Mid.input_flag = 1;
	
	// load aftertouch data
	Midi.Atouch = midi_data[0];
}






