/*
 * miniscope.c
 *
 * Created: 30.06.2016 20:12:22
 *  Author: Rolf Degen
 */ 

#include "globals.h"
#include "display/displ_funktion.h"
#include "miniscope.h"
#include "oscillators.h"
#include "xram.h"
#include "gui.h"
#include "keyquery.h"


// trigger symbol position into MiniScope
const uint8_t trigger_pos[] PROGMEM = {
	0,126,124,121,118,115,111,109,
	106,102, 98, 95, 92, 90, 88, 86,
	84, 83, 81, 79, 77, 75, 73, 70,
	68, 66, 65, 64, 63, 62, 60, 59,
	58, 57, 55, 54, 53, 53, 52, 51,
	51, 50, 50, 49, 49, 48, 48, 48,
	47, 47, 47, 46, 46, 46, 46, 45,
	45, 45, 45, 44, 44, 44, 44, 44,
	44
};


//*************************************************************************
// init Scope
//*************************************************************************
void init_Scope (void)
{
	Scope.adc_timebase = 5;
	Scope.adc_trigger_level = 0;
	Scope.scope_mode = 0;
	Scope.scope_gain = 63;
	Scope.timebase_level = 63;
}





//*************************************************************************
// set scope input in scope menu
//*************************************************************************
void set_scope_mode(void)
{
	switch (Scope.scope_mode)
	{
		case 0:
		draw_encoder_StrPos(4,"OSC",0);
		PORTB.OUT |= (1<<PIN3);
		PORTB.OUT |= (1<<PIN4);
		break;
		case 1:
		draw_encoder_StrPos(4,"MIC",23);
		PORTB.OUT &= ~(1<<PIN3);
		PORTB.OUT |= (1<<PIN4);
		break;
		case 2:
		draw_encoder_StrPos(4,"EXT",63);
		PORTB.OUT |= (1<<PIN3);
		PORTB.OUT &= ~(1<<PIN4);
		break;
	}
}

//*************************************************************************
// draw trigger symbol to y_pos
//*************************************************************************
void draw_trigger_symbol(uint8_t y_pos)
{
	// clear old symbol pos
	fillrectangle(2,25,8,163,color_black);
	
	// draw pfeil symbol
	
	vline(3,y_pos,6,color_yello);
	vline(4,y_pos+1,4,color_yello);
	vline(5,y_pos+2,2,color_yello);
	pixel(7,y_pos+3,color_yello);
	
}

//*************************************************************************
// set trigger values in scope menu
//*************************************************************************
void trigger_value(void)
{
	// set analog comparartor
	ACA.CTRLB = Scope.adc_trigger_level;
	ACA.AC0MUXCTRL = AC_MUXPOS_PIN4_gc | AC_MUXNEG_SCALER_gc;
	ACA.AC0CTRL = AC_INTMODE_RISING_gc | AC_INTLVL_MED_gc | AC_HYSMODE_SMALL_gc | AC_ENABLE_bm;
	
	// print scope_trigger_value
	if (Scope.adc_trigger_level == 0)
	{
		draw_encoder_StrPos(2,"OFF",0);
		// clear old symbol pos
		fillrectangle(2,25,8,163,color_black);
	}
	else
	{
		draw_encoder_position(2,Scope.adc_trigger_level);
		//uint8_t y_pos = 155 - (adc_trigger_level * 2);
		uint8_t y_pos = PGM_READ_BYTE(&(trigger_pos[Scope.adc_trigger_level]));		
		draw_trigger_symbol(y_pos);
	}
	
}

//*************************************************************************
// set timebase values in scope menu
//*************************************************************************
void timebase_value(void)
{
	switch (Scope.adc_timebase)
	{
		case 7:
		Scope.timebase_level = 250;
		draw_encoder_StrPos(3,"250",127);
		print_string("us",111,214,2,color_white);
		break;
		case 6:
		Scope.timebase_level = 125;
		draw_encoder_StrPos(3,"500",109);
		print_string("us",111,214,2,color_white);
		break;
		case 5:
		Scope.timebase_level = 63;
		draw_encoder_StrPos(3,"1.0",91);
		print_string("ms",111,214,2,color_white);
		break;
		case 4:
		Scope.timebase_level = 31;
		draw_encoder_StrPos(3,"2.0",73);
		print_string("ms",111,214,2,color_white);
		break;
		case 3:
		Scope.timebase_level = 13;
		draw_encoder_StrPos(3,"5.0",54);
		print_string("ms",111,214,2,color_white);
		break;
		case 2:
		Scope.timebase_level = 6;
		draw_encoder_StrPos(3,"10",36);
		print_string("ms",111,214,2,color_white);
		break;
		case 1:
		Scope.timebase_level = 3;
		draw_encoder_StrPos(3,"20",18);
		print_string("ms",111,214,2,color_white);
		break;
		case 0:
		Scope.timebase_level = 1;
		draw_encoder_StrPos(3,"50",0);
		print_string("ms",111,214,2,color_white);
		break;
	}
}

//*************************************************************************
// Scope sampling to buffer
//*************************************************************************
void Scope_sampling(void)
{
	// MiniScope with trigger function ------------------------------------
	uint8_t temp = Scope.timebase_reg + Scope.timebase_level;
	
	if (temp < Scope.timebase_reg && Scope.adc_trigger <= 1)
	{
		// read sample from adc
		uint8_t sample = ADCA_CH0RES >> 4;	// load sample
		
		// trigger off_automatic
		if (Scope.adc_trigger_level == 0 || Scope.adc_trigger == 1 || page_nr == 12 || Scope.trigger_out_time == 900)	// if trigger off  ore sampling page ?
		{																	// than recording scope-samples
			Scope.adc_buffer[Scope.adc_wr_pointer] = sample;
			Scope.adc_wr_pointer++;
			if (Scope.adc_wr_pointer == 151)
			{
				Scope.adc_wr_pointer = 0;
				Scope.adc_trigger = 99;
			}
		}
	}
	Scope.timebase_reg = temp;
}

//*************************************************************************
// Sample recording
//*************************************************************************
void Sample_recording(void)
{
	/*
	// recording with dithering (noise reduction) -------------------------
	int16_t sample = ADCA_CH0RES;
	__int24 sample_16 = (int16_t)(sample * 16);
	sample_16 += quantization_error2;
	int8_t sample_8 = sample_16 >> 8;
	quantization_error2 = sample_16 - ((__int24)sample_8 << 8);
	
	// sampling monitor ---------------------------------------------------
	if (Sampling_monitor_flag == 1)
	{
		DACA.CH0DATAH = sample_8;
		DACB.CH0DATAH = sample_8;		
	}
	
	// sampling rec-mode: "Threshold" -------------------------------------
	if ((Sampling_rec_mode == 1) && (Sampling_recflag == 1) && (sampling_flag == 0))
	{
		if ((Sampling_thresflag == 1) && (((uint8_t)sample_8 - 128) >= Sampling_threslev))
		{
			sampling_flag = 1;
			draw_rec_symbol();
		}
	}
	
	// write Sample into 1MByte XRAM --------------------------------------
	if (sampling_flag == 1)
	{
		hugemem_ptr_t p = BOARD_EBI_SRAM_BASE;
		hugemem_write8(p+sampling_wr_addr, sample_8);
		sampling_wr_addr++;
		if (sampling_wr_addr >= Xram.max_space)
		{
			stop_sampling();	
		}
	}
	*/
}

//*************************************************************************
// stop sampling
//*************************************************************************
void stop_sampling(void)
{
	Gui.Sam_rec_flag = 0;
	Gui.Sam_rec_key_flag = 0;
	REC_LED_off;
	Gui.rec_led_flag = 0;
	Osc.wave_mode[0] = 1;
	Osc.wave_mode[1] = 2;
	Osc.op_value = 0;
	Osc.sync_function[0] = 1;
	Osc.loop_endaddr[0] = sampling_wr_addr;
	Osc.loop_endaddr[1] = 0;
	Xram.sample_length[0] = sampling_wr_addr;
	Xram.sample_length[1] = 0;
	file_lenght = sampling_wr_addr;
	Gui.block_size = (Xram.sample_length[0] >> 8) >> Gui.Zoom_value;
	Osc.loop_A[0] = 0;
	Osc.loop_B[0] = file_lenght;
	Osc.loop_function[0] = 0;
	Osc.loop_function[1] = 0;
	Osc.mix_value = 0;
	Osc.mix = 0;
	Fx.Value = 0;
	Gui.redraw_flag[0] = FALSE;
	Gui.redraw_flag[1] = FALSE;
	Gui.redraw_flag2 = FALSE;
	enable_encoder(1,1,1,1,1);
	enable_key(1,1,1,1,1,1,1,1,1,1,1,1,1);
	draw_rec_symbol();
	fillrectangle(175,140,199,152,color_black);
	uint16_t addr = (sampling_wr_addr >> 10);
	draw_3digit_with(addr,175,140,6,color_white);
	REC_LED_off;
}





