/*
 * modulation.c
 *
 * Created: 26.06.2016 11:51:26
 *  Author: rolfd
 */ 

#include "modulation.h"
#include "filter.h"
#include "sequencer.h"
#include "oscillators.h"
#include "display/displ_funktion.h"
#include "op.h"



//*************************************************************************
// calc modulation
//*************************************************************************
void modulation_matrix (void)
{
	// copy modulation sources in arry and scaled 0-255 -------------------
	uint8_t modulation_sources[15];
	modulation_sources[MOD_SRC_ENV1]	= Env.value_8[0];		// max. Value = 255
	modulation_sources[MOD_SRC_ENV2]	= Env.value_8[1];		// max. Value = 255
	modulation_sources[MOD_SRC_LFO1]	= LFO.out[0];			// max. Value = 255
	modulation_sources[MOD_SRC_LFO2]	= LFO.out[1];			// max. Value = 255
	modulation_sources[MOD_SRC_LFO3]	= LFO.out[2];			// max. Value = 255
	modulation_sources[MOD_SRC_VELO]	= midi_velocity << 1;	// max. Value = 255
	modulation_sources[MOD_SRC_NOTE]	= midi_note << 1;		// max. Value = 255
	modulation_sources[MOD_SRC_NOISE]	= Noise.sample;			// max. Value = 255
	modulation_sources[MOD_SRC_SEQ]		= SEQ.mod_value;		// max. Value = 255
	modulation_sources[MOD_SRC_MWHEEL]  = Midi.Modwheel;		// max. Value = 255
	modulation_sources[MOD_SRC_PBEND]   = U14ShiftRight6(midi_pitchbend);// max. Value = 255
	modulation_sources[MOD_SRC_ATOUCH]  = Midi.Atouch;			// max. Value = 255
	modulation_sources[MOD_SRC_POTI1]	= poti1_value;
	modulation_sources[MOD_SRC_POTI2]	= poti2_value;
	modulation_sources[MOD_SRC_OFFSET]	= 127;					// DC_Offset
	
	/*
	// destinations result values in arry ---------------------------------
	int8_t modulation_destination[19];
	modulation_destination[MOD_DST_VCA];		// VCA_gain
	modulation_destination[MOD_DST_VCA_BAL];	// VCA_BAL
	modulation_destination[MOD_DST_VCF];		// VCF
	modulation_destination[MOD_DST_VCF_BAL];	// VCF
	modulation_destination[MOD_DST_RESO];		// resonance
	modulation_destination[MOD_DST_LFO1];		// LFO1
	modulation_destination[MOD_DST_LFO2];		// LFO2
	modulation_destination[MOD_DST_LFO3];		// LFO3
	modulation_destination[MOD_DST_OSC1_f];		// Osc1 fine pitch
	modulation_destination[MOD_DST_OSC2_f];		// Osc2 fine pitch
	modulation_destination[MOD_DST_OSC1];		// Osc1 pitch
	modulation_destination[MOD_DST_OSC2];		// Osc2 pitch
	modulation_destination[MOD_DST_OSC12_f];	// Osc1+2 fine pitch
	modulation_destination[MOD_DST_OSC12];	// Osc1+2 pitch
	modulation_destination[MOD_DST_Osc1_PRM];	// Osc1 Parameter (Prm)
	modulation_destination[MOD_DST_Osc2_PRM];	// Osc2 Parameter (Prm)
	modulation_destination[MOD_DST_OSC_MIX];	// Osc_MIX
	modulation_destination[MOD_DST_Fx_Time];	// Fx_Time
	modulation_destination[MOD_DST_Fx_PAN];		// Fx balance
	*/
	
	
	// Load initial values of modulation and scale to 0-8192  -------------
	uint16_t dst_[19];
	dst_[MOD_DST_VCA]		= 255;								// VCA
	dst_[MOD_DST_VCA_BAL]	= 8192;								// VCA_BAL
	dst_[MOD_DST_VCF]		= U8U8Mul(VCF.cutoff_value, 128);	// VCF
	dst_[MOD_DST_VCF_BAL]	= 8192;								// VCF_BAL
	dst_[MOD_DST_RESO]		= U8U8Mul(VCF.reso_value, 128);		// Resonance
	dst_[MOD_DST_LFO1]		= (pgm_read_word (&(LFO_increment[LFO.rate_volume[0]])));
	dst_[MOD_DST_LFO2]		= (pgm_read_word (&(LFO_increment[LFO.rate_volume[1]])));
	dst_[MOD_DST_LFO3]		= (pgm_read_word (&(LFO_increment[LFO.rate_volume[2]])));
	dst_[MOD_DST_OSC1_f]	= 8192;								// max value osc1 fpitch
	dst_[MOD_DST_OSC2_f]	= 8192;								// max value osc2 fpitch
	dst_[MOD_DST_OSC12_f]	= 8192;								// max value osc1+2 fpitch
	dst_[MOD_DST_OSC1]		= 8192;								// max value osc1 pitch
	dst_[MOD_DST_OSC2]		= 8192;								// max value osc2 pitch
	dst_[MOD_DST_OSC12]		= 8192;								// max value osc1+2 pitch
	dst_[MOD_DST_Osc1_PRM]	= U8U8Mul(Osc.prm_value[0], 128);	// Osc1 parameter value
	dst_[MOD_DST_Osc2_PRM]	= U8U8Mul(Osc.prm_value[1], 128);	// Osc2 parameter value
	dst_[MOD_DST_OSC_MIX]	= U8U8Mul(Osc.mix_value, 128);		// 
	dst_[MOD_DST_Fx_Time]	= U8U8Mul(Fx.Time, 128);			// Fx_Time
	dst_[MOD_DST_Fx_PAN]	= 8192;								// max fx balance
	
	Poti1_LED_off;
	Poti2_LED_off;
	
	uint8_t lfo1_status = 0, lfo2_status = 0, lfo3_status = 0;
	
	// calc modulation ----------------------------------------------------
	for (uint8_t slot = 0; slot < 12; slot++)
	{
		uint8_t source_number = slot_array[slot][0];				// load source.nr
		uint8_t destination_number = slot_array[slot][1];			// load destination.nr
		int8_t amount = (slot_array[slot][2]-63);					// load amount value (0-127) and make signed -63/+64
		uint8_t source_value = modulation_sources[source_number];	// load source.value
		
		// next slot if amount == 0
		if (!amount){ continue;}

		// target modulation without VCA ----------------------------------
		if (destination_number != MOD_DST_VCA)
		{
			int16_t modulation_ = dst_[destination_number];
			// For those sources, use relative modulation
			if (source_number <= MOD_SRC_LFO3 ||
			source_number == MOD_SRC_PBEND ||
			source_number == MOD_SRC_NOTE)
			{
				modulation_ += S8S8Mul(amount, source_value + 128);
			}
			else
			{
				modulation_ += S8U8Mul(amount, source_value);
			}
			dst_[destination_number] = S16ClipU14(modulation_);
		}			
		// target modulation only VCA (modulation multiplicativ) ----------
		else
		{
			if (amount < 0)
			{
				amount = -amount;
				source_value = 255 - source_value;
			}
			if (amount != 64)
			{
				source_value = (U8Mix(255, source_value, amount << 2)) >> 8;
			}
			dst_[MOD_DST_VCA] = U8U8MulShift8(dst_[MOD_DST_VCA], source_value);		
		}
					
		// set poti led (init is led_off) ---------------------------------
		if (source_number == MOD_SRC_POTI1 && amount != 0){Poti1_LED_on;}
		if (source_number == MOD_SRC_POTI2 && amount != 0){Poti2_LED_on;}
		
		// set LFO1-3 status ----------------------------------------------
		if (source_number == MOD_SRC_LFO1 && amount != 0){lfo1_status = 1;}
		if (source_number == MOD_SRC_LFO2 && amount != 0){lfo2_status = 1;}
		if (source_number == MOD_SRC_LFO3 && amount != 0){lfo3_status = 1;}
	}
	
	// Update destinations ************************************************
	
	// Update LFO rate (16Bit)
	LFO.rate[0] = (dst_[MOD_DST_LFO1]);
	LFO.rate[1] = (dst_[MOD_DST_LFO2]);
	LFO.rate[2] = (dst_[MOD_DST_LFO3]);
	if (lfo1_status == 1){LFO.run_flag[0] = 1;} else{LFO.run_flag[0] = 0;}
	if (lfo2_status == 1){LFO.run_flag[1] = 1;} else{LFO.run_flag[1] = 0;}
	if (lfo3_status == 1){LFO.run_flag[2] = 1;} else{LFO.run_flag[2] = 0;}

	// update Resonce values ----------------------------------------------
	uint16_t reso_ = U14ShiftRight6(dst_[MOD_DST_RESO]);	
	CV_Reso1 = reso_;
	CV_Reso2 = reso_;
	
	// update Fx_Delay values and Pan -------------------------------------
	uint8_t fx_PAN = U14ShiftRight6(dst_[MOD_DST_Fx_PAN]);
	CV_delay_value1 = U8U8MulShift8(fx_PAN, Fx.Value << 1);
	CV_delay_value2 = U8U8MulShift8(255 - fx_PAN, Fx.Value << 1);
	
	// update Fx_Time & Feedback ------------------------------------------
	CV_delay_time = U14ShiftRight6(dst_[MOD_DST_Fx_Time]);
	CV_delay_feedback = Fx.Feedback;
		
	// update osc_pitch ---------------------------------------------------
	Osci.osc1_fpitch = dst_[MOD_DST_OSC1_f];
	Osci.osc2_fpitch = dst_[MOD_DST_OSC2_f];
	Osci.osc12_fpitch = dst_[MOD_DST_OSC12_f];
	Osci.osc1_pitch = dst_[MOD_DST_OSC1];
	Osci.osc2_pitch = dst_[MOD_DST_OSC2];
	Osci.osc12_pitch = dst_[MOD_DST_OSC12];
	frequency_tune();
	
	// update osc_prm -----------------------------------------------------
	Osc.prm[0] = U15ShiftRight7(dst_[MOD_DST_Osc1_PRM]);
	Osc.prm[1] = U15ShiftRight7(dst_[MOD_DST_Osc2_PRM]);
	
	// update osc_mix -----------------------------------------------------
	Osc.mix = U14ShiftRight6(dst_[MOD_DST_OSC_MIX]);
	
	// update VCF values and Balance --------------------------------------
	uint8_t vcf_value = U14ShiftRight6(dst_[MOD_DST_VCF]);
	uint8_t vcf_bal = U14ShiftRight6(dst_[MOD_DST_VCF_BAL]);
	
	uint16_t vcf_1 = U8Mix(vcf_value,1, vcf_bal) >> 7;
	if (vcf_1 > 255)
	{
		vcf_1 = 255;
	}
	uint16_t vcf_2 = U8Mix(1,vcf_value, vcf_bal) >> 7;
	if (vcf_2 > 255)
	{
		vcf_2 = 255;
	}
	CV_VCF_Cha1 = pgm_read_word(&(Env_conv10[vcf_1]));
	CV_VCF_Cha2 = pgm_read_word(&(Env_conv10[vcf_2]));
	
	// update VCA values and Balance --------------------------------------
	uint8_t vca_bal = U14ShiftRight6(dst_[MOD_DST_VCA_BAL]);
	uint16_t vca_1 = U8Mix(dst_[MOD_DST_VCA],1, vca_bal) >> 7;
	if (vca_1 > 255)
	{
		vca_1 = 255;
	}
	uint16_t vca_2 = U8Mix(1,dst_[MOD_DST_VCA], vca_bal) >> 7;
	if (vca_2 > 255)
	{
		vca_2 = 255;
	}
	CV_VCA_Cha1 = pgm_read_word(&(Env_conv10[vca_1]));
	CV_VCA_Cha2 = pgm_read_word(&(Env_conv10[vca_2]));
	
	
}