/*
 * op.h
 *
 * Created: 29.04.2017 23:31:39
 *  Author: rolfd
 */ 


#ifndef OP_H_
#define OP_H_
#define INLINE static inline
#endif /* OP_H_ */


//*************************************************************************
// inline function
//*************************************************************************
INLINE uint8_t U8U8MulShift8(uint8_t a, uint8_t b) {
	uint8_t result;
	asm(
	"mul %1, %2"      "\n\t"
	"mov %0, r1"      "\n\t"
	"eor r1, r1"      "\n\t"
	: "=r" (result)
	: "a" (a), "a" (b)
	);
	return result;
}

INLINE uint16_t U16ShiftRight4(uint16_t a) {
	uint16_t result;
	asm(
	"movw %A0, %A1" "\n\t"
	"lsr %B0"      "\n\t"
	"ror %A0"      "\n\t"
	"lsr %B0"      "\n\t"
	"ror %A0"      "\n\t"
	"lsr %B0"      "\n\t"
	"ror %A0"      "\n\t"
	"lsr %B0"      "\n\t"
	"ror %A0"      "\n\t"
	: "=r" (result)
	: "a" (a)
	);
	return result;
}

INLINE uint16_t U16U8MulShift8(uint16_t a, uint8_t b)
{
	uint16_t result;
	asm(
	"eor %B0, %B0"    "\n\t"
	"mul %A1, %A2"    "\n\t"
	"mov %A0, r1"     "\n\t"
	"mul %B1, %A2"  "\n\t"
	"add %A0, r0"     "\n\t"
	"adc %B0, r1"     "\n\t"
	"eor r1, r1"      "\n\t"
	: "=&r" (result)
	: "a" (a), "a" (b)
	);
	return result;
}

INLINE uint16_t U8MixU16(uint8_t a, uint8_t b, uint8_t balance) {
	uint16_t sum;
	asm(
	"mul %3, %2"      "\n\t"  // b * balance
	"movw %A0, r0"    "\n\t"  // to sum
	"com %2"          "\n\t"  // 255 - balance
	"mul %1, %2"      "\n\t"  // a * (255 - balance)
	"com %2"          "\n\t"  // reset balance to its previous value
	"add %A0, r0"     "\n\t"  // add to sum L
	"adc %B0, r1"     "\n\t"  // add to sum H
	"eor r1, r1"      "\n\t"  // reset r1 after multiplication
	: "&=r" (sum)
	: "a" (a), "a" (balance), "a" (b)
	);
	return sum >> 8;
}

INLINE uint16_t U8MixU16_(uint8_t a, uint8_t b, uint8_t balance) {
	uint16_t sum;
	asm(
	"mul %3, %2"      "\n\t"  // b * balance
	"movw %A0, r0"    "\n\t"  // to sum
	"com %2"          "\n\t"  // 255 - balance
	"mul %1, %2"      "\n\t"  // a * (255 - balance)
	"com %2"          "\n\t"  // reset balance to its previous value
	"add %A0, r0"     "\n\t"  // add to sum L
	"adc %B0, r1"     "\n\t"  // add to sum H
	"eor r1, r1"      "\n\t"  // reset r1 after multiplication
	: "&=r" (sum)
	: "a" (a), "a" (balance), "a" (b)
	);
	return sum;
}

INLINE uint8_t InterpolateSample(
	const uint8_t* table, uint16_t phase) {
	uint8_t result;
	uint8_t work;
	asm(
	"movw r30, %A2"           "\n\t"  // copy base address to r30:r31
	"add r30, %B3"            "\n\t"  // increment table address by phaseH
	"adc r31, r1"             "\n\t"  // just carry
	"mov %1, %A3"             "\n\t"  // move phaseL to working register
	"lpm %0, z+"              "\n\t"  // load sample[n]
	"lpm r1, z+"              "\n\t"  // load sample[n+1]
	"mul %1, r1"              "\n\t"  // multiply second sample by phaseL
	"movw r30, r0"            "\n\t"  // result to accumulator
	"com %1"                  "\n\t"  // 255 - phaseL -> phaseL
	"mul %1, %0"              "\n\t"  // multiply first sample by phaseL
	"add r30, r0"             "\n\t"  // accumulate L
	"adc r31, r1"             "\n\t"  // accumulate H
	"eor r1, r1"              "\n\t"  // reset r1 after multiplication
	"mov %0, r31"             "\n\t"  // use sum H as output
	: "=r" (result), "=r" (work)
	: "r" (table), "r" (phase)
	: "r30", "r31"
	);
	return result;
}

INLINE int16_t S8U8Mul(int8_t a, uint8_t b) {
	int16_t result;
	asm(
	"mulsu %1, %2"    "\n\t"
	"movw %0, r0"      "\n\t"
	"eor r1, r1"      "\n\t"
	: "=r" (result)
	: "a" (a), "a" (b)
	);
	return result;
}

INLINE int16_t S8S8Mul(int8_t a, int8_t b) {
	int16_t result;
	asm(
	"muls %1, %2"    "\n\t"
	"movw %0, r0"      "\n\t"
	"eor r1, r1"      "\n\t"
	: "=r" (result)
	: "a" (a), "a" (b)
	);
	return result;
}

INLINE int8_t S8U8MulShift8(int8_t a, uint8_t b) {
	uint8_t result;
	asm(
	"mulsu %1, %2"    "\n\t"
	"mov %0, r1"      "\n\t"
	"eor r1, r1"      "\n\t"
	: "=r" (result)
	: "a" (a), "a" (b)
	);
	return result;
}

INLINE uint8_t U8MixShift8(uint8_t a, uint8_t b, uint8_t balance) {
	uint16_t sum;
	asm(
	"mul %3, %2"      "\n\t"  // b * balance
	"movw %A0, r0"    "\n\t"  // to sum
	"com %2"          "\n\t"  // 255 - balance
	"mul %1, %2"      "\n\t"  // a * (255 - balance)
	"com %2"          "\n\t"  // reset balance to its previous value
	"add %A0, r0"     "\n\t"  // add to sum L
	"adc %B0, r1"     "\n\t"  // add to sum H
	"eor r1, r1"      "\n\t"  // reset r1 after multiplication
	: "&=r" (sum)
	: "a" (a), "a" (balance), "a" (b)
	);
	return sum >> 8;
}

INLINE int16_t S16S8MulShift8(int16_t a, int8_t b) {
	int16_t result;
	asm(
	"eor %B0, %B0"    "\n\t"
	"muls %A2, %B1"   "\n\t"
	"movw %A0, r0"    "\n\t"
	"mulsu %A2, %A1"  "\n\t"
	"eor r0, r0"      "\n\t"
	"sbc %B0, r0"     "\n\t"
	"add %A0, r1"     "\n\t"
	"adc %B0, r0"     "\n\t"
	"eor r1, r1"      "\n\t"
	: "=&r" (result)
	: "a" (a), "a" (b)
	);
	return result;
}

INLINE int16_t S16ClipU14(int16_t value) {
	uint8_t msb = value >> 8;
	if (msb & 0x80) {
		return 0;
		} if (msb & 0x40) {
		return 16383;
	}
	return value;
}

// The code generated by gcc for >> 6 is short but uses a loop. This saves
// a couple of cycles. Note that this solution only works for operands with
// a 14-bits resolution.
INLINE uint8_t U14ShiftRight6(uint16_t value) {
	uint8_t b = value >> 8;
	uint8_t a = value & 0xff;
	uint8_t result;
	asm(
	"add %1, %1"       "\n\t"
	"adc %2, %2"       "\n\t"
	"add %1, %1"       "\n\t"
	"adc %2, %2"       "\n\t"
	: "=r" (result)
	: "a" (a), "0" (b)
	);
	return result;
}

INLINE uint16_t U8U8Mul(uint8_t a, uint8_t b) {
	uint16_t result;
	asm(
	"mul %1, %2"    "\n\t"
	"movw %0, r0"      "\n\t"
	"eor r1, r1"      "\n\t"
	: "=r" (result)
	: "a" (a), "a" (b)
	);
	return result;
}

INLINE uint8_t U8Swap4(uint8_t a) {
	uint8_t result;
	asm(
	"mov %0, %1"      "\n\t"
	"swap %0"         "\n\t"
	: "=r" (result)
	: "a" (a)
	);
	return result;
}

INLINE uint8_t U8AddClip(uint8_t value, uint8_t increment, uint8_t max) {
	value += increment;
	if (value > max) {
		value = max;
	}
	return value;
}

/*
static inline uint8_t U8Mix(
uint8_t a, uint8_t b,
uint8_t gain_a, uint8_t gain_b) {
	word sum;
	asm(
	"mul %3, %4"      "\n\t"  // b * gain_b
	"movw %A0, r0"    "\n\t"  // to sum
	"mul %1, %2"      "\n\t"  // a * gain_a
	"add %A0, r0"     "\n\t"  // add to sum L
	"adc %B0, r1"     "\n\t"  // add to sum H
	"eor r1, r1"      "\n\t"  // reset r1 after multiplication
	: "&=r" (sum)
	: "a" (a), "a" (gain_a), "a" (b), "a" (gain_b)
	);
	return sum.bytes[1];
}
*/

INLINE uint16_t U8Mix(uint8_t a, uint8_t b, uint8_t balance) {
	uint16_t sum;
	asm(
	"mul %3, %2"      "\n\t"  // b * balance
	"movw %A0, r0"    "\n\t"  // to sum
	"com %2"          "\n\t"  // 255 - balance
	"mul %1, %2"      "\n\t"  // a * (255 - balance)
	"com %2"          "\n\t"  // reset balance to its previous value
	"add %A0, r0"     "\n\t"  // add to sum L
	"adc %B0, r1"     "\n\t"  // add to sum H
	"eor r1, r1"      "\n\t"  // reset r1 after multiplication
	: "&=r" (sum)
	: "a" (a), "a" (balance), "a" (b)
	);
	return sum;
}

INLINE uint8_t U8Mix4(
uint8_t a, uint8_t b,
uint8_t gain_a, uint8_t gain_b) {
	uint16_t sum;
	asm(
	"mul %3, %4"      "\n\t"  // b * gain_b
	"movw %A0, r0"    "\n\t"  // to sum
	"mul %1, %2"      "\n\t"  // a * gain_a
	"add %A0, r0"     "\n\t"  // add to sum L
	"adc %B0, r1"     "\n\t"  // add to sum H
	"eor r1, r1"      "\n\t"  // reset r1 after multiplication
	: "&=r" (sum)
	: "a" (a), "a" (gain_a), "a" (b), "a" (gain_b)
	);
	return sum >> 8;
}

INLINE uint8_t InterpolateTwoTables(
const uint8_t* table_a, const uint8_t* table_b,
uint16_t phase, uint8_t gain_a, uint8_t gain_b) {
	return U8Mix4(
	InterpolateSample(table_a, phase),
	InterpolateSample(table_b, phase),
	gain_a, gain_b);
}

INLINE int8_t S8S8MulShift8(int8_t a, int8_t b) {
	uint8_t result;
	asm(
	"muls %1, %2"    "\n\t"
	"mov %0, r1"      "\n\t"
	"eor r1, r1"      "\n\t"
	: "=r" (result)
	: "a" (a), "a" (b)
	);
	return result;
}

INLINE uint8_t U15ShiftRight7(uint16_t value) {
	uint8_t b = value >> 8;
	uint8_t a = value & 0xff;
	uint8_t result;
	asm(
	"add %1, %1" "\n\t"
	"adc %2, %2" "\n\t"
	: "=r" (result)
	: "a" (a), "0" (b)
	);
	return result;
}


