//*************************************************************************
// Oscillators.c
//
// Created: 01.07.2016 16:16:51
//  Author: Rolf Degen
//*************************************************************************

#include "globals.h"
#include "oscillators.h"
#include "wavetable.h"
#include "voice.h"
#include "xram.h"
#include "gui.h"
#include "envelope.h"
#include "op.h"


	
//*************************************************************************
// Oscillator Render Task
//*************************************************************************
void Osc_Render(uint8_t osc_nr)
{
	if (Osc.wave_mode[osc_nr] == 1)
	{
		OscRender_Sample(osc_nr);
		return;
	}
	else if (Osc.wave_mode[osc_nr] == 2)
	{
		OscRender_OFF(osc_nr);
		return;
	}
		
	
	// set osc shape ------------------------------------------------------	
	switch (Osc.shape[osc_nr])
	{
		case(OSC_USER):
		OscRender_USER(osc_nr);
		break;
		case(OSC_SAW):
		OscRender_SAW_SQUARE(osc_nr);
		break;
		case(OSC_SQUARE):
		OscRender_SAW_SQUARE(osc_nr);
		break;
		case(OSC_DPWM):
		OscRender_DPWM(osc_nr);
		break;
		case(OSC_TRIANG):
		OscRender_TRIANG(osc_nr);
		break;
		case(OSC_ZSAW):
		OscRender_CZSAW(osc_nr);
		break;
		case(OSC_ZSAW_LP):
		OscRender_CzResoSaw(osc_nr);
		break;
		case(OSC_ZSAW_PK):
		OscRender_CzResoSaw(osc_nr);
		break;
		case(OSC_ZSAW_HP):
		OscRender_CzResoSaw(osc_nr);
		break;
		case(OSC_ZPULS_LP):
		OscRender_CZPULS(osc_nr);
		break;
		case(OSC_ZPULS_PK):
		OscRender_CZPULS(osc_nr);
		break;
		case(OSC_ZPULS_HP):
		OscRender_CZPULS(osc_nr);
		break;
		case(OSC_ZSYNC):
		OscRender_CZRESO(osc_nr);
		break;
		case(OSC_ZTRI):
		OscRender_CZRESO(osc_nr);
		break;
		case(OSC_SAW_PAD):
		OscRender_PAD(osc_nr);
		break;
		case(OSC_QUAD_PWM):
		OscRender_PAD(osc_nr);
		break;
		case(OSC_FM):
		OscRender_FM(osc_nr);
		break;
		case(OSC_FMFB):
		OscRender_FM(osc_nr);
		break;
		case(OSC_8BIT):
		OscRender_8BIT(osc_nr);
		break;
		case(OSC_CRUSH):
		OscRender_CRUSH(osc_nr);
		break;
		case(OSC_NOISE):
		OscRender_Noise(osc_nr);
		break;
	}		
}


//*************************************************************************
// OscRender: Sample
//*************************************************************************
void OscRender_Sample (uint8_t osc_nr)
{
	uint8_t block_size = 40;
	uint32_t phase = 0;
	__uint24 phase_increment = 0;
	uint8_t phase_carry = 0;
	uint8_t *buffer;	// pointer name
	//uint8_t bufwrpointer = 0;
	uint8_t i = 0;
	__uint24 offset = 0;
	//__uint24 end_addr;
	uint8_t sample1 = 0;
	uint8_t sample2 = 0;
	__uint24 loop_A = 0;
	__uint24 loop_B = 0;
	
	// Osc1
	if (osc_nr == 0)
	{
		phase = Osc1.phase_x;
		phase_carry = Osc1.phase_carry2;
		phase_increment = Osc.phase_increment[osc_nr];
		//end_addr = Xram.sample_length[0];
		offset = 0;
		loop_A = Osc.loop_A[0];
		loop_B = Osc.loop_B[0];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2a;
		}
		else {buffer = Voice.Buffer1a;}
	}
	// Osc2
	else
	{
		phase = Osc2.phase_x;
		phase_carry = Osc2.phase_carry2;
		phase_increment = Osc.phase_increment[osc_nr];
		//end_addr = Xram.sample_length[1];
		offset = 0xef1ff - (Xram.sample_length[osc_nr]);
		loop_A = Osc.loop_A[1];
		loop_B = Osc.loop_B[1];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2b;
		}
		else {buffer = Voice.Buffer1b;}
	}
	
	hugemem_ptr_t p = BOARD_EBI_SRAM_BASE + offset;
	uint8_t sample;
	
	uint32_t addr = 0;
	
	
	// sample loop
	while(block_size--)
	{
		// update phase
		phase += phase_increment;
		if (phase < phase_increment)
		{
			phase_carry++;
		}
		addr = ((uint32_t)phase_carry << 16) + (phase >> 16);
		
		// if loop points available ?
		if (Osc.loop_function[osc_nr] == 1)
		{
			if (addr >= (loop_B-1))
			{
				addr = loop_A;
				phase = (uint32_t)loop_A << 16;
				phase_carry = loop_A >> 16;
			}
			
			//interpolation sample
			sample1 = hugemem_read8(p + addr);
			sample2 = hugemem_read8(p + addr + 1);
			uint8_t phase_x = (phase >> 8) & 0xff;
			sample = (U8MixShift8(sample1,sample2, phase_x));
		}
		
		else  // no loop point
		{
			if (addr >= (Xram.sample_length[osc_nr]-1))
			{
				phase = (uint32_t)Xram.sample_length[osc_nr] << 16;
				phase_carry = Xram.sample_length[osc_nr] >> 16;
				addr = Xram.sample_length[osc_nr];
				sample = 127;
			}
			else
			{
				
				//interpolation sample
				sample1 = hugemem_read8(p + addr);
				sample2 = hugemem_read8(p + addr + 1);
				uint8_t phase_x = (phase >> 8) & 0xff;
				sample = (U8MixShift8(sample1,sample2, phase_x));
			}	
		}
		
		// write sample in sound-buffer
		buffer[i] = sample;
		i++;	
	}
	
	
	
	// update phase
	if (osc_nr == 0)
	{
		Osc1.phase_x = phase;
		Osc1.phase_carry2 = phase_carry;
	}
	else
	{
		Osc2.phase_x = phase;
		Osc2.phase_carry2 = phase_carry;
	}

}

//*************************************************************************
// OscRender: User Wafeforms
//*************************************************************************
void OscRender_USER (uint8_t osc_nr)
{
	uint8_t block_size = 40;
	uint8_t shape;
	uint8_t parameter;
	uint16_t wave_addr;
	__uint24 offset;
	uint8_t *buffer;	// pointer audio_buffer
	uint8_t i = 0;
	
	if (osc_nr == 0)
	{
		shape = Osc.shape[0];
		parameter = Osc.prm[0];
		wave_addr = Osc.wave[0] << 8;
		offset = 0xEFC00;
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2a;
		}
		else {buffer = Voice.Buffer1a;}
	}
	else
	{
		shape = Osc.shape[1];
		parameter = Osc.prm[1];
		wave_addr = Osc.wave[1] << 8;
		offset = 0xF7C00;
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2b;
		}
		else {buffer = Voice.Buffer1b;}
	}
	
	if (parameter > 127)
	{
		parameter = 127;
	}
		
	hugemem_ptr_t p = BOARD_EBI_SRAM_BASE + offset;
	uint8_t sample1 = 0;
	uint8_t sample2 = 0;
	
			
	// sample loop (fill first buffer) ------------------------------------
	while(block_size--)
	{
		// update phase
		Osc.phase[osc_nr] += Osc.phase_increment[osc_nr];
		
		// Osc sync
		if (osc_nr == 0){Osc12.syncbuffer[i] = Osc.phase[osc_nr] < Osc.phase_increment[osc_nr] ? 1:0;}
		else{if (Osc.op_value == OP_SYNC){if (Osc12.syncbuffer[i]) Osc.phase[osc_nr] = 0;}}
		
		// calc table addr
		addr.addr16 = (Osc.phase[osc_nr] >> 16) + wave_addr;
		
		// load sample1
		sample1 = hugemem_read8(p + addr.addr16);
	
		// load sample2
		addr.addr8[0]++;
		sample2 = hugemem_read8(p + addr.addr16);
		
		//interpolation two samples
		uint8_t phase_x = (Osc.phase[osc_nr] >> 8) & 0xff;
		uint8_t sample = (U8MixShift8(sample1,sample2, phase_x));
				
		// osc parameter
		if (sample < parameter)
		{
			if ( shape == 0)sample += parameter >> 1;
			else sample = parameter;
		}
		
		// write sample in current buffer
		buffer[i] = sample;
		i++;
	}
}


//*************************************************************************
// OscRender: Bandlimit SAW_SQUARE
//*************************************************************************
void OscRender_SAW_SQUARE(uint8_t osc_nr)
{
	uint8_t block_size = 40;
	uint8_t shape;
	uint8_t parameter;
	uint8_t *buffer;	// pointer name
	uint8_t i = 0;
		
	if (osc_nr == 0)
	{
		shape = Osc.shape[0];
		parameter = Osc.prm[0];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2a;
		}
		else {buffer = Voice.Buffer1a;}
	}
	else
	{
		shape = Osc.shape[1];
		parameter = Osc.prm[1];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2b;
		}
		else {buffer = Voice.Buffer1b;}
	}
	
	if (parameter > 127)
	{
		parameter = 127;
	}
	
	uint8_t mid_note;// = 0;
	if (midi_note > 12)
	{
		mid_note = midi_note - 12;
	}
	else mid_note = midi_note;
	
	uint8_t balance_index = U8Swap4(mid_note);
	uint8_t gain_2 = balance_index & 0xf0;
	uint8_t gain_1 = ~gain_2;

	uint8_t wave_index = balance_index & 0xf;
	uint16_t waveform_table[7];
	
	if (shape == OSC_SAW)
	{
		waveform_table[0] = wav_res_bandlimited_saw_0;
		waveform_table[1] = wav_res_bandlimited_saw_1;
		waveform_table[2] = wav_res_bandlimited_saw_2;
		waveform_table[3] = wav_res_bandlimited_saw_3;
		waveform_table[4] = wav_res_bandlimited_saw_4;
		waveform_table[5] = wav_res_bandlimited_saw_5;
		//waveform_table[6] = wav_res_bandlimited_triangle_0;
		waveform_table[6] = wav_res_sine;
	}
	else
	{
		waveform_table[0] = wav_res_bandlimited_square_0;
		waveform_table[1] = wav_res_bandlimited_square_5;
		waveform_table[2] = wav_res_bandlimited_square_10;
		waveform_table[3] = wav_res_bandlimited_square_13;
		waveform_table[4] = wav_res_bandlimited_square_14;
		waveform_table[5] = wav_res_bandlimited_square_15;
		waveform_table[6] = wav_res_sine;
	}
	
	uint8_t* wave_1 = waveform_table[wave_index];
	wave_index = U8AddClip(wave_index, 1, kNumZonesFullSampleRate);
	uint8_t* wave_2 = waveform_table[wave_index];
	
	while (block_size--)
	{
		// update Phase
		Osc.phase[osc_nr] += Osc.phase_increment[osc_nr];
		
		// Osc sync
		if (osc_nr == 0){Osc12.syncbuffer[i] = Osc.phase[osc_nr] < Osc.phase_increment[osc_nr] ? 1:0;}
		else{if (Osc.op_value == OP_SYNC){if (Osc12.syncbuffer[i]) Osc.phase[osc_nr] = 0;}}
		
		uint16_t phase_integral = Osc.phase[osc_nr] >> 8;
		
		uint8_t sample = InterpolateTwoTables( wave_1, wave_2,phase_integral, gain_1, gain_2);
		
		if (sample < parameter) {
			// Add a discontinuity.
			sample += parameter >> 1;
		}
		
		// write sample in current buffer
		buffer[i] = ~sample;
		i++;
	}
}

//*************************************************************************
// OscRender: Dirty PWM
//*************************************************************************
void OscRender_DPWM(uint8_t osc_nr)
{
	uint8_t block_size = 40;
	uint8_t parameter;
	uint8_t *buffer;	// pointer name
	uint8_t i = 0;
	
	if (osc_nr == 0)
	{
		parameter = Osc.prm[0];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2a;
		}
		else {buffer = Voice.Buffer1a;}		
	}
	else
	{
		parameter = Osc.prm[1];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2b;
		}
		else {buffer = Voice.Buffer1b;}
	}
	
	if (parameter > 127)
	{
		parameter = 127;
	}
	
	while (block_size--)
	{
		// update Phase
		Osc.phase[osc_nr] += Osc.phase_increment[osc_nr];
		
		// Osc sync
		if (osc_nr == 0){Osc12.syncbuffer[i] = Osc.phase[osc_nr] < Osc.phase_increment[osc_nr] ? 1:0;}
		else{if (Osc.op_value == OP_SYNC){if (Osc12.syncbuffer[i]) Osc.phase[osc_nr] = 0;}}
		
		// calc parameter
		uint16_t phase_integral = Osc.phase[osc_nr] >> 8;
		uint8_t sample = (phase_integral >> 8) < 127 + parameter ? 0 : 255;
		
		// write sample in current buffer
		buffer[i] = ~sample;
		i++;
	}
}

//*************************************************************************
// OscRender: TRIANG
//*************************************************************************
void OscRender_TRIANG(uint8_t osc_nr)
{
	uint8_t block_size = 40;
	uint8_t parameter;
	uint8_t *buffer;	// pointer name
	uint8_t i = 0;
	
	if (osc_nr == 0)
	{
		parameter = Osc.prm[0];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2a;
		}
		else {buffer = Voice.Buffer1a;}
	}
	else
	{
		parameter = Osc.prm[1];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2b;
		}
		else {buffer = Voice.Buffer1b;}
	}
	
	if (parameter > 127)
	{
		parameter = 127;
	}
	
	while (block_size--)
	{
		// update Phase
		Osc.phase[osc_nr] += Osc.phase_increment[osc_nr];
		
		// Osc sync
		if (osc_nr == 0){Osc12.syncbuffer[i] = Osc.phase[osc_nr] < Osc.phase_increment[osc_nr] ? 1:0;}
		else{if (Osc.op_value == OP_SYNC){if (Osc12.syncbuffer[i]) Osc.phase[osc_nr] = 0;}}
		
		uint16_t phase_integral = Osc.phase[osc_nr] >> 8;
		uint8_t tri = phase_integral >> 7;
		uint8_t v = phase_integral & 0x8000 ? tri : ~tri;
		if (v < parameter) { // fold triangle
			v = (parameter << 1) - v;
		}
		
		// write sample in current buffer
		buffer[i] = ~v;
		i++;
	}
}

//*************************************************************************
// OscRender: CZsaw
//*************************************************************************
void OscRender_CZSAW(uint8_t osc_nr)
{
	uint8_t block_size = 40;
	uint8_t parameter;
	uint8_t *buffer;	// pointer name
	uint8_t i = 0;
	
	if (osc_nr == 0)
	{
		parameter = Osc.prm[0];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2a;
		}
		else {buffer = Voice.Buffer1a;}
	}
	else
	{
		parameter = Osc.prm[1];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2b;
		}
		else {buffer = Voice.Buffer1b;}
	}
	
	if (parameter > 127)
	{
		parameter = 127;
	}
		
	while (block_size--)
	{
		// update Phase
		Osc.phase[osc_nr] += Osc.phase_increment[osc_nr];
		
		// Osc sync
		if (osc_nr == 0){Osc12.syncbuffer[i] = Osc.phase[osc_nr] < Osc.phase_increment[osc_nr] ? 1:0;}
		else{if (Osc.op_value == OP_SYNC){if (Osc12.syncbuffer[i]) Osc.phase[osc_nr] = 0;}}
		
		// calc parameter
		uint16_t phase_integral = Osc.phase[osc_nr] >> 8;
		uint8_t phi = phase_integral >> 8;
		 uint8_t clipped_phi = phase_integral < 0x2000 ? phase_integral >> 5 : 0xff;
		// Interpolation causes more aliasing here.
		uint8_t sample = pgm_read_byte (&(wav_res_sine[U8MixU16(phi, clipped_phi, parameter << 1)]));
		
		// write sample in current buffer
		buffer[i] = ~sample;
		i++;
	}
}

//*************************************************************************
// OscRender: Zreso
//*************************************************************************
void OscRender_CZRESO(uint8_t osc_nr)
{
	uint8_t block_size = 40;
	uint8_t shape;
	uint8_t parameter;
	uint8_t *buffer;	// pointer name
	uint8_t i = 0;
	uint16_t phase_2 = Osc.data_secondary_phase[osc_nr];
	
	if (osc_nr == 0)
	{
		shape = Osc.shape[0];
		parameter = Osc.prm[0];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2a;
		}
		else {buffer = Voice.Buffer1a;}
	}
	else
	{
		shape = Osc.shape[1];
		parameter = Osc.prm[1];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2b;
		}
		else {buffer = Voice.Buffer1b;}
	}
	
	if (parameter > 127)
	{
		parameter = 127;
	}

	// calc integral
	uint16_t phase_increment_integral = Osc.phase_increment[osc_nr] >> 8;
	uint16_t increment = phase_increment_integral + (
	(phase_increment_integral * (uint32_t)(parameter)) >> 3);
	
	while (block_size--)
	{
		// update Phase
		Osc.phase[osc_nr] += Osc.phase_increment[osc_nr];
		
		// Osc sync
		if (osc_nr == 0){Osc12.syncbuffer[i] = Osc.phase[osc_nr] < Osc.phase_increment[osc_nr] ? 1:0;}
		else{if (Osc.op_value == OP_SYNC){if (Osc12.syncbuffer[i]) Osc.phase[osc_nr] = 0;}}
		
		uint16_t phase_integral = Osc.phase[osc_nr] >> 8;
		
		if (Osc.phase[osc_nr] < Osc.phase_increment[osc_nr])
		{
			phase_2 = 0;
		}
		
		phase_2 += increment;
		uint8_t carrier = InterpolateSample(wav_res_sine, phase_2);
		
		// CZSYNC synthesis
		if (shape == OSC_ZSYNC)
		{
			// write sample in current buffer
			buffer[i] = ~(phase_integral < 0x8000 ? carrier : 128);
			i++;
		}
		else
		{
			// CZTRI synthesis
			uint8_t window = 0;
			window = (phase_integral & 0x8000) ?
			~(uint8_t)(phase_integral >> 7) :
			phase_integral >> 7;
			// write sample in sound-buffer
			buffer[i] = ~U8U8MulShift8(carrier, window);
			i++;
		}
	}
	
	// update secondary phase
	Osc.data_secondary_phase[osc_nr] = phase_2;
}


//*************************************************************************
// OscRender: CzResoSaw
//*************************************************************************
void OscRender_CzResoSaw(uint8_t osc_nr)
{
	uint8_t block_size = 40;
	uint8_t shape;
	uint8_t parameter;
	uint8_t *buffer;	// pointer name
	uint8_t i = 0;
	uint16_t phase_2 = Osc.data_secondary_phase[osc_nr];
	
	if (osc_nr == 0)
	{
		shape = Osc.shape[0];
		parameter = Osc.prm[0];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2a;
		}
		else {buffer = Voice.Buffer1a;}
	}
	else
	{
		shape = Osc.shape[1];
		parameter = Osc.prm[1];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2b;
		}
		else {buffer = Voice.Buffer1b;}
	}
	
	if (parameter > 127)
	{
		parameter = 127;
	}
	
	uint16_t phase_increment_integral = Osc.phase_increment[osc_nr] >> 8;
	
	uint16_t increment = phase_increment_integral + (
	(phase_increment_integral * (uint32_t)(parameter)) >> 3);

	
	// BEGIN_SAMPLE_LOOP
	while (block_size--)
	{
		// update Phase
		Osc.phase[osc_nr] += Osc.phase_increment[osc_nr];
		
		// Osc sync
		if (osc_nr == 0){Osc12.syncbuffer[i] = Osc.phase[osc_nr] < Osc.phase_increment[osc_nr] ? 1:0;}
		else{if (Osc.op_value == OP_SYNC){if (Osc12.syncbuffer[i]) Osc.phase[osc_nr] = 0;}}
		
		// calc integral
		uint16_t phase_integral = Osc.phase[osc_nr] >> 8;
		
		if (Osc.phase[osc_nr] < Osc.phase_increment[osc_nr]) {
			phase_2 = (shape == OSC_ZSAW_LP) ? 0 : 32768;
		}
		phase_2 += increment;
		uint8_t carrier = InterpolateSample(wav_res_sine, phase_2);
		uint8_t window = ~(phase_integral >> 8);
		if (shape == OSC_ZSAW_HP)
		{
			// write sample in sound-buffer
			buffer[i] = ~(S8U8MulShift8(carrier + 128, window) + 128);
			i++;
		}
		else
		{
			buffer[i] = ~U8U8MulShift8(carrier, window);
			i++;
		}
	}
	
	// update secondary phase
	Osc.data_secondary_phase[osc_nr] = phase_2;
}

//*************************************************************************
// OscRender: CzPulsReso
//*************************************************************************
void OscRender_CZPULS(uint8_t osc_nr)
{
	uint8_t block_size = 40;
	uint8_t shape;
	uint8_t parameter;
	uint8_t *buffer;	// pointer name
	uint8_t i = 0;
	uint16_t phase_2 = Osc.data_secondary_phase[osc_nr];
	
	if (osc_nr == 0)
	{
		shape = Osc.shape[0];
		parameter = Osc.prm[0];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2a;
		}
		else {buffer = Voice.Buffer1a;}
	}
	else
	{
		shape = Osc.shape[1];
		parameter = Osc.prm[1];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2b;
		}
		else {buffer = Voice.Buffer1b;}
	}
	
	if (parameter > 127)
	{
		parameter = 127;
	}
	
	// calc integral
	uint16_t phase_increment_integral = Osc.phase_increment[osc_nr] >> 8;
	uint16_t increment = phase_increment_integral + (
	(phase_increment_integral * (uint32_t)(parameter)) >> 2);
	
	while (block_size--)
	{
		// update Phase
		Osc.phase[osc_nr] += Osc.phase_increment[osc_nr];
		
		// Osc sync
		if (osc_nr == 0){Osc12.syncbuffer[i] = Osc.phase[osc_nr] < Osc.phase_increment[osc_nr] ? 1:0;}
		else{if (Osc.op_value == OP_SYNC){if (Osc12.syncbuffer[i]) Osc.phase[osc_nr] = 0;}}
		
		uint16_t phase_integral = Osc.phase[osc_nr] >> 8;
		
		if (Osc.phase[osc_nr] < Osc.phase_increment[osc_nr]) // if carry
		{
			phase_2 = (shape == OSC_ZPULS_LP) ? 0 : 32768;
		}
		
		phase_2 += increment;
		uint8_t carrier = pgm_read_byte (&(wav_res_sine[phase_2 >> 8]));
		if (shape == OSC_ZPULS_PK)
		{
			carrier >>= 1;
			carrier += 128;
		}
		uint8_t window = 0;
		if (phase_integral < 0x4000)
		{
			window = 255;
			} else if (phase_integral < 0x8000) {
			window = ~(phase_integral - 0x4000) >> 6;
		}
		if (shape == OSC_ZPULS_HP) {
			// write sample in sound-buffer
			buffer[i] = ~(S8U8MulShift8(carrier + 128, window) + 128);
			i++;
		}
		else
		{
			buffer[i] = ~U8U8MulShift8(carrier, window);
			i++;
		}
	}
	
	// update secondary phase
	Osc.data_secondary_phase[osc_nr] = phase_2;
		
}

//*************************************************************************
// OscRender: Quad saw or pwm (mit aliasing)
//*************************************************************************
void OscRender_PAD(uint8_t osc_nr)
{
	uint8_t block_size = 40;
	uint8_t shape;
	uint8_t parameter;
	uint8_t *buffer;	// pointer name
	uint8_t i = 0;
	uint8_t value;
	
	if (osc_nr == 0)
	{
		shape = Osc.shape[0];
		parameter = Osc.prm[0];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2a;
		}
		else {buffer = Voice.Buffer1a;}
	}
	else
	{
		shape = Osc.shape[1];
		parameter = Osc.prm[1];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2b;
		}
		else {buffer = Voice.Buffer1b;}
	}
	
	if (parameter > 127)
	{
		parameter = 127;
	}
	
	uint16_t phase_spread;
	phase_spread = ((uint32_t)((Osc.phase_increment[osc_nr] >> 8) * parameter) >> 13);
	phase_spread++;
	
	uint16_t phase_increment_x = Osc.phase_increment[osc_nr] >> 8;
	uint16_t increments[3];
	for (uint8_t i = 0; i < 3; ++i) {
		phase_increment_x += phase_spread;
		increments[i] = phase_increment_x;
	}
	
	if (shape == OSC_SAW_PAD) {
		//BEGIN_SAMPLE_LOOP
		while (block_size--)
		{
			// update Phase
			Osc.phase[osc_nr] += Osc.phase_increment[osc_nr];
			
			// Osc sync
			if (osc_nr == 0){Osc12.syncbuffer[i] = Osc.phase[osc_nr] < Osc.phase_increment[osc_nr] ? 1:0;}
			else{if (Osc.op_value == OP_SYNC){if (Osc12.syncbuffer[i]) Osc.phase[osc_nr] = 0;}}
			
			if (osc_nr == 0)
			{
				uint16_t phase_intergral = Osc.phase[osc_nr] >> 8;
				Osc1.data_qs_phase[0] += increments[0];
				Osc1.data_qs_phase[1] += increments[1];
				Osc1.data_qs_phase[2] += increments[2];
				value = (phase_intergral >> 10);
				value += (Osc1.data_qs_phase[0] >> 10);
				value += (Osc1.data_qs_phase[1] >> 10);
				value += (Osc1.data_qs_phase[2] >> 10);
			}
			else
			{
				uint16_t phase_intergral = Osc.phase[osc_nr] >> 8;
				Osc2.data_qs_phase[0] += increments[0];
				Osc2.data_qs_phase[1] += increments[1];
				Osc2.data_qs_phase[2] += increments[2];
				value = (phase_intergral >> 10);
				value += (Osc2.data_qs_phase[0] >> 10);
				value += (Osc2.data_qs_phase[1] >> 10);
				value += (Osc2.data_qs_phase[2] >> 10);
			}
			
			// write sample in sound-buffer
			buffer[i] = ~value;
			i++;
		}
	}
	else { //WAVEFORM_QUAD_PWM
		uint16_t pwm_phase = (uint16_t)(127 + parameter) << 8;
		while (block_size--)
		{
			// update Phase
			Osc.phase[osc_nr] += Osc.phase_increment[osc_nr];
			
			// Osc sync
			if (osc_nr == 0){Osc12.syncbuffer[i] = Osc.phase[osc_nr] < Osc.phase_increment[osc_nr] ? 1:0;}
			else{if (Osc.op_value == OP_SYNC){if (Osc12.syncbuffer[i]) Osc.phase[osc_nr] = 0;}}
			
			if (osc_nr == 0)
			{
				uint16_t phase_integral = Osc.phase[osc_nr] >> 8;
				Osc1.data_qs_phase[0] += increments[0];
				Osc1.data_qs_phase[1] += increments[1];
				Osc1.data_qs_phase[2] += increments[2];
				value = phase_integral < pwm_phase ? 0 : 63;
				value += Osc1.data_qs_phase[0] < pwm_phase ? 0 : 63;
				value += Osc1.data_qs_phase[1] < pwm_phase ? 0 : 63;
				value += Osc1.data_qs_phase[2] < pwm_phase ? 0 : 63;
			}
			else
			{
				uint16_t phase_integral = Osc.phase[osc_nr] >> 8;
				Osc2.data_qs_phase[0] += increments[0];
				Osc2.data_qs_phase[1] += increments[1];
				Osc2.data_qs_phase[2] += increments[2];
				value = phase_integral < pwm_phase ? 0 : 63;
				value += Osc2.data_qs_phase[0] < pwm_phase ? 0 : 63;
				value += Osc2.data_qs_phase[1] < pwm_phase ? 0 : 63;
				value += Osc2.data_qs_phase[2] < pwm_phase ? 0 : 63;
			}
			
			// write sample in sound-buffer
			buffer[i] = ~value;
			i++;
		}
	}
}


//*************************************************************************
// OscRender: FM and FMFB
//*************************************************************************
void OscRender_FM(uint8_t osc_nr)
{
	uint8_t block_size = 40;
	uint8_t shape;
	uint8_t parameter;
	uint8_t *buffer;	// pointer name
	uint8_t i = 0;
	uint8_t offset;
	uint16_t phase_2;
	uint8_t last_output;
		
	if (osc_nr == 0)
	{
		shape = Osc.shape[0];
		parameter = Osc.prm[0];
		offset = Osc.rng_value[0];
		phase_2 = Osc1.data_secondary_phase;
		last_output = Osc1.data_output_sample;
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2a;
		}
		else {buffer = Voice.Buffer1a;}
	}
	else
	{
		shape = Osc.shape[1];
		parameter = Osc.prm[1];
		offset = Osc.rng_value[1];
		phase_2 = Osc2.data_secondary_phase;
		last_output = Osc2.data_output_sample;
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2b;
		}
		else {buffer = Voice.Buffer1b;}
	}
	
	if (parameter > 127)
	{
		parameter = 127;
	}
	
	if (offset < 12) {
		offset = 0;
		} else if (offset > 36) {
		offset = 24;
		} else {
		offset = offset - 12;
	}
	
	uint16_t multiplier = pgm_read_word (&(fm_ratio[offset]));
	uint16_t increment = (int32_t)((Osc.phase_increment[osc_nr] >> 8) * multiplier) >> 8;
	parameter <<= 1;
	
	// The modulator will have a 16 bits phase increment, so we need to drop
	// the fractional part to 0 to ensure that the modulator frequency is
	// the correct multiple of the carrier frequency.
	
	Osc.phase[osc_nr] = Osc.phase[osc_nr] & 0xFFFF00; //phase_fractional = 0;
	
	 // FM or FMFB
	 uint8_t fb_phase_mod =
	 (shape == OSC_FM || parameter < 128) ? 0 : parameter - 128;
	
	// Beginn sample loop
	while (block_size--)
	{
		// update phases
		Osc.phase[osc_nr] += Osc.phase_increment[osc_nr];
		phase_2 += increment;
				
		// Osc sync
		if (osc_nr == 1){Osc12.syncbuffer[i] = Osc.phase[osc_nr] < Osc.phase_increment[osc_nr] ? 1:0;}
		else{if (Osc.op_value == OP_SYNC){if (Osc12.syncbuffer[i]) Osc.phase[osc_nr] = 0;}}
	
		uint8_t modulator = InterpolateSample(wav_res_sine,
		phase_2 + fb_phase_mod * last_output);
		uint16_t modulation = modulator * parameter;
		last_output = InterpolateSample(wav_res_sine,
		(Osc.phase[osc_nr] >> 8) + modulation);
		
		// write sample in current buffer
		buffer[i] = last_output;
		i++;
	}
	
	if (osc_nr == 0)
	{
		Osc1.data_secondary_phase = phase_2;
		Osc1.data_output_sample = last_output;
	}
	else
	{
		Osc2.data_secondary_phase = phase_2;
		Osc2.data_output_sample = last_output;
	}
}

//*************************************************************************
// OscRender: 8Bit Land
//*************************************************************************
void OscRender_8BIT(uint8_t osc_nr)
{
	uint8_t block_size = 40;
	//uint8_t shape;
	uint8_t parameter;
	uint8_t *buffer;	// pointer name
	uint8_t i = 0;
	
	if (osc_nr == 0)
	{
		//shape = Osc.shape[0];
		parameter = Osc.prm[0];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2a;
		}
		else {buffer = Voice.Buffer1a;}
	}
	else
	{
		//shape = Osc.shape[1];
		parameter = Osc.prm[1];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2b;
		}
		else {buffer = Voice.Buffer1b;}
	}
	
	if (parameter > 127)
	{
		parameter = 127;
	}
		
	// Beginn sample loop
	while (block_size--)
	{
		Osc.phase[osc_nr] += Osc.phase_increment[osc_nr];
		uint8_t x = parameter;
		
		// Osc sync
		if (osc_nr == 1){Osc12.syncbuffer[i] = Osc.phase[osc_nr] < Osc.phase_increment[osc_nr] ? 1:0;}
		else{if (Osc.op_value == OP_SYNC){if (Osc12.syncbuffer[i]) Osc.phase[osc_nr] = 0;}}

		
		// write sample in sound-buffer
		buffer[i] = (((Osc.phase[osc_nr] >> 16) ^ (x << 1)) & (~x)) + (x >> 1);
		i++;	
	}
}

//*************************************************************************
// OscRender: Crushed Sine
//*************************************************************************
void OscRender_CRUSH(uint8_t osc_nr)
{
	uint8_t block_size = 40;
	//uint8_t shape;
	uint8_t parameter;
	uint8_t *buffer;	// pointer name
	uint8_t i = 0;
	uint8_t decimate;
	uint8_t held_sample;
	
	if (osc_nr == 0)
	{
		//shape = Osc.shape[0];
		parameter = Osc.prm[0];
		decimate = Osc1.data_cr_decimate;
		held_sample = Osc1.data_cr_state;
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2a;
		}
		else {buffer = Voice.Buffer1a;}
	}
	else
	{
		//shape = Osc.shape[1];
		parameter = Osc.prm[1];
		decimate = Osc2.data_cr_decimate;
		held_sample = Osc2.data_cr_state;
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2b;
		}
		else {buffer = Voice.Buffer1b;}
	}
	
	if (parameter > 127)
	{
		parameter = 127;
	}
	
	// Beginn sample loop
	while (block_size--)
	{
		Osc.phase[osc_nr] += Osc.phase_increment[osc_nr];
		// Osc sync
		if (osc_nr == 1){Osc12.syncbuffer[i] = Osc.phase[osc_nr] < Osc.phase_increment[osc_nr] ? 1:0;}
		else{if (Osc.op_value == OP_SYNC){if (Osc12.syncbuffer[i]) Osc.phase[osc_nr] = 0;}}
		
		++decimate;
		if (parameter <= 63) {
			if (decimate >= parameter + 1) {
				decimate = 0;
				held_sample = InterpolateSample(wav_res_sine, Osc.phase[osc_nr] >> 8);
			}
			} else {
			if (decimate >= 128 - parameter) {
				decimate = 0;
				uint8_t tri = Osc.phase[osc_nr] >> 15;
				held_sample = (Osc.phase[osc_nr] >> 8) & 0x8000 ? ~tri : tri;
			}
		}
		
		// write sample in sound-buffer
		buffer[i] = held_sample;
		i++;
	}
	
	// update phase and bufwrpointer
	if (osc_nr == 0)
	{
		Osc1.data_cr_decimate = decimate;
		Osc1.data_cr_state = held_sample;
	}
	else
	{
		Osc2.data_cr_decimate = decimate;
		Osc2.data_cr_state = held_sample;
	}
}

//*************************************************************************
// OscRender: Noise
//*************************************************************************
void OscRender_Noise(uint8_t osc_nr)
{
	uint8_t block_size = 40;
	uint16_t parameter;
	uint8_t *buffer;	// pointer name
	uint8_t i = 0;
	
	if (osc_nr == 0)
	{
		parameter = Osc.prm[0];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2a;
		}
		else {buffer = Voice.Buffer1a;}
	}
	else
	{
		parameter = Osc.prm[1];
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2b;
		}
		else {buffer = Voice.Buffer1b;}
	}

	uint16_t rng_state = Noise.data_rng_state;
	
	if (rng_state == 0)
	{
		rng_state++;
	}

	

	if (parameter < 1)
	{
		parameter = 1;
	}
	
	// Beginn sample loop
	while (block_size--)
	{
		// random generator
		rng_state = (rng_state >> 1) ^ (-(rng_state & 1) & 0xb400);
		uint8_t noise_sample = rng_state >> 8;
		
		// Low-passed and high-passed white noise
		Noise.data_lp_noise_sample = U8Mix(Noise.data_lp_noise_sample,noise_sample,(parameter << 2)) >> 8;	// noise LP-Mixer

		uint8_t sample;
		if (parameter >= 64)
		{
			sample = noise_sample - Noise.data_lp_noise_sample - 128;
		}
		else
		{
			sample = Noise.data_lp_noise_sample ;
		}		
		// write sample in sound-buffer
		buffer[i] = sample;
		i++;
	}
	
	// send noise_sample to dac_out
	Noise.data_rng_state = rng_state;

	
	
}

//*************************************************************************
// OscRender: OFF
//*************************************************************************
void OscRender_OFF(uint8_t osc_nr)
{
	uint8_t block_size = 40;
	uint8_t *buffer;	// pointer name
	uint8_t i = 0;
	
	if (osc_nr == 0)
	{
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2a;
		}
		else {buffer = Voice.Buffer1a;}
	}
	else
	{
		if (Voice.buffer_nr == 0)
		{
			buffer = Voice.Buffer2b;
		}
		else {buffer = Voice.Buffer1b;}
	}
		
	// Beginn sample loop
	while (block_size--)
	{
		// write sample in current buffer
		buffer[i] = 128;
		i++;
	}

}


//*************************************************************************
// Phase increment table for oscillators
// Samplerate 40.000Hz for Waveforms
// Samplerate 44.100Hz for Samplefiles
// Phase increment  = (24Bit * Notenfrequenz / Samplefrequenz)
//*************************************************************************
__uint24 stepsize_tab1 [14] =
{
	// Waveform 40KHz
	3511479,		// Note C11
	3720282,
	3941502,
	4175863,
	4424186,
	4687262,
	4965981,
	5261273,
	5574125,
	5905580,
	6256744,
	6628789,
	7022958,
	7440564
};

__uint24 stepsize_tab2 [14] =
{
	// Sample 44.1KHz C10
	// stepsize_tab2 = stepsize_tab1/2.5544
	1374678,
	1456421,	
	1543025,
	1634773,
	1731986,
	1834976,
	1944089,
	2059690,
	2182166,
	2311925,
	2449399,
	2595047,
	2749357,
	2912842	
};



//*************************************************************************
// Frequency tune
//*************************************************************************
void frequency_tune(void)
{
	int8_t semi_temp;
	int16_t note;
	uint8_t note_number;
	uint8_t octave;
	__uint24 stepsize;
	__uint24 fPitch_1;
	__uint24 fPitch_2;
	__uint24 fPitchDiff;
	int32_t PitchDiff_temp;
	int16_t pitch_temp;
	__int24 mod_pitch_osc;
	__int24 mod_pitch_osc12;
	int16_t mod_pitch_osc_f;
	int16_t mod_pitch_osc12_f;
	//__int24 pitch_wheel;
	//int16_t wheel_temp;
	int16_t osc_fine;
	__int24 Osc1_stepsize;
	__int24 Osc2_stepsize;
	
	// Osc1 ---------------------------------------------------------------

	// calc osc1 rng (semiton)
	if (Osc1.mode_value != OSC_FM)
	{
		semi_temp = (int8_t)Osc.rng_value[0];
	}
	else semi_temp = 0;
	
	// enable osc key_tracking
	if (Osc.track_value[0] != 0)
	{
		note = midi_note + semi_temp;
	}
	else note = 69 + semi_temp;
	
	// calc note and octave
	note_number = (note % 12);
	octave = 13-(uint8_t)(note / 12);
	
	
		
	// calc osc1 base_frequenc --------------------------------------------
	if (Osc.wave_mode[0] == 0)
	{
		stepsize = stepsize_tab1[note_number];
		fPitch_1 = stepsize_tab1[note_number] >> octave;
		fPitch_2 = stepsize_tab1[note_number+1] >> octave;
	}
	else
	{
		stepsize = stepsize_tab2[note_number];
		fPitch_1 = stepsize_tab2[note_number] >> octave;
		fPitch_2 = stepsize_tab2[note_number+1] >> octave;
	}
	
	// calc OSC1 pitch ----------------------------------------------------
	fPitchDiff = fPitch_2-fPitch_1;
	PitchDiff_temp = (int32_t)fPitchDiff; // make int number
	
	// calc OSC1 pitch in ModMatrix ---------------------------------------
	pitch_temp = (int16_t)Osci.osc1_pitch - 8192;
	if (pitch_temp > 0)
	{
		mod_pitch_osc = (int32_t)(pitch_temp * PitchDiff_temp) >> 8;
	}
	else {mod_pitch_osc = (int32_t)(pitch_temp * PitchDiff_temp) >> 10;}
		
	// calc OSC1 fpitch in ModMatrix --------------------------------------
	pitch_temp = (int16_t)Osci.osc1_fpitch - 8192;
	if (pitch_temp > 0)
	{
		mod_pitch_osc_f = (int32_t)(pitch_temp * PitchDiff_temp) >> 13;
	}
	else {mod_pitch_osc_f = (int32_t)(pitch_temp * PitchDiff_temp) >> 15;}
	
	// calc OSC12 pitch in ModMatrix --------------------------------------
	pitch_temp = (int16_t)Osci.osc12_pitch - 8192;
	if (pitch_temp > 0)
	{
		mod_pitch_osc12 = (int32_t)(pitch_temp * PitchDiff_temp) >> 8;
	}
	else {mod_pitch_osc12 = (int32_t)(pitch_temp * PitchDiff_temp) >> 10;}
		
	// calc OSC12 fpitch in ModMatrix --------------------------------------
	pitch_temp = (int16_t)Osci.osc12_fpitch - 8192;
	if (pitch_temp > 0)
	{
		mod_pitch_osc12_f = (int32_t)(pitch_temp * PitchDiff_temp) >> 13;
	}
	else {mod_pitch_osc12_f = (int32_t)(pitch_temp * PitchDiff_temp) >> 15;}
	
	// calc OSC12  pitchbend ----------------------------------------------
	//wheel_temp = (int16_t)Osci.osc12_pitch - 8192;
	//pitch_wheel = (int32_t)(fPitchDiff * wheel_temp) >> 12;
	Osc1_stepsize = (stepsize >> octave);
	Osc1_stepsize += mod_pitch_osc_f + mod_pitch_osc + mod_pitch_osc12_f + mod_pitch_osc12;
	Osc.phase_increment[0] = Osc1_stepsize;
	

	// Osc2 ---------------------------------------------------------------

	// calc osc2 rng (semiton)
	if (Osc2.mode_value != OSC_FM)
	{
		semi_temp = (int8_t)Osc.rng_value[1];
	}
	else semi_temp = 0;
	
	// enable osc key_tracking
	if (Osc.track_value[1] != 0)
	{
		note = midi_note + semi_temp;
	}
	else note = 69 + semi_temp;
	
	// calc note and octave
	note_number = (note % 12);
	octave = 13-(uint8_t)(note / 12);
	
	// calc osc2 base_frequenc
	if (Osc.wave_mode[1] == 0)
	{
		stepsize = stepsize_tab1[note_number];
		fPitch_1 = stepsize_tab1[note_number] >> octave;
		fPitch_2 = stepsize_tab1[note_number+1] >> octave;
	}
	else
	{
		stepsize = stepsize_tab2[note_number];
		fPitch_1 = stepsize_tab2[note_number] >> octave;
		fPitch_2 = stepsize_tab2[note_number+1] >> octave;
	}
	
	// calc OSC2 pitch ----------------------------------------------------
	fPitchDiff = fPitch_2-fPitch_1;
	PitchDiff_temp = (int32_t)fPitchDiff; // make int number
	
	// calc osc2_fine -----------------------------------------------------
	osc_fine = (Osc.tune_value * (fPitchDiff >> 6));
	
	// calc OSC2 pitch in ModMatrix ---------------------------------------
	pitch_temp = (int16_t)Osci.osc2_pitch - 8192;
	if (pitch_temp > 0)
	{
		mod_pitch_osc = (int32_t)(pitch_temp * PitchDiff_temp) >> 8;
	}
	else {mod_pitch_osc = (int32_t)(pitch_temp * PitchDiff_temp) >> 10;}
		
	// calc OSC2 fpitch in ModMatrix --------------------------------------
	pitch_temp = (int16_t)Osci.osc2_fpitch - 8192;
	if (pitch_temp > 0)
	{
		mod_pitch_osc_f = (int32_t)(pitch_temp * PitchDiff_temp) >> 13;
	}
	else {mod_pitch_osc_f = (int32_t)(pitch_temp * PitchDiff_temp) >> 15;}
		
	// calc OSC12 pitch in ModMatrix --------------------------------------
	pitch_temp = (int16_t)Osci.osc12_pitch - 8192;
	if (pitch_temp > 0)
	{
		mod_pitch_osc12 = (int32_t)(pitch_temp * PitchDiff_temp) >> 8;
	}
	else {mod_pitch_osc12 = (int32_t)(pitch_temp * PitchDiff_temp) >> 10;}
	
	// calc OSC12 fpitch in ModMatrix --------------------------------------
	pitch_temp = (int16_t)Osci.osc12_fpitch - 8192;
	if (pitch_temp > 0)
	{
		mod_pitch_osc12_f = (int32_t)(pitch_temp * PitchDiff_temp) >> 13;
	}
	else {mod_pitch_osc12_f = (int32_t)(pitch_temp * PitchDiff_temp) >> 15;}
	
	// calc Osc2 ----------------------------------------------------------
	//wheel_temp = (int16_t)Osci.osc12_pitch - 8192;
	//pitch_wheel = (int32_t)(fPitchDiff * wheel_temp) >> 12;
	Osc2_stepsize = (stepsize >> octave);
	Osc2_stepsize += osc_fine + mod_pitch_osc_f + mod_pitch_osc + mod_pitch_osc12_f + mod_pitch_osc12_f;
	Osc.phase_increment[1] = Osc2_stepsize;	
}



