/*
 * sequencer.c
 *
 * Created: 29.06.2016 17:20:06
 *  Author: Rolf Degen
 */ 


#include "globals.h"
#include "display/displ_funktion.h"
#include "gui.h"
#include "sequencer.h"
#include "oscillators.h"


//*************************************************************************
// Sequencer / ARP Task
//*************************************************************************
void sequencer_task(void)
{
	//uint8_t note_;
	
	if (SEQ.seq_run != 0)
	{
		// ARP Mode -------------------------------------------------------
		if (SEQ.seq_mode == 1)
		{
			// Note length count ------------------------------------------
			if ( SEQ.seq_pulswidth_temp > 0)
			{
				SEQ.seq_pulswidth_temp--;
			}
			else
			{
				if (SEQ.seq_stop_flag == 1)
				{
					SEQ.seq_run = 0;
					SEQ.seq_stop_flag = 0;
				}
				
				// if note length count = 0 than clear gate and run release phase
				midi_gate_flag = 0;
				
				// reset arp_step if the last step
				if (SEQ.arp_note_size == 0 && NoteStack_ptr == 0)
				{
					SEQ.arp_step = 0;
				}
				
				// Seqencer Led off
				SEQ_LED_off;
			}
			
			// if HOLD mode dissapled -------------------------------------
			if (SEQ.seq_hold == 0 && SEQ.note_activity_flag == 1 && midi_gate_flag == 0)
			{
				if (SEQ.arp_note_size == 0) // If no note is pressed
				{
					SEQ.seq_speed_temp = 0; // sync clock
				}
				
				// save midi_notes into arp_stack
				for (SEQ.arp_note_size = 0; SEQ.arp_note_size < NoteStack_ptr; SEQ.arp_note_size++)
				{
					SEQ.arp_note_stack[SEQ.arp_note_size] = NoteStack_pool[SEQ.arp_note_size];
					SEQ.arp_note_stack_order[SEQ.arp_note_size] = NoteStack_pool[SEQ.arp_note_size];
				}
				
				// sort notes
				if (SEQ.seq_dir != 0)
				{
					SEQ.arp_sort_flag = 1;
				}
				else SEQ.arp_sort_flag = 0;
				
				SEQ.draw_note_flag = 1;
				SEQ.note_activity_flag = 0;
			}
			// if HOLD mode enabled ---------------------------------------
			if (SEQ.seq_hold == 1 && SEQ.seq_hold_flag == 0 && SEQ.note_activity_flag == 1 && midi_gate_flag == 0)
			{
				if (NoteStack_ptr == 1)
				{
					if (SEQ.arp_note_stack == 0) // If no note is pressed
					{
						SEQ.seq_speed_temp = 0; // sync clock
						SEQ.arp_octave_ = 0;
						
					}
					SEQ.arp_note_stack[0] = NoteStack_pool[0];
					SEQ.arp_step = 0;
					SEQ.arp_note_size = 1;
					SEQ.seq_hold_flag = 1;
					SEQ.draw_note_flag = 1;
					// sort notes
					if (SEQ.seq_dir != 0)
					{
						SEQ.arp_sort_flag = 1;
					}
					else SEQ.arp_sort_flag = 0;
				}
				else if (NoteStack_ptr == 2)
				{
					if (SEQ.arp_note_size < 16)
					{
						SEQ.arp_note_stack[SEQ.arp_note_size++] = NoteStack_pool[1];
					}
					SEQ.seq_hold_flag = 1;
					SEQ.draw_note_flag = 1;
					// sort notes
					if (SEQ.seq_dir != 0)
					{
						SEQ.arp_sort_flag = 1;
					}
					else SEQ.arp_sort_flag = 0;
				}
				else if (NoteStack_ptr > 2)
				{
					uint8_t count = NoteStack_ptr;
					for (SEQ.arp_note_size = 0; SEQ.arp_note_size < count; SEQ.arp_note_size++)
					{
						SEQ.arp_note_stack[SEQ.arp_note_size] = NoteStack_pool[SEQ.arp_note_size];
					}
					SEQ.seq_hold_flag = 1;
					SEQ.draw_note_flag = 1;
					// sort notes
					if (SEQ.seq_dir != 0)
					{
						SEQ.arp_sort_flag = 1;
					}
					else SEQ.arp_sort_flag = 0;
				}
			}
			
			// if new clock puls than sort notes in ArpStack and play it ------
			if (SEQ.seq_clock_flag == 1)
			{				
				if (!SEQ.arp_note_size)		// Back if no notes in arp_stack
				{
					SEQ_LED_on;
					return;
				}
				
				// Play direction -----------------------------------------
				Direction();
				
				// Calc note and play ---------------------------------
				SEQ.seq_pulswidth_temp = (65535 / (SEQ.seq_bpm * (SEQ.seq_div+1))) << 1;
				midi_sync_flag = 1;								// start envelopes
				SEQ.seq_clock_flag = 0;
				SEQ.draw_step_flag = 1;
				
				// set Seq_LED on -----------------------------------------
				SEQ_LED_on;		
			}
		}
	}
	
	// sequencer off mode -------------------------------------------------
	else
	{
		SEQ_LED_off;
	}
}
	
//*************************************************************************
// Sequencer and Arpeggio Direction
//*************************************************************************
void Direction(void)
{	
	uint8_t note_;
	
	if (SEQ.seq_dir < 5)		// directions without random
	{
		if (SEQ.arp_step_down == 0)					// switch-flag for direction
		{
Up:			if (!(SEQ.arp_step < SEQ.arp_note_size))
			{
				SEQ.arp_step = 0;
				SEQ.arp_octave_++;
				if (SEQ.arp_octave_ > SEQ.seq_oct)
				{
					SEQ.arp_octave_ = 0;
					if (SEQ.seq_dir > 1)			// Dir: all without Up & PL
					{
						if (SEQ.seq_dir == 3)		// Dir: <>
						{
							if (SEQ.arp_note_size > 2)
							{
								SEQ.arp_step = SEQ.arp_note_size - 2;
							}
							else if (SEQ.seq_oct > 0)
							{
								SEQ.arp_step = -1;
							}
							else SEQ.arp_step = 0;
						}
						else if (SEQ.seq_dir == 4)	// Dir: ><
						{
							SEQ.arp_step = SEQ.arp_note_size - 1;
						}
						SEQ.arp_step_down = 1;
						goto Down;
					}
				}
			}
			note_ = (SEQ.arp_note_stack[SEQ.arp_step]) + (12 * SEQ.arp_octave_);
			if (note_ > 0x7f)
			{
				note_ = 0x7f;
			}
			midi_note = note_;
			SEQ.played_step_nr = SEQ.arp_step;
			SEQ.arp_step++;
		}
		else
		{
Down:		if (!(SEQ.arp_step < SEQ.arp_note_size))
			{
				SEQ.arp_step = SEQ.arp_note_size - 1;
				SEQ.arp_octave_++;
			if (SEQ.arp_octave_ > SEQ.seq_oct)
				{
					SEQ.arp_octave_ = 0;
					if (SEQ.seq_dir != 2)
					{
						if (SEQ.seq_dir == 3)
						{
							SEQ.arp_step = 1;
						}
						else if (SEQ.seq_dir == 4)
						{
							SEQ.arp_step = 0;
						}
						SEQ.arp_step_down = 0;
						goto Up;
					}
				}
			}
			note_ = (SEQ.arp_note_stack[SEQ.arp_step]) + (12 * (SEQ.seq_oct - SEQ.arp_octave_));
			if (note_ > 0x7f)
			{
				note_ = 0x7f;
			}
			midi_note = note_;
			SEQ.played_step_nr = SEQ.arp_step;
			SEQ.arp_step--;
		}	
	}
	else  // Random
	{
		uint8_t step = Noise.sample % SEQ.arp_note_size;
		uint8_t octav = Noise.sample % (SEQ.seq_oct + 1);
		note_ = (SEQ.arp_note_stack[step]) + (12 * octav);
		if (note_ > 0x7f)
		{
			note_ = 0x7f;
		}
		midi_note = note_;
		SEQ.played_step_nr = step;
	}
}

//*************************************************************************
// Seq Load/Save query
//*************************************************************************
void draw_seq_load_save_query(void)
{
	print_string("SEQUENZ",27,174,2,color_white);
	
	// draw current wave, speed, xspeed, s&h
	// set encoder values
	set_encoder_val(SEQ.seq_nr,0,0,0);
	set_encoder_max(99,0,0,0);
	set_encoder_dyn(0,98,0,0);
	
	
	// draw load/save info
	fillrectangle(51,49,270,140,color_blue1);
	rectangle(54,52,84,212,color_white);
	fillrectangle(139,49,180,65,color_blue1);
	
	if (load_flag == 1)
	{
		print_string("Load",145,51,6,color_white);
		print_string("Press load or cancel",80,105,6,color_white);
	}
	else if (save_flag == 1)
	{
		print_string("Save",145,51,6,color_white);
		print_string("Press save or cancel",80,105,6,color_white);
	}
	print_string("Select Sequenz No:",75,75,6,color_white);
	fillrectangle(218,74,244,88,color_red);
	fillrectangle(217,75,245,87,color_red);
	draw_3digit_with(SEQ.seq_nr,220,75,6,color_white);	

}



//*************************************************************************
// draw seqencer value oct
//*************************************************************************
void draw_seq_value_oct(void)
{
	fillrectangle(268,140,285,148,color_black);
	fillrectangle(250,206,270,215,color_black);

	if (SEQ.seq_oct == 0)
	{
		print_string("-2",268,140,2,color_white);
		print_string("-2",252,206,2,color_white);
	}
	else if (SEQ.seq_oct == 1)
	{
		print_string("-1",268,140,2,color_white);
		print_string("-1",252,206,2,color_white);
	}
	else if (SEQ.seq_oct == 2)
	{
		print_string("0",268,140,2,color_white);
		print_string("0",255,206,2,color_white);
	}
	else if (SEQ.seq_oct == 3)
	{
		print_string("+1",268,140,2,color_white);
		print_string("+1",252,206,2,color_white);
	}
	else
	{
		print_string("+2",268,140,2,color_white);
		print_string("+2",252,206,2,color_white);
	}
}

//*************************************************************************
// Sort notes in arpeggio mode (Bubblsort)
//*************************************************************************
void sort_notes(uint8_t size )
{
	uint8_t i = 0, j = 0, temp = 0;
	
	for (i = 0; i < size; i++)
	{
		for (j = 0; j < size - 1; j++)
		{
			if (SEQ.arp_note_stack[j + 1] < SEQ.arp_note_stack[j])
			{
				temp = SEQ.arp_note_stack[j];
				SEQ.arp_note_stack[j] = SEQ.arp_note_stack[j + 1];
				SEQ.arp_note_stack[j + 1] = temp;
			}
		}
	}
	
	// sort finish
	SEQ.arp_sort_flag = 0;
}