/*
 * voice.c
 *
 * Created: 02.07.2016 09:24:50
 *  Author: Rolf Degen
 */ 

#include "globals.h"
#include "voice.h"
#include "oscillators.h"
#include "wavetable.h"
#include "envelope.h"
#include "op.h"


//*************************************************************************
// init voice buffer (clear buffers)
//*************************************************************************
void init_Voice(void)
{
	// clear interrupt
	cli();
	
	// clear Buffer1+2
	uint8_t buffer_size = 40;
	uint8_t i = 0;
	while(buffer_size--)
	{
		Voice.Buffer1a[i] = 128;
		Voice.Buffer1b[i] = 128;
		Voice.Buffer2a[i] = 128;
		Voice.Buffer2b[i] = 128;
		i++;
	}
	
	Voice.buffer_nr = 0;
	Voice.buf_read = 0;
	
	// set interrupt
	TCC0.INTCTRLA = 0b00000011;
	TCC1.INTCTRLA = 0b00000001;
	TCE1.INTCTRLA = 0b00000000;
	sei();
	
}

//*************************************************************************
// Interrupt calc voices and fill voices buffer
// interrupt is init in voices output routine
//*************************************************************************
void Voice_Buffer(void)
{
	// calc voise
	
	uint8_t buf_size = 40;
	uint8_t i = 0;
	uint8_t sample1;
	uint8_t sample2;
	uint8_t value;
	uint8_t noise;
	//uint8_t osc_gain;
	uint8_t *Buffer_a;
	uint8_t *Buffer_b;
	uint8_t *Buffer_c;
	
	if (Voice.buffer_nr == 0)
	{
		Buffer_a = Voice.Buffer2a;
		Buffer_b = Voice.Buffer2b;
		Buffer_c = Voice.Buffer3b;	// noise
	}
	else
	{
		Buffer_a = Voice.Buffer1a;
		Buffer_b = Voice.Buffer1b;
		Buffer_c = Voice.Buffer3a;	// noise
	}
	
	
	switch(Osc.op_value)
	{
		// Osc_OP: SUM ----------------------------------------------------
		case OP_SUM:
		while(buf_size--)
		{
			sample1 = Buffer_a[i];
			sample2 = Buffer_b[i];
			noise = Buffer_c[i];
			value = U8MixShift8(sample1,sample2,Osc.mix);
			value = U8MixShift8(value,noise,Osc.noise_value);
			Buffer_a[i] = value;
			Buffer_b[i] = value;
			i++;
		}
		break;
		
		// Osc_OP: 2CH ----------------------------------------------------
		case OP_2CH:
		while(buf_size--)
		{
			sample1 = Buffer_a[i];
			sample2 = Buffer_b[i];
			noise = Buffer_c[i];
			Buffer_a[i] = U8MixShift8(sample1,noise,Osc.noise_value);
			Buffer_b[i] = U8MixShift8(sample2,noise,Osc.noise_value);
			i++;
		}
		break;
		
		// Osc_OP: SYNC ---------------------------------------------------
		case OP_SYNC:
		while(buf_size--)
		{
			sample1 = Buffer_a[i];
			sample2 = Buffer_b[i];
			noise = Buffer_c[i];
			value = U8MixShift8(sample1,sample2,Osc.mix);
			value = U8MixShift8(value,noise,Osc.noise_value);
			Buffer_a[i] = value;
			Buffer_b[i] = value;
			i++;
		}
		break;
		
		// Osc_OP: Ring ---------------------------------------------------
		case OP_RING:
		while(buf_size--)
		{
			sample1 = Buffer_a[i];
			sample2 = Buffer_b[i];
			noise = Buffer_c[i];
			value = S8S8MulShift8(sample1+128,sample2+128);
			value += 128;
			value = U8MixShift8(sample1,value,Osc.mix);
			value = U8MixShift8(value,noise,Osc.noise_value);
			Buffer_a[i] = value;
			Buffer_b[i] = value;
			i++;
		}
		break;
		
		// Osc_OP: XOR ----------------------------------------------------
		case OP_XOR:
		while(buf_size--)
		{
			sample1 = Buffer_a[i];
			sample2 = Buffer_b[i];
			noise = Buffer_c[i];
			value = sample1 ^ sample2;
			value = value ^ Osc.mix;
			value = U8MixShift8(value,noise,Osc.noise_value);
			Buffer_a[i] = value;
			Buffer_b[i] = value;
			i++;
		}
		break;
				
		// Osc_OP: BITS -----------------------------------------------
		case OP_BITS:
		while(buf_size--)
		{
			sample1 = Buffer_a[i];
			sample2 = Buffer_b[i];
			noise = Buffer_c[i];
			uint8_t osc_gain = Osc.mix_value >> 4;
			osc_gain = 255 -((1 << osc_gain) - 1);
			value = (sample1 >> 1);
			value += (sample2 >> 1);
			value &= osc_gain;
			value = U8MixShift8(value,noise,Osc.noise_value);
			Buffer_a[i] = value;
			Buffer_b[i] = value;
			i++;
		}
		break;
		
		// Osc_OP: FUZZ -----------------------------------------------
		case OP_FUZZ:
		while(buf_size--)
		{
			sample1 = Buffer_a[i];
			sample2 = Buffer_b[i];
			noise = Buffer_c[i];
			uint8_t osc_gain = Osc.mix;
			value = (sample1 >> 1);
			value += (sample2 >> 1);
			value = U8MixShift8(value,pgm_read_byte (&(wav_res_distortion[value])), osc_gain);
			value = U8MixShift8(value,noise,Osc.noise_value);
			Buffer_a[i] = value;
			Buffer_b[i] = value;
			i++;
		}
		break;
		
	}
}


